# La table ASCII

Le processeur d'un ordinateur, c'est-à-dire le composant qui s'occupe de traiter les informations, exécuter les programmes, etc., ne comprend que les instructions formulées en **binaire**  ; il ne peut que lire une suite d’éléments pouvant être dans deux états : 0 ou 1.

Donc, tous les programmes informatiques que vous pourriez écrire, toutes les instructions SQL, tous les fichiers, sont _in fine_ traduits en code machine, donc une longue suite de 0 et de 1, pour pouvoir être lus ou exécutés. Chacun de ces 0 ou 1 est un **bit**. Ces bits sont regroupés par huit pour former un **octet**. 

L'ordinateur ne connaît donc pas la notion de caractères. Il ne sait pas ce qu'est un "A", ou un "è". Il a donc fallu créer une **table de conversion** pour traduire les caractères de la langue courante en une série de bits. La table **ASCII** était née !

La table ASCII est donc une table de conversion, qui permet de **traduire en code binaire 128 caractères**, dont 33 caractères de contrôle (séparateur de fichier, saut de page, ...) et 95 caractères affichables. Les caractères affichables sont **les 26 lettres de l'alphabet, en majuscules et en minuscules, les 10 chiffres arabes et toute une série de caractères spéciaux** courants (||#||, ||;||, ||)||, ||<||, ...).


[[question]]
| De combien de bits avons-nous besoin pour stocker ces 128 caractères ?

Chaque bit peut prendre deux valeurs différentes. Donc avec 1 bit, je peux représenter 2 caractères. Avec 2 bits, je représente 2^2^, donc 4 caractères. Et 128 = 2^7^. J'ai donc besoin de 7 bits pour pouvoir représenter toute ma table ASCII.
L'ordinateur travaillant sur des octets, on code donc la table ASCII sur un octet, avec le 8^e^ bit à 0. "00110100" représente par exemple le caractère "4", "01001101" le caractère "M" (le premier bit étant celui de droite, le 8^e^ celui de gauche).


# Jeux de caractères

On s'est ensuite rendu compte que ces 128 caractères n'étaient pas suffisants. En effet, ils ne comprennent pas les caractères accentués ("é", "à"). Ils ne comprennent pas non plus tous les caractères cyrilliques, japonais,etc.
On a alors commencé à utiliser le huitième bit. Ce qui permettait de représenter 128 caractères supplémentaires (donc 2^8^ = 256 en tout).
Mais avec 128 caractères supplémentaires, on n'a pas de quoi représenter tous les caractères qui n'existent pas dans la table ASCII. On a donc créé plusieurs **jeux de caractères** différents.
Exemples :


- l'ISO 8859-1 (ou latin1) : qui permet de couvrir une bonne partie des langues d'Europe occidentale en ajoutant les lettres accentuées aux caractères ASCII de base ("å", "é", "ô", "ñ", ...)
- l'ISO 8859-7 : qui permet de représenter les lettres grecques
- l'ISO 8859-11 : qui contient une bonne partie des glyphes de la langue thaï
- l'ISO 8859-15 : qui est une révision de l'ISO 8859-1, et qui remplace quelques caractères peu utilisés par d'autres, plus nécessaires, comme l'euro ("€")

Dès lors, lorsque l'on crée un fichier, un programme ou autre, il faut préciser quel jeu de caractère (ou encodage) est utilisé.


## L'UTF-8

Il restait un petit problème : comment faire des documents (ou autres) qui doivent utiliser plusieurs jeux de caractères différents ? On a donc créé un nouveau type d'encodage, permettant de représenter les caractères avec **deux octets** au lieu d'un. 16 bits donnent 2^16^ = 65536 possibilités. On peut donc, en utilisant deux octets, représenter plus de 65000 caractères. Cet encodage s'appelle l'**UTF-8**.
Le désavantage d'un tel encodage est évidemment le **coût en mémoire**, deux octets prenant plus de place qu'un. Cependant, tous les caractères ne sont pas codés sur deux octets. S'il s'agit d'un caractère de base de la table ASCII, le 8^e^ bit est à 0, ce qui indique qu'il n'est codé que sur un octet. Par contre, lorsque le 8^e^ bit est à 1, cela indique qu'on a affaire à un caractère spécial et qu'il est codé sur deux octets.
**Donc, en UTF-8, un "é" prendra plus de place qu'un "e"** (deux octets au lieu d'un).


[[question]]
| En quoi cela nous concerne-t-il ?

Je vous l'ai dit, il est nécessaire, pour tout fichier/programme/..., de préciser l'encodage utilisé. Je vous propose de choisir l'UTF-8.

A chaque connexion à MySQL, exécutez donc la commande suivante :


```sql
SET NAMES 'utf8';
```

Cette commande définit l'UTF-8 comme l'encodage dans lequel **les requêtes sont émises par le client**, ainsi que celui **utilisé par le serveur pour communiquer** avec le client.
Lors de la création de notre base de données dans un prochain chapitre, nous définirons également son encodage.


[[attention]]
| Il faut bien exécuter cette commande à chaque connexion !

Il est également possible d'ajouter une option lors de la connexion, pour éviter de devoir exécuter `SET NAMES`. En vous connectant de la manière suivante, l'encodage sera défini à UTF-8 directement à la connexion :


```console
mysql -u root -p --default-character-set=utf8
```



# Interclassement

L'interclassement est **un ensemble de règles** qui vient s'ajouter à l'encodage. Pour chaque jeu de caractères, plusieurs interclassements peuvent exister, qui définissent deux choses :


- **l'ordre des caractères** : par exemple, l'ordre alphabétique. Lorsque l'on trie les données avec MySQL, le tri sera basé sur l'interclassement.
- **les équivalences de caractères** : on peut par exemple définir que les majuscules sont équivalentes aux minuscules (interclassement **insensible à la casse**). De sorte que si l'on recherche les données qui contienne "a", on trouve également celles qui contiennent "A".


Je vous propose de garder l'interclassement par défaut (qui est insensible à la casse).

*[ASCII]: American Standard Code for Information Interchange