# Chaînes de type texte


## CHAR et VARCHAR

Pour stocker un texte relativement court (moins de 255 octets), vous pouvez utiliser les types `CHAR` et `VARCHAR`. Ces deux types s'utilisent avec un paramètre qui précise la taille que peut prendre votre texte (entre 1 et 255). La différence entre `CHAR` et `VARCHAR` est la manière dont ils sont stockés en mémoire. Un `CHAR(x)` stockera toujours _x_ octets, en remplissant si nécessaire le texte avec des espaces vides pour le compléter, tandis qu'un `VARCHAR(x)` stockera jusqu'à _x_ octets (entre 0 et _x_), et stockera en plus en mémoire la taille du texte stocké.

Si vous entrez un texte plus long que la taille maximale définie pour le champ, celui-ci sera tronqué.


[[information]]
| Je parle ici en **octets**, et non en caractères pour ce qui est de la taille des champs. C'est important : si la plupart des caractères sont stockés en mémoire sur un seul octet, ce n'est pas toujours le cas. Par exemple, lorsque l'on utilise l'encodage UTF-8, les caractères accentués (é, è, ...) sont codés sur deux octets.

Petit tableau explicatif, en prenant l'exemple d'un `CHAR` ou d'un `VARCHAR` de 5 octets maximum :




+-----------------+---------+---------------+----------+---------------+
|Texte            |CHAR(5)  |Mémoire requise|VARCHAR(5)|Mémoire requise|
+=================+=========+===============+==========+===============+
|''               |`'   '`  |5 octets       |`''`      |1 octet        |
+-----------------+---------+---------------+----------+---------------+
|'tex'            |`'tex  '`|5 octets       |`'tex'`   |4 octets       |
+-----------------+---------+---------------+----------+---------------+
|'texte'          |`'texte'`|5 octets       |`'texte'` |6 octets       |
+-----------------+---------+---------------+----------+---------------+
|'texte trop long'|`'texte'`|5 octets       |`'texte'` |6 octets       |
+-----------------+---------+---------------+----------+---------------+




Vous voyez donc que dans le cas où le texte fait la longueur maximale autorisée, un `CHAR(x)` prend moins de place en mémoire qu'un `VARCHAR(x)`. Préférez donc le `CHAR(x)` dans le cas où vous savez que vous aurez toujours _x_ octets (par exemple si vous stockez un code postal). Par contre, si la longueur de votre texte risque de varier d'une ligne à l'autre, définissez votre colonne comme un `VARCHAR(x)`.


## TEXT


[[question]]
| Et si je veux pouvoir stocker des textes de plus de 255 octets ?

Il suffit alors d'utiliser le type `TEXT`, ou un de ses dérivés `TINYTEXT`, `MEDIUMTEXT` ou `LONGTEXT`. La différence entre ceux-ci étant la place qu'ils permettent d'occuper en mémoire. Petit tableau habituel :




+------------+-----------------+--------------------------------+
|Type        |Longueur maximale|Mémoire occupée                 |
+============+=================+================================+
|`TINYTEXT`  |2^8 octets       |Longueur de la chaîne + 1 octet |
+------------+-----------------+--------------------------------+
|`TEXT`      |2^16 octets      |Longueur de la chaîne + 2 octets|
+------------+-----------------+--------------------------------+
|`MEDIUMTEXT`|2^24 octets      |Longueur de la chaîne + 3 octets|
+------------+-----------------+--------------------------------+
|`LONGTEXT`  |2^32 octets      |Longueur de la chaîne + 4 octets|
+------------+-----------------+--------------------------------+





# Chaînes de type binaire

Comme les chaînes de type texte que l'on vient de voir, une chaîne binaire n'est rien d'autre qu'une suite de caractères. 

Cependant, si les textes sont affectés par l'encodage et l'interclassement, ce n'est pas le cas des chaînes binaires. Une chaîne binaire n'est rien d'autre qu'une suite d'octets. Aucune interprétation n'est faite sur ces octets. Ceci a deux conséquences principales.


- Une chaîne binaire traite directement l'octet, et pas le caractère que l'octet représente. Donc par exemple, une recherche sur une chaîne binaire sera toujours sensible à la casse, puisque "A" (code binaire : 01000001) sera toujours différent de "a" (code binaire : 01100001).
- Tous les caractères sont utilisables, y compris les fameux caractères de contrôle non-affichables définis dans la table ASCII.


Par conséquent, les types binaires sont parfaits pour stocker des données "brutes" comme des images par exemple, tandis que les chaînes de texte sont parfaites pour stocker...du texte !  :D 

Les types binaires sont définis de la même façon que les types de chaînes de texte. `VARBINARY(x)` et `BINARY(x)` permettent de stocker des chaînes binaires de _x_ caractères maximum (avec une gestion de la mémoire identique à `VARCHAR(x)` et `CHAR(x)`). Pour les chaînes plus longues, il existe les types `TINYBLOB`, `BLOB`, `MEDIUMBLOB` et `LONGBLOB`, également avec les mêmes limites de stockage que les types `TEXT`.


# SET et ENUM


## ENUM

Une colonne de type `ENUM` est une colonne pour laquelle on définit un certain nombre de valeurs autorisées, de type "chaîne de caractère". Par exemple, si l'on définit une colonne _espece_ (pour une espèce animale) de la manière suivante : 

```sql
espece ENUM('chat', 'chien', 'tortue')
```

La colonne _espece_  pourra alors contenir les chaînes  "chat", "chien" ou "tortue", mais pas les chaînes "lapin" ou "cheval". 

En plus de "chat", "chien" et "tortue", la colonne _espece_ pourrait prendre deux autres valeurs :


- si vous essayez d'introduire une chaîne non-autorisée, MySQL stockera une chaîne vide `''` dans le champ ;
- si vous autorisez le champ à ne pas contenir de valeur (vous verrez comment faire ça dans le chapitre sur la création des tables), le champ contiendra `NULL`, qui correspond à "pas de valeur" en SQL (et dans beaucoup de langages informatiques).


Pour remplir un champ de type `ENUM`, deux possibilités s'offrent à vous :


- soit remplir directement avec la valeur choisie ("chat", "chien" ou "tortue" dans notre exemple) ;
- soit utiliser l'index de la valeur, c'est-à-dire le nombre associé par MySQL à la valeur. Ce nombre est compris entre 1 et le nombre de valeurs définies. L'index est attribué selon l'ordre dans lequel les valeurs ont été données lors de la création du champ. De plus, la chaîne vide (stockée en cas de valeur non-autorisée) correspond à l'index 0. Le tableau suivant reprend les valeurs d'index pour notre exemple précédent : le champ _espece_.





+--------+------+
|Valeur  |Index |
+========+======+
|`NULL`  |`NULL`|
+--------+------+
|`''`    |0     |
+--------+------+
|'chat'  |1     |
+--------+------+
|'chien' |2     |
+--------+------+
|'tortue'|3     |
+--------+------+




Afin que tout soit bien clair : si vous voulez stocker "chien" dans votre champ, vous pouvez donc y insérer "chien" ou insérer 2 (sans guillemets, il s'agit d'un nombre, pas d'un caractère).


[[information]]
| Un `ENUM` peut avoir maximum 65535 valeurs possibles


## SET

`SET` est fort semblable à `ENUM`. Une colonne `SET` est en effet une colonne qui permet de stocker une chaîne de caractères dont les valeurs possibles sont prédéfinies par l'utilisateur. La différence avec `ENUM`, c'est qu'on peut stocker dans la colonne entre 0 et _x_ valeur(s), _x_ étant le nombre de valeurs autorisées.

Donc, si l'on définit une colonne de type `SET` de la manière suivante :


```sql
espece SET('chat', 'chien', 'tortue')
```

On pourra stocker dans cette colonne :


- `''` (chaîne vide ) ;
- `'chat'` ;
- `'chat,tortue'` ;
- `'chat,chien,tortue'` ;
- `'chien,tortue'` ;
- ...


Vous remarquerez que lorsqu'on stocke plusieurs valeurs, il faut les séparer par une virgule, sans espace et entourer la totalité des valeurs par des guillemets (non pas chaque valeur séparément). Par conséquent, les valeurs autorisées d'une colonne `SET` ne peuvent pas contenir de virgule elles-mêmes.


[[information]]
| On ne peut pas stocker la même valeur plusieurs fois dans un SET. "chien,chien" par exemple, n'est donc pas valable.

Les colonnes `SET` utilisent également un système d'index, quoiqu'un peu plus complexe que pour le type `ENUM`. `SET` utilise en effet un système d'index binaire. Concrètement, la présence/absence des valeurs autorisées va être enregistrée sous forme de bits, mis à 1 si la valeur correspondante est présente, à 0 si la valeur correspondante est absente.
Si l'on reprend notre exemple, on a donc :


```sql
espece SET('chat', 'chien', 'tortue')
```

Trois valeurs sont autorisées. Il nous faut donc trois bits pour savoir quelles valeurs sont stockées dans le champ. Le premier, à droite, correspondra à "chat", le second (au milieu) à "chien" et le dernier (à gauche) à "tortue".


- 000 signifie qu'aucune valeur n'est présente.
- 001 signifie que 'chat' est présent.
- 100 signifie que 'tortue' est présent.
- 110 signifie que 'chien' et 'tortue' sont présents.
- ...


Par ailleurs, ces suites de bits représentent des nombres en binaire convertibles en décimal. Ainsi 000 en binaire correspond à 0 en nombre décimal, 001 correspond à 1, 010 correspond à 2, 011 à 3...

Puisque j'aime bien les tableaux, je vous en fais un, ce sera peut-être plus clair.




+--------+-------+-------+
|Valeur  |Binaire|Décimal|
+========+=======+=======+
|'chat'  |001    |1      |
+--------+-------+-------+
|'chien' |010    |2      |
+--------+-------+-------+
|'tortue'|100    |4      |
+--------+-------+-------+




Pour stocker 'chat' et 'tortue' dans un champ, on peut donc utiliser 'chat,tortue' ou 101 (addition des nombres binaires correspondants) ou 5 (addition des nombres décimaux correspondants).

Notez que cette utilisation des binaires a pour conséquence que l'ordre dans lequel vous rentrez vos valeurs n'a pas d'importance. Que vous écriviez 'chat,tortue' ou 'tortue,chat' ne fait aucune différence. Lorsque vous récupérerez votre champ, vous aurez 'chat,tortue' (dans le même ordre que lors de la définition du champ).


[[information]]
| Un champ de type SET peut avoir au plus 64 valeurs définies


## Avertissement

`SET` et `ENUM` sont des types **propres à MySQL**. Ils sont donc à utiliser avec une grande prudence !


[[question]]
| Pourquoi avoir inventé ces types propres à MySQL ?

La plupart des SGBD implémentent ce qu'on appelle des contraintes d'assertions, qui permettent de définir les valeurs que peuvent prendre une colonne (par exemple, on pourrait définir une contrainte pour une colonne contenant un âge, devant être compris entre 0 et 130). 

MySQL n'implémente pas ce type de contrainte et a par conséquent créé deux types de données spécifiques (`SET` et `ENUM`), pour pallier en partie ce manque.


[[question]]
| Dans quelles situations faut-il utiliser `ENUM` ou `SET` ?

La meilleure réponse à cette question est : **jamais !** Je déconseille fortement l'utilisation des `SET` et des `ENUM`. Je vous ai présenté ces deux types par souci d'exhaustivité, mais il faut toujours éviter autant que possible les fonctionnalités propres à un seul SGBD. Ceci afin d'éviter les problèmes si un jour vous voulez en utiliser un autre.

Mais ce n'est pas la seule raison. Imaginez que vous vouliez utiliser un `ENUM` ou un `SET` pour un système de catégories. Vous avez donc des éléments qui peuvent appartenir à une catégorie (dans ce cas, vous utilisez une colonne `ENUM` pour la catégorie) ou appartenir à plusieurs catégories (et vous utilisez `SET`). 


```sql
categorie ENUM("Soupes", "Viandes", "Tarte", "Dessert")
categorie SET("Soupes", "Viandes", "Tarte", "Dessert")
```

Tout se passe plutôt bien tant que vos éléments appartiennent aux catégories que vous avez définies au départ. Et puis tout à coup, vous vous retrouvez avec un élément qui ne correspond à aucune de vos catégories, mais qui devrait plutôt se trouver dans la catégorie "Entrées". Avec `SET` ou `ENUM`, il vous faut modifier la colonne _categorie_ pour ajouter "Entrées" aux valeurs possibles. Or, une des règles de base à respecter lorsque l'on conçoit une base de données, est que **la structure de la base (donc les tables, les colonnes) ne doit pas changer lorsque l'on ajoute des données**. Par conséquent, tout ce qui est susceptible de changer doit être une donnée, et non faire partie de la structure de la base. 

Il existe deux solutions pour éviter les `ENUM`, et une solution pour éviter les `SET`.

**Pour éviter `ENUM`**


- Vous pouvez faire de la colonne _categorie_ une simple colonne `VARCHAR(100)`. Le désavantage est que vous ne pouvez pas limiter les valeurs entrées dans cette colonne. Cette vérification pourra éventuellement se faire à un autre niveau (par exemple au niveau du PHP si vous faites un site web avec PHP et MySQL).
- Vous pouvez aussi ajouter une table _Categorie_ qui reprendra toutes les catégories possibles. Dans la table des éléments, il suffira alors de stocker une référence vers la catégorie de l'élément.


**Pour éviter `SET`**
La solution consiste en la création de deux tables : une table _Categorie_, qui reprend les catégories possibles, et une table qui lie les éléments aux catégories auxquels ils appartiennent.

*[ASCII]: American Standard Code for Information Interchange
*[SGBD]: Système de Gestion de Bases de Données