Pour les données temporelles, MySQL dispose de cinq types qui permettent, lorsqu'ils sont bien utilisés, de faire énormément de choses.

Avant d'entrer dans le vif du sujet, une petite remarque importante : lorsque vous stockez une date dans MySQL, certaines vérifications sont faites sur la validité de la date entrée. Cependant, ce sont des vérifications de base : le jour doit être compris entre 1 et 31 et le mois entre 1 et 12. Il vous est tout à fait possible d'entrer une date telle que le 31 février 2011. Soyez donc prudents avec les dates que vous entrez et récupérez.

Les cinq types temporels de MySQL sont `DATE`, `DATETIME`, `TIME`, `TIMESTAMP` et `YEAR`. 


# DATE, TIME et DATETIME

Comme son nom l'indique, `DATE` sert à stocker une date. `TIME` sert quant à lui à stocker une heure, et `DATETIME` stocke...une date ET une heure !  :D 


## DATE

Pour entrer une date, l'ordre des données est la seule contrainte. Il faut donner d'abord l'année (deux ou quatre chiffres), ensuite le mois (deux chiffres) et pour finir, le jour (deux chiffres), sous forme de nombre ou de chaîne de caractères. S'il s'agit d'une chaîne de caractères, n'importe quelle ponctuation peut être utilisée pour délimiter les parties (ou aucune). Voici quelques exemples d'expressions correctes (A représente les années, M les mois et J les jours) :


- `'AAAA-MM-JJ'` (c'est sous ce format-ci qu'une `DATE` est stockée dans MySQL)
- `'AAMMJJ'`
- `'AAAA/MM/JJ'`
- `'AA+MM+JJ'`
- `'AAAA%MM%JJ'`
- `AAAAMMJJ` (nombre)
- `AAMMJJ` (nombre)


L'année peut donc être donnée avec deux ou quatre chiffres. Dans ce cas, le siècle n'est pas précisé, et c'est MySQL qui va décider de ce qu'il utilisera, selon ces critères :


- si l'année donnée est entre 00 et 69, on utilisera le 21^e^ siècle, on ira donc de 2000 à 2069 ;
- par contre, si l'année est comprise entre 70 et 99, on utilisera le 20^e^ siècle, donc entre 1970 et 1999.



[[information]]
| MySQL supporte des `DATE` allant de '1001-01-01' à '9999-12-31'


## DATETIME

Très proche de `DATE`, ce type permet de stocker une heure, en plus d'une date. Pour entrer un `DATETIME`, c'est le même principe que pour `DATE` : pour la date, année-mois-jour, et pour l'heure, il faut donner d'abord l'heure, ensuite les minutes, puis les secondes. Si on utilise une chaîne de caractères, il faut séparer la date et l'heure par une espace. Quelques exemples corrects (H représente les heures, M les minutes et S les secondes) : 


- `'AAAA-MM-JJ HH:MM:SS'` (c'est sous ce format-ci qu'un `DATETIME` est stocké dans MySQL)
- `'AA*MM*JJ HH+MM+SS'`
- `AAAAMMJJHHMMSS` (nombre)



[[information]]
| MySQL supporte des `DATETIME` allant de '1001-01-01 00:00:00' à '9999-12-31 23:59:59'


## TIME

Le type `TIME` est un peu plus compliqué, puisqu'il permet non seulement de stocker une heure précise, mais aussi un intervalle de temps. On n'est donc pas limité à 24 heures, et il est même possible de stocker un nombre de jours ou un intervalle négatif. Comme dans `DATETIME`, il faut d'abord donner l'heure, puis les minutes, puis les secondes, chaque partie pouvant être séparée des autres par le caractère ||:||. Dans le cas où l'on précise également un nombre de jours, alors les jours sont en premier et séparés du reste par une espace. Exemples :
 

- `'HH:MM:SS'`
- `'HHH:MM:SS'`
- `'MM:SS'`
- `'J HH:MM:SS'`
- `'HHMMSS'`
- `HHMMSS` (nombre)



[[information]]
| MySQL supporte des `TIME` allant de '-838:59:59' à '838:59:59'


# YEAR

Si vous n'avez besoin de retenir que l'année, `YEAR` est un type intéressant car il ne prend qu'un seul octet en mémoire. Cependant, un octet ne pouvant contenir que 256 valeurs différentes, `YEAR` est fortement limité : on ne peut y stocker que des années entre 1901 et 2155. Ceci dit, ça devrait suffire à la majorité d'entre vous pour au moins les cent prochaines années.

On peut entrer une donnée de type `YEAR` sous forme de chaîne de caractères ou d'entiers, avec 2 ou 4 chiffres. Si l'on ne précise que deux chiffres, le siècle est ajouté par MySQL selon les mêmes critères que pour `DATE` et `DATETIME`, **à une exception près** : si l'on entre 00 (un entier donc), il sera interprété comme la valeur par défaut de `YEAR` 0000. Par contre, si l'on entre `'00'` (une chaîne de caractères), elle sera bien interprétée comme l'année 2000. 
Plus de précisions sur les valeurs par défaut des types temporels dans quelques instants !


# TIMESTAMP

Par définition, le timestamp d'une date est le nombre de secondes écoulées depuis le 1er janvier 1970, 0h0min0s (TUC) et la date en question. 
Les timestamps étant stockés sur 4 octets, il existe une limite supérieure : le 19 janvier 2038 à 3h14min7s. Par conséquent, vérifiez bien que vous êtes dans l'intervalle de validité avant d'utiliser un timestamp.

Le type `TIMESTAMP` de MySQL est cependant un peu particulier. Prenons par exemple le 4 octobre 2011, à 21h05min51s.
Entre cette date et le 1er janvier 1970, 0h0min0s, il s'est écoulé exactement 1317755151 secondes. Le nombre 1317755151 est donc, par définition, le timestamp de cette date du 4 octobre 2011, 21h05min51s.
Pourtant, pour stocker cette date dans un `TIMESTAMP` SQL, ce n'est pas 1317755151 qu'on utilisera, mais 20111004210551. C'est-à-dire l'équivalent, au format numérique, du `DATETIME '2011-10-04 21:05:51'`. 
Le `TIMESTAMP` SQL n'a donc de timestamp que le nom. Il ne sert pas à stocker un nombre de secondes, mais bien une date sous format numérique AAAAMMJJHHMMSS (alors qu'un `DATETIME` est donc stocké sous forme de chaîne de caractères).

Il n'est donc pas possible de stocker un "vrai" timestamp dans une colonne de type `TIMESTAMP`. C'est évidemment contre-intuitif, et source d'erreur.
Notez que malgré cela, le `TIMESTAMP` SQL a les même limites qu'un vrai timestamp : il n'acceptera que des date entre le 1e janvier 1970 à 00h00min00s et le 19 janvier 2038 à 3h14min7s.


# La date par défaut

Lorsque MySQL rencontre une date/heure incorrecte, ou qui n'est pas dans l'intervalle de validité du champ, la valeur par défaut est stockée à la place. Il s'agit de la valeur "zéro" du type. On peut se référer à cette valeur par défaut en utilisant `'0'` (caractère), 0 (nombre) ou la représentation du "zéro" correspondant au type de la colonne (voir tableau ci-dessous).




+-----------+------------------------+
|Type       |Date par défaut ("zéro")|
+===========+========================+
|`DATE`     |`'0000-00-00'`          |
+-----------+------------------------+
|`DATETIME` |`'0000-00-00 00:00:00'` |
+-----------+------------------------+
|`TIME`     |`'00:00:00'`            |
+-----------+------------------------+
|`YEAR`     |`0000`                  |
+-----------+------------------------+
|`TIMESTAMP`|`00000000000000`        |
+-----------+------------------------+




Une exception toutefois, si vous insérez un `TIME` qui dépasse l'intervalle de validité, MySQL ne le remplacera pas par le "zéro", mais par la plus proche valeur appartenant à l'intervalle de validité (-838:59:59 ou 838:59:59).

*[TUC]: Temps Universel Coordonné