# Création

Nous allons donc créer notre base de données, que nous appellerons _elevage_. Rappelez-vous, lors de la création de votre utilisateur MySQL, vous lui avez donné tous les droits sur la base _elevage_, qui n'existait pas encore. Si vous choisissez un autre nom de base, vous n'aurez aucun droit dessus.

La commande SQL pour créer une base de données est la suivante :


```sql
CREATE DATABASE nom_base;
```

Avouez que je ne vous surmène pas le cerveau pour commencer…

Cependant, attendez avant de créer votre base de données _elevage_. Je vous rappelle qu'il faut également définir l'encodage utilisé (l'UTF-8 dans notre cas). Voici donc la commande complète à taper pour créer votre base :


```sql
CREATE DATABASE elevage CHARACTER SET 'utf8';
```

Lorsque nous créerons nos tables dans la base de données, automatiquement elles seront encodées également en UTF-8.


# Suppression

Si vous avez envie d'essayer cette commande, faites-le maintenant, tant qu'il n'y a rien dans votre base de données. Soyez très prudents, car vous effacez tous les fichiers créés par MySQL qui servent à stocker les informations de votre base.


```sql
DROP DATABASE elevage;
```

Si vous essayez cette commande alors que la base de données _elevage_ n'existe pas, MySQL vous affichera une erreur :


```console
mysql> DROP DATABASE elevage;
ERROR 1008 (HY000) : Can't drop database 'elevage'; database doesn't exist
mysql>
```

Pour éviter ce message d'erreur, si vous n'êtes pas sûrs que la base de données existe, vous pouvez utiliser l'option `IF EXISTS`, de la manière suivante :


```sql
DROP DATABASE IF EXISTS elevage;
```

Si la base de données existe, vous devriez alors avoir un message du type :


```console
Query OK, 0 rows affected (0.00 sec)
```

Si elle n'existe pas, vous aurez :


```console
Query OK, 0 rows affected, 1 warning (0.00 sec)
```

Pour afficher les warnings de MySQL, il faut utiliser la commande

```sql
SHOW WARNINGS;
```

Cette commande affiche un tableau :




+-----+----+-----------------------------------------------------+
|Level|Code|Message                                              |
+=====+====+=====================================================+
|Note |1008|Can't drop database 'elevage'; database doesn't exist|
+-----+----+-----------------------------------------------------+






# Utilisation d'une base de données

Vous avez maintenant créé une base de données (si vous l'avez effacée avec `DROP DATABASE`, recréez-la). Mais pour pouvoir agir sur cette base, vous devez encore avertir MySQL que c'est bien sûr cette base-là que vous voulez travailler. Une fois de plus, la commande est très simple :


```sql
USE elevage
```

C'est tout ! À partir de maintenant, toutes les actions effectuées le seront sur la base de données _elevage_ (création et modification de tables par exemple).

Notez que vous pouvez spécifier la base de données sur laquelle vous allez travailler lors de la connexion à MySQL. Il suffit d'ajouter le nom de la base à la fin de la commande de connexion :


```console
mysql -u sdz -p elevage
```