Avant de voir la syntaxe permettant de créer une table, résumons un peu. Nous voulons donc créer une table _Animal_ avec six colonnes telles que décrites dans le tableau suivant.




+-----------------+----------------+-------------+-------+------------------------------------------+
|Caractéristique  |Nom du champ    |Type         |`NULL`?|Divers                                    |
+=================+================+=============+=======+==========================================+
|Numéro d'identité|_id_            |`SMALLINT`   |Non    |Clé primaire + auto-incrément + `UNSIGNED`|
+-----------------+----------------+-------------+-------+------------------------------------------+
|Espèce           |_espece_        |`VARCHAR(40)`|Non    |-                                         |
+-----------------+----------------+-------------+-------+------------------------------------------+
|Sexe             |_sexe_          |`CHAR(1)`    |Oui    |-                                         |
+-----------------+----------------+-------------+-------+------------------------------------------+
|Date de naissance|_date_naissance_|`DATETIME`   |Non    |-                                         |
+-----------------+----------------+-------------+-------+------------------------------------------+
|Commentaires     |_commentaires_  |`TEXT`       |Oui    |-                                         |
+-----------------+----------------+-------------+-------+------------------------------------------+
|Nom              |_nom_           |`VARCHAR(30)`|Oui    |-                                         |
+-----------------+----------------+-------------+-------+------------------------------------------+





# Syntaxe

Par souci de clarté, je vais diviser l'explication de la syntaxe de `CREATE TABLE` en deux. La première partie vous donne la syntaxe globale de la commande, et la deuxième partie s'attarde sur la description des colonnes créées dans la table.


## Création de la table


```sql
CREATE TABLE [IF NOT EXISTS] Nom_table (
    colonne1 description_colonne1,
    [colonne2 description_colonne2,
    colonne3 description_colonne3,
    ...,]
    [PRIMARY KEY (colonne_clé_primaire)]
)
[ENGINE=moteur];
```

Le `IF NOT EXISTS` est facultatif (d'où l'utilisation de crochets [ ]), et a le même rôle que dans la commande `CREATE DATABASE` : si une table de ce nom existe déjà dans la base de données, la requête renverra un warning plutôt qu'une erreur si `IF NOT EXISTS` est spécifié.
Ce n'est pas non plus une erreur de ne pas préciser la clé primaire directement à la création de la table. Il est tout à fait possible de l'ajouter par la suite. Nous verrons comment un peu plus tard.


## Définition des colonnes

Pour définir une colonne, il faut donc donner son nom en premier, puis sa description. La description est constituée au minimum du type de la colonne. Exemple :


```sql
nom VARCHAR(30),
sexe CHAR(1)
```

C'est aussi dans la description que l'on précise si la colonne peut contenir `NULL` ou pas (par défaut, `NULL` est autorisé). Exemple :


```sql
espece VARCHAR(40) NOT NULL,
date_naissance DATETIME NOT NULL
```

L'auto-incrémentation se définit également à cet endroit. Notez qu'il est également possible de définir une colonne comme étant la clé primaire dans sa description. Il ne faut alors plus l'indiquer après la définition de toutes les colonnes. Je vous conseille néanmoins de ne pas l'indiquer à cet endroit, nous verrons plus tard pourquoi.


```sql
id SMALLINT UNSIGNED NOT NULL AUTO_INCREMENT [PRIMARY KEY]
```

Enfin, on peut donner une valeur par défaut au champ. Si lorsque l'on insère une ligne, aucune valeur n'est précisée pour le champ, c'est la valeur par défaut qui sera utilisée. Notez que si une colonne est autorisée à contenir `NULL` et qu'on ne précise pas de valeur par défaut, alors `NULL` est implicitement considéré comme valeur par défaut.

**Exemple** :


```sql
espece VARCHAR(40) NOT NULL DEFAULT 'chien'
```


[[attention]]
| Une valeur par défaut DOIT être une constante. Ce ne peut pas être une fonction (comme par exemple la fonction `NOW()` qui renvoie la date et l'heure courante).


# Application : création de _Animal_

Si l'on met tout cela ensemble pour créer la table _Animal_ (je rappelle que nous utiliserons le moteur InnoDB), on a donc :


```sql
CREATE TABLE Animal (
    id SMALLINT UNSIGNED NOT NULL AUTO_INCREMENT,
    espece VARCHAR(40) NOT NULL,
    sexe CHAR(1),
    date_naissance DATETIME NOT NULL,
    nom VARCHAR(30),
    commentaires TEXT,
    PRIMARY KEY (id)
)
ENGINE=INNODB;
```


[[information]]
| Je n'ai pas gardé la valeur par défaut pour le champ _espece_, car je trouve que ça n'a pas beaucoup de sens dans ce contexte. C'était juste un exemple pour vous montrer la syntaxe.


# Vérifications

Au cas où vous ne me croiriez pas (et aussi un peu parce que cela pourrait vous être utile un jour), voici deux commandes vous permettant de vérifier que vous avez bien créé une jolie table _Animal_ avec les six colonnes que vous vouliez.


```sql
SHOW TABLES;      -- liste les tables de la base de données

DESCRIBE Animal;  -- liste les colonnes de la table avec leurs caractéristiques
```