Deux possibilités s'offrent à nous lorsque l'on veut insérer une ligne dans une table : soit donner une valeur pour chaque colonne de la ligne, soit ne donner les valeurs que de certaines colonnes, auquel cas il faut bien sûr préciser de quelles colonnes il s'agit.


# Insertion sans préciser les colonnes

Je rappelle pour les distraits que notre table _Animal_ est composée de six colonnes : _id_, _espece_, _sexe_, _date_naissance_, _nom_ et _commentaires_.

Voici donc la syntaxe à utiliser pour insérer une ligne dans _Animal_, sans renseigner les colonnes pour lesquelles on donne une valeur (implicitement, MySQL considère que l'on donne une valeur pour chaque colonne de la table).


```sql
INSERT INTO Animal 
VALUES (1, 'chien', 'M', '2010-04-05 13:43:00', 'Rox', 'Mordille beaucoup');
```

**Deuxième exemple** : cette fois-ci, on ne connaît pas le sexe et on n'a aucun commentaire à faire sur la bestiole :


```sql
INSERT INTO Animal 
VALUES (2, 'chat', NULL, '2010-03-24 02:23:00', 'Roucky', NULL);
```

**Troisième et dernier exemple** : on donne `NULL` comme valeur d'_id_, ce qui en principe est impossible puisque _id_ est défini comme `NOT NULL` et comme clé primaire. Cependant, l'auto-incrémentation fait que MySQL va calculer tout seul comme un grand quel _id_ il faut donner à la ligne (ici : 3).


```sql
INSERT INTO Animal 
VALUES (NULL , 'chat', 'F', '2010-09-13 15:02:00', 'Schtroumpfette', NULL);
```

Vous avez maintenant trois animaux dans votre table :




+--+------+----+-------------------+--------------+-----------------+
|Id|Espèce|Sexe|Date de naissance  |Nom           |Commentaires     |
+==+======+====+===================+==============+=================+
|1 |chien |M   |2010-04-05 13:43:00|Rox           |Mordille beaucoup|
+--+------+----+-------------------+--------------+-----------------+
|2 |chat  |NULL|2010-03-24 02:23:00|Roucky        |NULL             |
+--+------+----+-------------------+--------------+-----------------+
|3 |chat  |F   |2010-09-13 15:02:00|Schtroumpfette|NULL             |
+--+------+----+-------------------+--------------+-----------------+




Pour vérifier, vous pouvez utiliser la requête suivante :


```sql
SELECT * FROM Animal;
```

Deux choses importantes à retenir ici.


- _id_ est un nombre, on ne met donc pas de guillemets autour. Par contre, l'espèce, le nom, la date de naissance et le sexe sont donnés sous forme de chaînes de caractères. Les guillemets sont donc indispensables. Quant à `NULL`, il s'agit d'un marqueur SQL qui, je rappelle, signifie "pas de valeur". Pas de guillemets donc.
- Les valeurs des colonnes sont données dans le bon ordre (donc dans l'ordre donné lors de la création de la table). C'est indispensable évidemment. Si vous échangez le nom et l'espèce par exemple, comment MySQL pourrait-il le savoir ?



# Insertion en précisant les colonnes

Dans la requête, nous allons donc écrire explicitement à quelle(s) colonne(s) nous donnons une valeur. Ceci va permettre deux choses.


- On ne doit plus donner les valeurs dans l'ordre de création des colonnes, mais dans l'ordre précisé par la requête.
- On n'est plus obligé de donner une valeur à chaque colonne ; plus besoin de `NULL` lorsqu'on n'a pas de valeur à mettre.


Quelques exemples :


```sql
INSERT INTO Animal (espece, sexe, date_naissance) 
    VALUES ('tortue', 'F', '2009-08-03 05:12:00');
INSERT INTO Animal (nom, commentaires, date_naissance, espece) 
    VALUES ('Choupi', 'Né sans oreille gauche', '2010-10-03 16:44:00', 'chat');
INSERT INTO Animal (espece, date_naissance, commentaires, nom, sexe) 
    VALUES ('tortue', '2009-06-13 08:17:00', 'Carapace bizarre', 'Bobosse', 'F');
```

Ce qui vous donne trois animaux supplémentaires (donc six en tout, il faut suivre !)


# Insertion multiple

Si vous avez plusieurs lignes à introduire, il est possible de le faire en une seule requête de la manière suivante :


```sql
INSERT INTO Animal (espece, sexe, date_naissance, nom) 
VALUES ('chien', 'F', '2008-12-06 05:18:00', 'Caroline'),
        ('chat', 'M', '2008-09-11 15:38:00', 'Bagherra'),
        ('tortue', NULL, '2010-08-23 05:18:00', NULL);
```

Bien entendu, vous êtes alors obligés de préciser les mêmes colonnes pour chaque entrée, quitte à mettre `NULL` pour certaines. Mais avouez que ça fait quand même moins à écrire !