En plus de restreindre une recherche en lui donnant des critères grâce à la clause `WHERE`, il est possible de restreindre le nombre de lignes récupérées.
Cela se fait grâce à la clause `LIMIT`.


# Syntaxe

`LIMIT` s'utilise avec deux paramètres.



- Le nombre de lignes que l'on veut récupérer.
- Le décalage, introduit par le mot-clé `OFFSET` et qui indique à partir de quelle ligne on récupère les résultats. Ce paramètre est facultatif. S'il n'est pas précisé, il est mis à 0.



```sql
LIMIT nombre_de_lignes [OFFSET decalage];
```

**Exemple** :

```sql
SELECT * 
FROM Animal 
ORDER BY id 
LIMIT 6 OFFSET 0;

SELECT * 
FROM Animal 
ORDER BY id 
LIMIT 6 OFFSET 3;
```

Avec la première requête, vous devriez obtenir six lignes, les six plus petits _id_ puisque nous n'avons demandé aucun décalage (`OFFSET 0`). 




+--+------+----+-------------------+--------------+----------------------+
|id|espece|sexe|date_naissance     |nom           |commentaires          |
+==+======+====+===================+==============+======================+
|1 |chien |M   |2010-04-05 13:43:00|Rox           |Mordille beaucoup     |
+--+------+----+-------------------+--------------+----------------------+
|2 |chat  |NULL|2010-03-24 02:23:00|Roucky        |NULL                  |
+--+------+----+-------------------+--------------+----------------------+
|3 |chat  |F   |2010-09-13 15:02:00|Schtroumpfette|NULL                  |
+--+------+----+-------------------+--------------+----------------------+
|4 |tortue|F   |2009-08-03 05:12:00|NULL          |NULL                  |
+--+------+----+-------------------+--------------+----------------------+
|5 |chat  |NULL|2010-10-03 16:44:00|Choupi        |Né sans oreille gauche|
+--+------+----+-------------------+--------------+----------------------+
|6 |tortue|F   |2009-06-13 08:17:00|Bobosse       |Carapace bizarre      |
+--+------+----+-------------------+--------------+----------------------+





Par contre, dans la deuxième, vous récupérez toujours six lignes, mais vous devriez commencer au quatrième plus petit _id_, puisqu'on a demandé un décalage de trois lignes.




+--+------+----+-------------------+--------+----------------------+
|id|espece|sexe|date_naissance     |nom     |commentaires          |
+==+======+====+===================+========+======================+
|4 |tortue|F   |2009-08-03 05:12:00|NULL    |NULL                  |
+--+------+----+-------------------+--------+----------------------+
|5 |chat  |NULL|2010-10-03 16:44:00|Choupi  |Né sans oreille gauche|
+--+------+----+-------------------+--------+----------------------+
|6 |tortue|F   |2009-06-13 08:17:00|Bobosse |Carapace bizarre      |
+--+------+----+-------------------+--------+----------------------+
|7 |chien |F   |2008-12-06 05:18:00|Caroline|NULL                  |
+--+------+----+-------------------+--------+----------------------+
|8 |chat  |M   |2008-09-11 15:38:00|Bagherra|NULL                  |
+--+------+----+-------------------+--------+----------------------+
|9 |tortue|NULL|2010-08-23 05:18:00|NULL    |NULL                  |
+--+------+----+-------------------+--------+----------------------+





**Exemple avec un seul paramètre** :

```sql
SELECT * 
FROM Animal 
ORDER BY id 
LIMIT 10;
```

Cette requête est donc équivalente à :


```sql
SELECT * 
FROM Animal 
ORDER BY id 
LIMIT 10 OFFSET 0;
```


# Syntaxe alternative

MySQL accepte une autre syntaxe pour la clause `LIMIT`. Ce n'est cependant pas la norme SQL donc idéalement vous devriez toujours utiliser la syntaxe officielle. Vous vous apercevrez toutefois que cette syntaxe est énormément usitée, je ne pouvais donc pas ne pas la mentionner


```sql
SELECT * 
FROM Animal 
ORDER BY id 
LIMIT [decalage, ]nombre_de_lignes;
```

Tout comme pour la syntaxe officielle, le décalage n'est pas obligatoire, et vaudra 0 par défaut. Si vous le précisez, n'oubliez pas la virgule entre le décalage et le nombre de lignes désirées.