Il est possible de faire une recherche sur un intervalle à l'aide uniquement des opérateurs de comparaison >= et <=. Par exemple, on peut rechercher les animaux qui sont nés entre le 5 janvier 2008 et le 23 mars 2009 de la manière suivante :


```sql
SELECT * 
FROM Animal 
WHERE date_naissance <= '2009-03-23' 
    AND date_naissance >= '2008-01-05';
```

Ça fonctionne très bien. Cependant, SQL dispose d'un opérateur spécifique pour les intervalles, qui pourrait vous éviter les erreurs d'inattention classiques (< au lieu de > par exemple) en plus de rendre votre requête plus lisible et plus performante : `BETWEEN minimum AND maximum` (_between_ signifie "entre" en anglais). La requête précédente peut donc s'écrire :


```sql
SELECT * 
FROM Animal 
WHERE date_naissance BETWEEN '2008-01-05' AND '2009-03-23';
```

`BETWEEN` peut s'utiliser avec des dates, mais aussi avec des nombres (`BETWEEN 0 AND 100`) ou avec des chaînes de caractères (`BETWEEN 'a' AND 'd'`) auquel cas c'est l'ordre alphabétique qui sera utilisé (toujours insensible à la casse sauf si l'on utilise des chaînes binaires : `BETWEEN BINARY 'a' AND BINARY 'd'`).
Bien évidemment, on peut aussi exclure un intervalle avec `NOT BETWEEN`.