En plus des index "simples", que je viens de vous décrire, il existe trois types d'index qui ont des propriétés particulières. Les index `UNIQUE`, les index `FULLTEXT`, et enfin les index `SPATIAL`.
Je ne détaillerai pas les propriétés et utilisations des index `SPATIAL`. Sachez simplement qu'il s'agit d'un type d'index utilisé dans des bases de données recensant des données spatiales (tiens donc !  :p ), donc des points, des lignes, des polygones…

Sur ce, c'est parti !


# Index UNIQUE

Avoir un index `UNIQUE` sur une colonne (ou plusieurs) permet de s'assurer que jamais vous n’insérerez deux fois la même valeur (ou combinaison de valeurs) dans la table.

Par exemple, vous créez un site internet, et vous voulez le doter d'un espace membre. Chaque membre devra se connecter grâce à un pseudo et un mot de passe. Vous avez donc une table _Membre_, qui contient 4 colonnes : _id_, _pseudo_, _mot_de_passe_ et _date_inscription_.
Deux membres peuvent avoir le même mot de passe, pas de problème. Par contre, que se passerait-il si deux membres avaient le même pseudo ? Lors de la connexion, il serait impossible de savoir quel mot de passe utiliser et sur quel compte connecter le membre.
Il faut donc absolument éviter que deux membres utilisent le même pseudo. Pour cela, on va utiliser un index `UNIQUE` sur la colonne _pseudo_ de la table _Membre_.

Autre exemple, dans notre table _Animal_ cette fois. Histoire de ne pas confondre les animaux, vous prenez la décision de ne pas nommer de la même manière deux animaux de la même espèce. Il peut donc n'y avoir qu'une seule tortue nommée Toto. Un chien peut aussi se nommer Toto, mais un seul. La combinaison (espèce, nom) doit n'exister qu'une et une seule fois dans la table. Pour s'en assurer, il suffit de créer un index unique sur les colonnes _espece_ et _nom_ (un seul index, sur les deux colonnes).


## Contraintes

Lorsque vous mettez un index `UNIQUE` sur une table, vous ne mettez pas seulement un index, vous ajoutez surtout une **contrainte**.
Les contraintes sont une notion importante en SQL. Sans le savoir, ou sans savoir que c'était appelé comme ça, vous en avez déjà utilisé. En effet, lorsque vous empêchez une colonne d'accepter `NULL`, vous lui mettez une **contrainte** `NOT NULL`.
De même, les valeurs par défaut que vous pouvez donner aux colonnes sont des contraintes. Vous contraignez la colonne à prendre une certaine valeur si aucune autre n'est précisée.


# Index FULLTEXT

Un index `FULLTEXT` est utilisé pour faire des recherches de manière puissante et rapide sur un texte. Seules les tables utilisant le moteur de stockage MyISAM peuvent avoir un index `FULLTEXT`, et cela uniquement sur les colonnes de type `CHAR`, `VARCHAR` ou `TEXT` (ce qui est plutôt logique).

Une différence importante (très importante !!! :diable: ) entre les index `FULLTEXT` et les index classiques (et `UNIQUE`) est que l'on ne peut plus utiliser les fameux "index par la gauche" dont je vous ai parlé précédemment. Donc, si vous voulez faire des recherches "fulltext" sur deux colonnes (parfois l'une, parfois l'autre, parfois les deux ensemble), il vous faudra créer **trois** index `FULLTEXT` : (_colonne1_), (_colonne2_) et (_colonne1_, _colonne2_).