# Principe des jointures

Sans surprise, le principe des jointures est de joindre plusieurs tables. Pour ce faire, on utilise les informations communes des tables.

Par exemple, lorsque nous avons ajouté dans notre base les informations sur les espèces (leur nom latin et leur description), je vous ai dit que ce serait une très mauvaise idée de tout mettre dans la table _Animal_, car il nous faudrait alors répéter la même description pour tous les chiens, la même pour toutes les tortues, etc.
Cependant, vous avez sans doute remarqué que du coup, si vous voulez afficher la description de l'espèce de Cartouche (votre petit préféré), vous avez besoin de deux requêtes.

**Étape 1** : on trouve l'_id_ de l'espèce de Cartouche grâce à la table _Animal_.

```sql
SELECT espece_id FROM Animal WHERE nom = 'Cartouche';
```




+---------+
|espece_id|
+=========+
|1        |
+---------+





**Étape 2** : on trouve la description de l'espèce grâce à son _id_.

```sql
SELECT description FROM Espece WHERE id = 1;
```




+------------------------------------------------------------------------+
|description                                                             |
+========================================================================+
|Bestiole à quatre pattes qui aime les caresses et tire souvent la langue|
+------------------------------------------------------------------------+





Ne serait-ce pas merveilleux de pouvoir faire tout ça (et plus encore) en une seule requête ? 
C'est là que les jointures entrent en jeu ; on va utiliser l'information commune entre les deux tables : l'_id_ de l'espèce, qui est présente dans _Animal_ avec la colonne _espece_id_, et dans _Espece_ avec la colonne _id_.


```sql
SELECT Espece.description 
FROM Espece 
INNER JOIN Animal 
    ON Espece.id = Animal.espece_id 
WHERE Animal.nom = 'Cartouche';
```


Et voilà le travail !




+------------------------------------------------------------------------+
|description                                                             |
+========================================================================+
|Bestiole à quatre pattes qui aime les caresses et tire souvent la langue|
+------------------------------------------------------------------------+






En fait, lorsque l'on fait une jointure, on crée une table virtuelle et temporaire qui reprend les colonnes des tables liées. Le schéma ci-dessous illustre ce principe.

Au départ, on a deux tables : _Animal_ (_id_, _sexe_, _nom_, _race_id_, _espece_id_) et _Espece_ (_id_, _nom_courant_, _nom_latin_). Les deux premières lignes d'_Animal_ correspondent à la première ligne d'_Espece_, et la troisième ligne d'_Animal_ à la deuxième ligne d'_Espece_. Une fois les deux tables jointes, on obtient une table possédant toutes les colonnes d'_Animal_ et toutes les colonnes d'_Espece_, avec les valeurs correspondantes de chaque table. On peut voir que les cinquième et sixième colonnes de la table de jointure ont les mêmes valeurs.
Ensuite, de cette table **virtuelle**, on peut extraire ce que l'on veut. La colonne _nom_latin_ pour la ligne ayant "Caribou" dans la colonne _nom_, par exemple.

![Principe des jointures](/media/galleries/1255/d5dccca9-f94c-4ba5-8139-48f0379f11b7.png)


# Notion d'alias

Je fais ici une petite parenthèse avant de vous expliquer en détail le fonctionnement des jointures pour vous parler d'un petit truc bien utile : les alias.

Les alias sont des noms de remplacement, que l'on donne de manière temporaire (le temps d'une requête en fait) à une colonne, une table, une donnée. Les alias sont introduits par le mot-clé `AS`. Ce mot-clé est facultatif, vous pouvez très bien définir un alias sans utiliser `AS`, mais je préfère personnellement toujours le mettre. Je trouve qu'on y voit plus clair.

Comment ça marche ?

Prenons cette requête toute simple :


```sql
SELECT 5+3;
```




+---+
|5+3|
+===+
|8  |
+---+





Imaginons que ce calcul savant représente en fait le nombre de chiots de Cartouche, qui a eu une première portée de 5 chiots, et une seconde de seulement 3 chiots. Nous voudrions donc indiquer qu'il s'agit bien de ça, et non pas d'un calcul inutile destiné simplement à illustrer une notion obscure.

Facile ! Il suffit d'utiliser les alias :


```sql
SELECT 5+3 AS Chiots_Cartouche;

-- OU, sans utiliser AS

SELECT 5+3 Chiots_Cartouche;
```




+----------------+
|Chiots_Cartouche|
+================+
|8               |
+----------------+





Bon, tout ça c'est bien joli, mais pour l'instant ça n'a pas l'air très utile…
Prenons un exemple plus parlant : retournez voir le schéma qui explique le principe des jointures un peu plus haut. La table virtuelle résultant de la jointure des tables _Espece_ et _Animal_ possède plusieurs colonnes : 
_

- **id**
- sexe
- nom
- race_id
- espece_id
- **id**
- nom_courant
- nom_latin

_ 
Mais que vois-je ? J'ai deux colonnes _id_ ! Comment faire pour les différencier ? Comment être sûr de savoir à quoi elles se rapportent ?
Avec les alias pardi ! Il suffit de donner l'alias _espece_id_ à la colonne _id_ de la table _Espece_, et _animal_id_ à la colonne _id_  de la table _Animal_. 
Tout à coup, ça vous semble plus intéressant non ?

Je vais vous laisser sur ce sentiment. Il n'y a pas grand-chose de plus à dire sur les alias, vous en comprendrez toute l'utilité à travers les nombreux exemples dans la suite de ce cours. L'important pour l'instant est que vous sachiez que ça existe et comment les définir.