L'air de rien, dans l'introduction, je vous ai déjà montré comment faire une jointure. La première émotion passée, vous devriez vous être dit "Tiens, mais ça n'a pas l'air bien compliqué en fait, les jointures".
En effet, une fois que vous aurez compris comment réfléchir aux jointures, tout se fera tout seul. Personnellement, ça m'aide vraiment d'imaginer la table virtuelle créée par la jointure, et de travailler sur cette table pour tout ce qui est conditions, tris, etc.

Revoici la jointure que je vous ai fait faire, et qui est en fait une jointure **interne**.


```sql
SELECT Espece.description 
FROM Espece 
INNER JOIN Animal 
    ON Espece.id = Animal.espece_id 
WHERE Animal.nom = 'Cartouche';
```

Décomposons !


- **`SELECT Espece.description`** : je sélectionne la colonne _description_ de la table _Espece_.
- **`FROM Espece`** : je travaille sur la table _Espece_.
- **`INNER JOIN Animal`** : je la joins (avec une jointure interne) à la table _Animal_.
- **`ON Espece.id = Animal.espece_id`** : la jointure se fait sur les colonnes _id_ de la table _Espece_ et _espece_id_ de la table _Animal_, qui doivent donc correspondre.
- **`WHERE Animal.nom = 'Cartouche'`** : dans la table résultant de la jointure, je sélectionne les lignes qui ont la valeur "Cartouche" dans la colonne _nom_ venant de la table _Animal_.


Si vous avez compris ça, vous avez tout compris !


# Syntaxe

Comme d'habitude, voici donc la syntaxe à utiliser pour faire des requêtes avec jointure(s) interne(s).


```sql
SELECT *                                   -- comme d'habitude, vous sélectionnez les colonnes que vous voulez
FROM nom_table1   
[INNER] JOIN nom_table2                    -- INNER explicite le fait qu'il s'agit d'une jointure interne, mais c'est facultatif
    ON colonne_table1 = colonne_table2     -- sur quelles colonnes se fait la jointure
                                           -- vous pouvez mettre colonne_table2 = colonne_table1, l'ordre n'a pas d'importance

[WHERE ...]                               
[ORDER BY ...]                            -- les clauses habituelles sont bien sûr utilisables !
[LIMIT ...]
```


## Condition de jointure

La clause `ON` sert à préciser la condition de la jointure. C'est-à-dire sur quel(s) critère(s) les deux tables doivent être jointes. Dans la plupart des cas, il s'agira d'une condition d'égalité simple, comme `ON Animal.espece_id = Espece.id`. Il est cependant tout à fait possible d'avoir plusieurs conditions à remplir pour lier les deux tables. On utilise alors les opérateurs logiques habituels. Par exemple, une jointure peut très bien se faire sur plusieurs colonnes :


```sql
SELECT *
FROM nom_table1
INNER JOIN nom_table2
   ON colonne1_table1 = colonne1_table2
      AND colonne2_table1 = colonne2_table2
      [AND ...];
```


## Expliciter le nom des colonnes

Il peut arriver que vous ayez dans vos deux tables des colonnes portant le même nom. C'est le cas dans notre exemple, puisque la table _Animal_ comporte une colonne _id_, tout comme la table _Espece_. Il est donc important de préciser de quelle colonne on parle dans ce cas-là.
Vous l'avez vu dans notre requête, on utilise pour ça l'opérateur **.** (_nom_table.nom_colonne_). Pour les colonnes ayant un nom non-ambigu (qui n'existe dans aucune autre table de la jointure), il n'est pas obligatoire de préciser la table.
En général, je précise la table quand il s'agit de grosses requêtes avec plusieurs jointures. En revanche, pour les petites jointures courantes, il est vrai que c'est moins long à écrire si on ne précise pas la table.

**Exemple** : sélection du nom des animaux commençant par "Ch", ainsi que de l'id et la description de leur espèce.


```sql
SELECT Espece.id,                   -- ici, pas le choix, il faut préciser
       Espece.description,          -- ici, on pourrait mettre juste description
       Animal.nom                   -- idem, la précision n'est pas obligatoire. C'est cependant plus clair puisque les espèces ont un nom aussi
FROM Espece   
INNER JOIN Animal
     ON Espece.id = Animal.espece_id
WHERE Animal.nom LIKE 'Ch%';
```




+--+-----------------------------------------------------------------+-------+
|id|description                                                      |nom    |
+==+=================================================================+=======+
|2 |Bestiole à quatre pattes qui saute très haut et grimpe aux arbres|Choupi |
+--+-----------------------------------------------------------------+-------+
|3 |Bestiole avec une carapace très dure                             |Cheli  |
+--+-----------------------------------------------------------------+-------+
|3 |Bestiole avec une carapace très dure                             |Chicaca|
+--+-----------------------------------------------------------------+-------+






## Utiliser les alias

Les alias sont souvent utilisés avec les jointures. Ils permettent notamment de renommer les tables, et ainsi d'écrire moins de code.

**Exemple** : on renomme la table _Espece_ "_e_", et la table _Animal_ "_a_".


```sql
SELECT e.id,                  
       e.description,          
       a.nom                   
FROM Espece AS e          -- On donne l'alias "e" à Espece
INNER JOIN Animal AS a    -- et l'alias "a" à Animal.
     ON e.id = a.espece_id
WHERE a.nom LIKE 'Ch%';
```


Comme vous le voyez, le code est plus compact. Ici encore, c'est quelque chose que j'utilise souvent pour de petites requêtes ponctuelles. Par contre, pour de grosses requêtes, je préfère les noms explicites ; c'est ainsi plus facile de s'y retrouver.

Une autre utilité des alias est de renommer les colonnes pour que le résultat soit plus clair. Observez le résultat de la requête précédente. Vous avez trois colonnes : _id_, _description_ et _nom_. Le nom de la table dont provient la colonne n'est indiqué nulle part. A priori, vous savez ce que vous avez demandé, surtout qu'il n'y a pas encore trop de colonnes, mais imaginez que vous sélectionniez une vingtaine de colonnes. Ce serait quand même mieux de savoir de quel _id_ on parle, s'il s'agit du nom de l'animal, de son maître, du père, du fils ou du Saint-Esprit !
Il est intéressant là aussi d'utiliser les alias.

**Exemple** : on donne des alias aux colonnes (_id_espece_ pour _id_ de la table _Espece_, _description_espece_ pour _Espece.description_ et _nom_bestiole_ pour _Animal.nom_).


```sql
SELECT Espece.id AS id_espece,                  
       Espece.description AS description_espece,          
       Animal.nom AS nom_bestiole                   
FROM Espece   
INNER JOIN Animal
     ON Espece.id = Animal.espece_id
WHERE Animal.nom LIKE 'Ch%';
```




+---------+-----------------------------------------------------------------+------------+
|id_espece|description_espece                                               |nom_bestiole|
+=========+=================================================================+============+
|2        |Bestiole à quatre pattes qui saute très haut et grimpe aux arbres|Choupi      |
+---------+-----------------------------------------------------------------+------------+
|3        |Bestiole avec une carapace très dure                             |Cheli       |
+---------+-----------------------------------------------------------------+------------+
|3        |Bestiole avec une carapace très dure                             |Chicaca     |
+---------+-----------------------------------------------------------------+------------+





C'est tout de suite plus clair !


# Pourquoi "interne" ?

`INNER JOIN` permet donc de faire une jointure **interne** sur deux tables. Mais que signifie donc ce "interne" ?

C'est très simple ! Lorsque l'on fait une jointure interne, cela veut dire qu'on exige qu'il y ait des données de part et d'autre de la jointure. Donc, si l'on fait une jointure sur la colonne _a_ de la table _A_ et la colonne _b_ de la table _B_ :


```sql
SELECT * 
FROM A
INNER JOIN B 
    ON A.a = B.b
```

Ceci retournera **uniquement** les lignes pour lesquelles _A.a_ et _B.b_ correspondent.

**Exemple** : on veut connaître la race des chats :


```sql
SELECT Animal.nom AS nom_animal, Race.nom AS race
FROM Animal
INNER JOIN Race
    ON Animal.race_id = Race.id
WHERE Animal.espece_id = 2             -- ceci correspond aux chats
ORDER BY Race.nom, Animal.nom;
```




+--------------+----------+
|nom_animal    |race      |
+==============+==========+
|Callune       |Bleu russe|
+--------------+----------+
|Caribou       |Bleu russe|
+--------------+----------+
|Cawette       |Bleu russe|
+--------------+----------+
|Feta          |Bleu russe|
+--------------+----------+
|Filou         |Bleu russe|
+--------------+----------+
|Raccou        |Bleu russe|
+--------------+----------+
|Schtroumpfette|Bleu russe|
+--------------+----------+
|Bagherra      |Maine coon|
+--------------+----------+
|Bilba         |Maine coon|
+--------------+----------+
|Capou         |Maine coon|
+--------------+----------+
|Cracotte      |Maine coon|
+--------------+----------+
|Farceur       |Maine coon|
+--------------+----------+
|Milla         |Maine coon|
+--------------+----------+
|Zara          |Maine coon|
+--------------+----------+
|Zonko         |Maine coon|
+--------------+----------+
|Boucan        |Singapura |
+--------------+----------+
|Boule         |Singapura |
+--------------+----------+
|Fiero         |Singapura |
+--------------+----------+





On peut voir ici que les chats Choupi et Roucky pour lesquels je n'ai pas d'information sur la race (_race_id_ est `NULL`), ne sont pas repris dans les résultats. De même, aucun des chats n'est de la race "Sphynx", celle-ci n'est donc pas reprise. Si je veux les inclure, je dois utiliser une jointure externe.