Comme je viens de vous le dire, une jointure externe permet de sélectionner également les lignes pour lesquelles il n'y a pas de correspondance dans une des tables jointes. MySQL permet deux types de jointures externes : les jointures par la gauche et les jointures par la droite.


# Jointures par la gauche

Lorsque l'on fait une jointure par la gauche (grâce aux mots-clés `LEFT JOIN` ou `LEFT OUTER JOIN`), cela signifie que l'on veut toutes les lignes de la table de gauche (sauf restrictions dans une clause `WHERE` bien sûr), même si certaines n'ont pas de correspondance avec une ligne de la table de droite.

Alors, table de gauche, table de droite, laquelle est laquelle ? C'est très simple, nous lisons de gauche à droite, donc la table de gauche est la première table mentionnée dans la requête, c'est-à-dire, en général, la table donnée dans la clause `FROM`.

Si l'on veut de nouveau connaître la race des chats, mais que cette fois-ci nous voulons également afficher les chats qui n'ont pas de race, on peut utiliser la jointure suivante (je ne prends que les chats dont le nom commence par "C" afin de réduire le nombre de lignes, mais vous pouvez choisir les conditions que vous voulez) :


```sql
SELECT Animal.nom AS nom_animal, Race.nom AS race
FROM Animal                                               -- Table de gauche
LEFT JOIN Race                                            -- Table de droite
    ON Animal.race_id = Race.id
WHERE Animal.espece_id = 2 
    AND Animal.nom LIKE 'C%'
ORDER BY Race.nom, Animal.nom;

-- OU 

SELECT Animal.nom AS nom_animal, Race.nom AS race
FROM Animal                                               -- Table de gauche
LEFT OUTER JOIN Race                                      -- Table de droite
    ON Animal.race_id = Race.id
WHERE Animal.espece_id = 2
    AND Animal.nom LIKE 'C%'
ORDER BY Race.nom, Animal.nom;
```




+----------+----------+
|nom_animal|race      |
+==========+==========+
|Choupi    |NULL      |
+----------+----------+
|Callune   |Bleu russe|
+----------+----------+
|Caribou   |Bleu russe|
+----------+----------+
|Cawette   |Bleu russe|
+----------+----------+
|Capou     |Maine coon|
+----------+----------+
|Cracotte  |Maine coon|
+----------+----------+





On ne connaît pas la race de Choupi, et pourtant il fait bien partie des lignes sélectionnées alors qu'avec la jointure interne il n'apparaissait pas. Simplement, les colonnes qui viennent de la table Race (la colonne `Race.nom AS race` dans ce cas-ci) sont `NULL` pour les lignes qui n'ont pas de correspondance (la ligne de Choupi ici).


# Jointures par la droite

Les jointures par la droite (`RIGHT JOIN` ou `RIGHT OUTER JOIN`), c'est évidemment le même principe, sauf que ce sont toutes les lignes de la table de droite qui sont sélectionnées même s'il n'y a pas de correspondance dans la table de gauche.

**Exemple** : toujours avec les races de chats.


```sql
SELECT Animal.nom AS nom_animal, Race.nom AS race
FROM Animal                                                -- Table de gauche
RIGHT JOIN Race                                            -- Table de droite
    ON Animal.race_id = Race.id
WHERE Race.espece_id = 2
ORDER BY Race.nom, Animal.nom;

- OU

SELECT Animal.nom AS nom_animal, Race.nom AS race
FROM Animal                                              -- Table de gauche
RIGHT OUTER JOIN Race                                    -- Table de droite
    ON Animal.race_id = Race.id
WHERE Race.espece_id = 2
ORDER BY Race.nom, Animal.nom;
```




+--------------+----------+
|nom_animal    |race      |
+==============+==========+
|Callune       |Bleu russe|
+--------------+----------+
|Caribou       |Bleu russe|
+--------------+----------+
|Cawette       |Bleu russe|
+--------------+----------+
|Feta          |Bleu russe|
+--------------+----------+
|Filou         |Bleu russe|
+--------------+----------+
|Raccou        |Bleu russe|
+--------------+----------+
|Schtroumpfette|Bleu russe|
+--------------+----------+
|Bagherra      |Maine coon|
+--------------+----------+
|Bilba         |Maine coon|
+--------------+----------+
|Capou         |Maine coon|
+--------------+----------+
|Cracotte      |Maine coon|
+--------------+----------+
|Farceur       |Maine coon|
+--------------+----------+
|Milla         |Maine coon|
+--------------+----------+
|Zara          |Maine coon|
+--------------+----------+
|Zonko         |Maine coon|
+--------------+----------+
|Boucan        |Singapura |
+--------------+----------+
|Boule         |Singapura |
+--------------+----------+
|Fiero         |Singapura |
+--------------+----------+
|NULL          |Sphynx    |
+--------------+----------+





On a bien une ligne avec la race "Sphynx", bien que nous n'ayons aucun sphynx dans notre table _Animal_.

À noter que toutes les jointures par la droite peuvent être faites grâce à une jointure par la gauche (et vice versa). Voici l'équivalent avec une jointure par la gauche de la requête que nous venons d'écrire :


```sql
SELECT Animal.nom AS nom_animal, Race.nom AS race
FROM Race                                               -- Table de gauche
LEFT JOIN Animal                                        -- Table de droite
    ON Animal.race_id = Race.id
WHERE Race.espece_id = 2
ORDER BY Race.nom, Animal.nom;
```