Les syntaxes que je vous ai montrées jusqu'ici, avec `[INNER] JOIN` et `LEFT|RIGHT [OUTER] JOIN`, sont les syntaxes classiques que vous retrouverez le plus souvent. Il existe cependant d'autres manières de faire des jointures.


# Jointures avec USING

Lorsque les colonnes qui servent à joindre les deux tables ont **le même nom**, vous pouvez utiliser la clause `USING` au lieu de la clause `ON`.


```sql
SELECT *
FROM table1
[INNER | LEFT | RIGHT] JOIN table2 USING (colonneJ);  -- colonneJ est présente dans les deux tables

-- équivalent à 

SELECT *
FROM table1
[INNER | LEFT | RIGHT] JOIN table2 ON Table1.colonneJ = table2.colonneJ;
```


[[information]]
| Si la jointure se fait sur plusieurs colonnes, il suffit de lister les colonnes en les séparant par des virgules : `USING (colonne1, colonne2,…)`



# Jointures naturelles

Comme pour les jointures avec `USING`, il est possible d'utiliser les jointures naturelles dans le cas où les colonnes servant à la jointure ont **le même nom** dans les deux tables.
Simplement, dans le cas d'une jointure naturelle, on ne donne pas la (les) colonne(s) sur laquelle (lesquelles) joindre les tables : c'est déterminé automatiquement. Donc si on a les trois tables suivantes :


- _table1_ : colonnes _A_, _B_, _C_
- _table2_ : colonnes _B_, _E_, _F_
- _table3_ : colonnes _A_, _C_, _E_


**Exemple 1** : jointure de _table1_ et _table2_ (une colonne ayant le même nom : _B_).


```sql
SELECT * 
FROM table1
NATURAL JOIN table2;

-- EST ÉQUIVALENT À

SELECT *
FROM table1
INNER JOIN table2 
    ON table1.B = table2.B;
```


**Exemple 2** : jointure de _table1_ et _table3_ (deux colonnes ayant le même nom : _A_ et _C_).


```sql
SELECT * 
FROM table1
NATURAL JOIN table3;

-- EST ÉQUIVALENT À

SELECT *
FROM table1
INNER JOIN table3
    ON table1.A = table3.A AND table1.C = table3.C;
```

Pour utiliser ce type de jointure, il faut donc être certain que **toutes les colonnes nécessaires** à la jointure ont le même nom dans les deux tables, mais aussi que les colonnes ayant le même nom sont **uniquement celles qui servent à la jointure**.

Notez que vous pouvez également réaliser une jointure externe par la gauche avec les jointures naturelles à l'aide de `NATURAL LEFT JOIN`.


# Jointures sans JOIN

Cela peut paraître absurde, mais il est tout à fait possible de faire une jointure sans utiliser le mot `JOIN`. Ce n'est cependant possible que pour les jointures internes.
Il suffit de mentionner les tables que l'on veut joindre dans la clause `FROM` (séparées par des virgules), et de mettre la condition de jointure dans la clause `WHERE` (sans clause `ON`).


```sql
SELECT *
FROM table1, table2
WHERE table1.colonne1 = table2.colonne2;

-- équivalent à 

SELECT *
FROM table1
[INNER] JOIN table2
    ON table1.colonne1 = table2.colonne2;
```

Je vous déconseille cependant d'utiliser cette syntaxe. En effet, lorsque vous ferez des grosses jointures, avec plusieurs conditions dans la clause `WHERE`, vous serez bien contents de pouvoir différencier au premier coup d’œil les conditions de jointures des conditions "normales".