Maintenant que vous savez comment faire une jointure, on va un peu s'amuser. Cette partie sera en fait un mini-TP. Je vous dis à quel résultat vous devez parvenir en utilisant des jointures, vous essayez, et ensuite, vous allez voir la réponse et les explications.

Techniquement, vous avez vu toute la théorie nécessaire pour réaliser toutes les requêtes que je vous demanderai ici. Cependant, il y aura des choses que je ne vous ai pas "montrées" explicitement, comme les jointures avec plus de deux tables, ou les auto-jointures (joindre une table avec elle-même). 
C'est voulu ! 

Je ne m'attends pas à ce que vous réussissiez à construire toutes les requêtes sans jeter un œil à la solution. Le but ici est de vous faire réfléchir, et surtout de vous faire prendre conscience qu'on peut faire pas mal de chose en SQL, en combinant plusieurs techniques par exemple. 
Si un jour vous vous dites "Tiens, ce serait bien si en SQL on pouvait…", arrêtez de vous le dire et faites-le, simplement ! Vous serez probablement plus souvent limités par votre imagination que par SQL.

Bien, ça c'est dit, donc allons-y !


[[attention]]
| Pour jouer le jeu jusqu'au bout, je vous propose de considérer que vous ne connaissez pas les _id_ correspondants aux différentes races et espèces. Donc quand je demande la liste des chiens par exemple, il n'est pas intéressant de sélectionner les animaux `WHERE espece_id = 1` ; il est bien plus utile de faire une jointure avec la table _Espece_.



# A/ Commençons par des choses faciles

Des jointures sur deux tables, avec différentes conditions à respecter.


## 1. Moi, j'aime bien les chiens de berger

**Vous devez obtenir la liste des races de chiens qui sont des chiens de berger.**


[[information]]
| On considère (même si ce n'est pas tout à fait vrai) que les chiens de berger ont "berger" dans leur nom de race.



[[secret]]
| ```sql
| SELECT Race.nom AS Race 
| FROM Race
| INNER JOIN Espece ON Espece.id = Race.espece_id
| WHERE Espece.nom_courant = 'chien' AND Race.nom LIKE '%berger%';
| ```
| 
| 
| Bon, c'était juste un échauffement. Normalement vous ne devriez pas avoir eu de difficultés avec cette requête. Peut-être avez-vous oublié la condition `Espece.nom_courant = 'chien'` ? On ne sait jamais, une race de chat (ou autre) pourrait très bien contenir "berger", or j'ai explicitement demandé les chiens.




## 2. Mais de quelle couleur peut bien être son pelage ?

**Vous devez obtenir la liste des animaux (leur nom, date de naissance et race) pour lesquels nous n'avons aucune information sur la couleur que devrait avoir leur pelage.**


[[information]]
| Dans la description des races, j'utilise parfois "pelage", parfois "poils", et parfois "robe".



[[secret]]
| ```sql
| SELECT Animal.nom AS nom_animal, Animal.date_naissance, Race.nom AS race
| FROM Animal
| LEFT JOIN Race
|     ON Animal.race_id = Race.id
| WHERE (Race.description NOT LIKE '%poil%' 
|         AND Race.description NOT LIKE '%robe%'
|         AND Race.description NOT LIKE '%pelage%'
|       )
|       OR Race.id IS NULL;
| ```
| 
| 
| Il faut donc :
| 
| 
| - soit qu'on ne connaisse pas la race (on n'a alors aucune information sur le pelage de la race, a fortiori) ;
| - soit qu'on connaisse la race, mais que dans la description de celle-ci il n'y ait pas d'information sur le pelage.
| 
| 
| 
| **1/ On ne connaît pas la race**
| 
| Les animaux dont on ne connaît pas la race ont `NULL` dans la colonne _race_id_. Mais vu que l'on fait une jointure sur cette colonne, il ne faut pas oublier de faire une jointure **externe**, sinon tous ces animaux sans race seront éliminés.
| Une fois que c'est fait, pour les sélectionner, il suffit de mettre la condition `Animaux.race_id IS NULL` par exemple, ou simplement `Race.#n'importe quelle colonne# IS NULL` vu qu'il n'y a pas de correspondance avec _Race_.
| 
| **2/ Pas d'information sur le pelage dans la description de la race**
| 
| Je vous ai donné comme indice le fait que j'utilisais les mots "pelage", "poil" ou "robe" dans les descriptions des espèces. Il fallait donc sélectionner les races pour lesquelles la description ne contient pas ces mots. D'où l'utilisation de `NOT LIKE`.
| 
| Si on met tout ça ensemble : il fallait faire une jointure externe des tables _Animal_ et _Race_, et ensuite sélectionner les animaux qui répondaient à l'une ou l'autre des conditions (opérateur logique `OR`).




# B/ Compliquons un peu les choses

Jointures sur deux tables, ou plus !


## 1. La race ET l'espèce

**Vous devez obtenir la liste des chats et des perroquets amazones, avec leur sexe, leur espèce (nom latin) et leur race s'ils en ont une. Regroupez les chats ensemble, les perroquets ensemble et, au sein de l'espèce, regroupez les races.**



[[secret]]
| ```sql
| SELECT Animal.nom as nom_animal, Animal.sexe, Espece.nom_latin as espece, Race.nom as race
| FROM Animal
| INNER JOIN Espece
|     ON Animal.espece_id = Espece.id
| LEFT JOIN Race
|     ON Animal.race_id = Race.id
| WHERE Espece.nom_courant IN ('Perroquet amazone', 'Chat')
| ORDER BY Espece.nom_latin, Race.nom;
| ```
| 
| 
| Comme vous voyez, c'est non seulement très simple de faire des jointures sur plus d'une table, mais c'est également possible de mélanger jointures internes et externes. Si ça vous pose problème, essayez vraiment de vous imaginer les étapes. 
| D'abord, on fait la jointure d'_Animal_ et d'_Espece_. On se retrouve alors avec une grosse table qui possède toutes les colonnes d'_Animal_ et toutes les colonnes d'_Espece_. Ensuite, à cette grosse table (à la fois virtuelle et intermédiaire), on joint la table _Race_, grâce à la colonne _Animal.race_id_. 
| Notez que l'ordre dans lequel vous faites les jointures n'est pas important.
| 
| En ce qui concerne la clause `ORDER BY`, j'ai choisi de trier par ordre alphabétique, mais il est évident que vous pouviez également trier sur les _id_ de l'espèce et de la race. L'important ici était de trier d'abord sur une colonne d'_Espece_, ensuite sur une colonne de _Race_.
 



## 2. Futures génitrices

**Vous devez obtenir la liste des chiennes dont on connaît la race, et qui sont en âge de procréer (c'est-à-dire nées avant juillet 2010). Affichez leur nom, date de naissance et race.**



[[secret]]
| ```sql
| SELECT Animal.nom AS nom_chienne, Animal.date_naissance, Race.nom AS race
| FROM Animal
| INNER JOIN Espece
|     ON Animal.espece_id = Espece.id
| INNER JOIN Race
|     ON Animal.race_id = Race.id
| WHERE Espece.nom_courant = 'chien'
|     AND Animal.date_naissance < '2010-07-01'
|     AND Animal.sexe = 'F';
| ```
| 
| 
| Cette fois, il fallait faire une jointure interne avec _Race_ puisqu'on voulait que la race soit connue. Le reste de la requête ne présentait pas de difficulté majeure.




# C/ Et maintenant, le test ultime !

Jointures sur deux tables ou plus, avec éventuelles auto-jointures.

Je vous ai fait rajouter, à la fin du chapitre précédent, deux jolies petites colonnes dans la table _Animal_ : _mere_id_ et _pere_id_. Le moment est venu de les utiliser !


## 1. Mon père, ma mère, mes frères et mes sœurs (Wohooooo)

**Vous devez obtenir la liste des chats dont on connaît les parents, ainsi que le nom de ces parents.**



[[secret]]
| ```sql
| SELECT Animal.nom, Pere.nom AS Papa, Mere.nom AS Maman
| FROM Animal
| INNER JOIN Animal AS Pere
|     ON Animal.pere_id = Pere.id
| INNER JOIN Animal AS Mere
|     ON Animal.mere_id = Mere.id
| INNER JOIN Espece
|     ON Animal.espece_id = Espece.id
| WHERE Espece.nom_courant = 'chat';
| ```
| 
| 
| Si celle-là, vous l'avez trouvée tout seuls, je vous félicite ! Sinon, c'est un peu normal. Vous voici face au premier cas dans lequel les alias sont **obligatoires**. En effet, vous aviez sans doute compris que vous pouviez faire `FROM Animal INNER JOIN Animal`, puisque j'avais mentionné les auto-jointures, mais vous avez probablement bloqué sur la clause `ON`. Comment différencier les colonnes d'_Animal_ dans `FROM` des colonnes d'_Animal_ dans `JOIN` ? Vous savez maintenant qu'il suffit d'utiliser des alias. 
| Il faut faire une jointure sur trois tables puisqu'au final, vous avez besoin des noms de trois animaux. Or en liant deux tables _Animal_ ensemble, vous avez deux colonnes _nom_. Pour pouvoir en avoir trois, il faut lier trois tables.
| 
| Prenez le temps de bien comprendre les auto-jointures, le pourquoi du comment et le comment du pourquoi. Faites des schémas si besoin, imaginez les tables intermédiaires.




## 2. Je suis ton père

**Histoire de se détendre un peu, vous devez maintenant obtenir la liste des enfants de Bouli (nom, sexe et date de naissance).**



[[secret]]
| ```sql
| SELECT Animal.nom, Animal.sexe, Animal.date_naissance
| FROM Animal
| INNER JOIN Animal AS Pere
|     ON Animal.pere_id = Pere.id
| WHERE Pere.nom = 'Bouli';
| ```
| 
| 
| Après la requête précédente, celle-ci devrait vous sembler plutôt facile ! Notez qu'il y a plusieurs manières de faire bien sûr. En voici une autre :
| 
| 
| ```sql
| SELECT Enfant.nom, Enfant.sexe, Enfant.date_naissance
| FROM Animal
| INNER JOIN Animal AS Enfant
|     ON Enfant.pere_id = Animal.id
| WHERE Animal.nom = 'Bouli';
| ```
| 
| L'important, c'est le résultat ! Évidemment, si vous avez utilisé 45 jointures et 74 conditions, alors ce n'est pas bon non plus. Du moment que vous n'avez joint que deux tables, ça devrait être bon.




## 3. C'est un pure race ?

Courage, c'est la dernière (et la plus thrash  :ninja: )!

**Vous devez obtenir la liste des animaux dont on connaît le père, la mère, la race, la race du père, la race de la mère. Affichez le nom et la race de l'animal et de ses parents, ainsi que l'espèce de l'animal (pas des parents).**



[[secret]]
| ```sql
| SELECT Espece.nom_courant AS espece, Animal.nom AS nom_animal, Race.nom AS race_animal,
|     Pere.nom AS papa, Race_pere.nom AS race_papa,
|     Mere.nom AS maman, Race_mere.nom AS race_maman
| FROM Animal
| INNER JOIN Espece
|     ON Animal.espece_id = Espece.id
| INNER JOIN Race
|     ON Animal.race_id = Race.id
| INNER JOIN Animal AS Pere
|     ON Animal.pere_id = Pere.id
| INNER JOIN Race AS Race_pere
|     ON Pere.race_id = Race_pere.id
| INNER JOIN Animal AS Mere
|     ON Animal.mere_id = Mere.id
| INNER JOIN Race AS Race_mere
|     ON Mere.race_id = Race_mere.id;
| ```
| 
| 
| Pfiou  :euh:  ! Le principe est exactement le même que pour avoir le nom des parents. Il suffit de rajouter une jointure avec _Race_ pour le père, pour la mère, et pour l'enfant, en n'oubliant pas de bien utiliser les alias bien sûr.
| C'est avec ce genre de requête que l'on se rend compte à quel point il est important de bien structurer et indenter sa requête, et à quel point un choix d'alias intelligent peut clarifier les choses.



Si vous avez survécu jusqu'ici, vous devriez maintenant avoir compris en profondeur le principe des jointures, et être capables de manipuler de nombreuses tables sans faire tout tomber par terre.