Pour l'insertion nous n'allons nous servir que de sous-requêtes, pas de jointures. Quoique… La sous-requête pourrait très bien contenir une jointure ! 


# Sous-requête pour l'insertion

Vous venez d'acquérir un magnifique Maine Coon chez un éleveur voisin. Vous vous apprêtez à l'insérer dans votre base de données, mais vous ne vous souvenez absolument pas de l'id de la race Maine Coon, ni de celui de l'espèce Chat. Du coup, deux possibilités s'offrent à vous.


- Vous pouvez faire une requête pour sélectionner l'id de la race et de l'espèce, puis faire ensuite une requête d'insertion avec les données que vous venez d'afficher.
- Vous pouvez utiliser une sous-requête pour insérer directement l'id de la race et de l'espèce à partir du nom de la race.


Je ne sais pas ce que vous en pensez, mais moi je trouve la seconde option bien plus sexy !

Donc, allons-y. Qu'avons-nous comme information ?


- **Nom** : Yoda
- **Date de naissance** : 2010-11-09
- **Sexe** : mâle
- **Espèce** : chat
- **Race** : Maine coon


De quoi avons-nous besoin en plus pour l'insérer dans notre table ? L'_id_ de l'espèce et de la race.
Comment récupérer ces deux _id_ ? Ça, vous savez faire : une simple requête suffit.


```sql
SELECT id AS race_id, espece_id FROM Race WHERE nom = 'Maine coon';
```




+-------+---------+
|race_id|espece_id|
+=======+=========+
|5      |2        |
+-------+---------+





Bien, mais le but était de tout faire en une seule requête, pas d'insérer nous-mêmes 5 et 2 après les avoir récupérés.

C'est ici que les sous-requêtes interviennent. Nous allons utiliser une nouvelle syntaxe d'insertion.


## INSERT INTO... SELECT

Cette syntaxe permet de sélectionner des éléments dans des tables, afin de les insérer directement dans une autre.


```sql
INSERT INTO nom_table
   [(colonne1, colonne2, ...)]
SELECT [colonne1, colonne2, ...]
FROM nom_table2
[WHERE ...]
```

Vous n'êtes bien sûr pas obligés de préciser dans quelles colonnes se fait l'insertion, si vous sélectionnez une valeur pour toutes les colonnes de la table. Ce sera cependant rarement le cas puisque nous avons des clés primaires auto-incrémentées.

Avec cette requête, il est absolument indispensable (sinon l'insertion ne se fera pas), d'**avoir le même nombre de colonnes dans l'insertion et dans la sélection**, et qu'elles soient **dans le même ordre**.
Si vous n'avez pas le même nombre de colonnes, cela déclenchera une erreur. De plus, si l'ordre n'est pas bon, vous aurez probablement une insertion erronée.

Nous allons donc insérer le résultat de notre requête qui sélectionne les _id_ de l'espèce et la race directement dans notre table _Animal_.
Pas de souci, mais il faut également insérer le nom, le sexe, etc.

En effet ! Mais c'est très facile. Souvenez-vous, vous pouvez très bien faire des requêtes de ce type :

```sql
SELECT 'Yoda' AS nom;
```

Et donc, en combinant avec notre requête précédente :


```sql
SELECT id AS race_id, espece_id FROM Race WHERE nom = 'Maine coon';
```

Vous pouvez obtenir très facilement, et en une seule requête, tous les renseignements indispensables à l'insertion de notre petit Yoda !


```sql
SELECT 'Yoda', 'M', '2010-11-09', id AS race_id, espece_id
FROM Race WHERE nom = 'Maine coon';
```



[[attention]]
| Attention à ne pas oublier les guillemets autour des chaînes de caractères, sinon MySQL va essayer de trouver la colonne _Yoda_ de la table _Race_, et forcément ça générera une erreur.

Si tout se passe bien, cette requête devrait vous donner ceci :




+----+-+----------+-------+---------+
|Yoda|M|2010-11-09|race_id|espece_id|
+====+=+==========+=======+=========+
|Yoda|M|2010-11-09|5      |2        |
+----+-+----------+-------+---------+





Les noms qui sont donnés aux colonnes n'ont pas d'importance, mais vous pouvez les changer avec des alias si cela vous perturbe.

Venons-en maintenant à notre super insertion !


```sql
INSERT INTO Animal 
    (nom, sexe, date_naissance, race_id, espece_id)              -- Je précise les colonnes puisque je ne donne pas une valeur pour toutes.
SELECT  'Yoda', 'M', '2010-11-09', id AS race_id, espece_id     -- Attention à l'ordre !
FROM Race WHERE nom = 'Maine coon';
```

Sélectionnons maintenant les Maine coon de notre base, pour vérifier que l'insertion s'est faite correctement.


```sql
SELECT Animal.id, Animal.sexe, Animal.nom, Race.nom AS race, Espece.nom_courant as espece
FROM Animal
INNER JOIN Race ON Animal.race_id = Race.id
INNER JOIN Espece ON Race.espece_id = Espece.id
WHERE Race.nom = 'Maine coon';
```

Et qui voyons-nous apparaître dans les résultats ? Notre petit Yoda !




+--+----+--------+----------+------+
|id|sexe|nom     |race      |espece|
+==+====+========+==========+======+
|8 |M   |Bagherra|Maine coon|Chat  |
+--+----+--------+----------+------+
|30|M   |Zonko   |Maine coon|Chat  |
+--+----+--------+----------+------+
|32|M   |Farceur |Maine coon|Chat  |
+--+----+--------+----------+------+
|34|M   |Capou   |Maine coon|Chat  |
+--+----+--------+----------+------+
|39|F   |Zara    |Maine coon|Chat  |
+--+----+--------+----------+------+
|40|F   |Milla   |Maine coon|Chat  |
+--+----+--------+----------+------+
|42|F   |Bilba   |Maine coon|Chat  |
+--+----+--------+----------+------+
|43|F   |Cracotte|Maine coon|Chat  |
+--+----+--------+----------+------+
|61|F   |Yoda    |Maine coon|Chat  |
+--+----+--------+----------+------+