Cette partie sera relativement courte, puisque l'utilisation des sous-requêtes et des jointures est assez ressemblante entre la suppression et la modification.
Simplement, pour la suppression, les sous-requêtes et jointures ne peuvent servir qu'à sélectionner les lignes à supprimer.


# Utilisation des sous-requêtes

On peut, tout simplement, utiliser une sous-requête dans la clause `WHERE`. Par exemple, imaginez que nous ayons deux animaux dont le nom est "Carabistouille", un chat et un perroquet. Vous désirez supprimer Carabistouille-le-chat, mais garder Carabistouille-le-perroquet. Vous ne pouvez donc pas utiliser la requête suivante, qui supprimera les deux :


```sql
DELETE FROM Animal WHERE nom = 'Carabistouille';
```

Mais il suffit d'une sous-requête dans la clause `WHERE` pour sélectionner l'espèce, et le tour est joué !


```sql
DELETE FROM Animal
WHERE nom = 'Carabistouille' AND espece_id = 
    (SELECT id FROM Espece WHERE nom_courant = 'Chat');
```


## Limitations

Les limitations sur `DELETE` sont les mêmes que pour `UPDATE` : on ne peut pas supprimer des lignes d'une table qui est utilisée dans une sous-requête.


# Suppression avec jointure

Pour les jointures, c'est le même principe. Si je reprends le même problème que ci-dessus, voici comment supprimer la ligne voulue avec une jointure :


```sql
DELETE Animal                                        -- Je précise de quelles tables les données doivent être supprimées
FROM Animal                                          -- Table principale
INNER JOIN Espece ON Animal.espece_id = Espece.id    -- Jointure     
WHERE Animal.nom = 'Carabistouille' AND Espece.nom_courant = 'Chat';
```

Vous remarquez une petite différence avec la syntaxe "classique" de `DELETE` (sans jointure) : je précise le nom de la table dans laquelle les lignes doivent être supprimées juste après le `DELETE`. En effet, comme on utilise plusieurs tables, cette précision est obligatoire. Ici, on ne supprimera que les lignes d'_Animal_ correspondantes.