Les commandes d'insertion et modification possèdent une option : `IGNORE`, qui permet d'ignorer (tiens donc !  o_O ) l'insertion ou la modification si elle viole une contrainte d'unicité.


# Insertion

Nous avons mis une contrainte d'unicité (sous la forme d'un index `UNIQUE`) sur la colonne _nom_latin_ de la table _Espece_. Donc, si l'on essaye d'insérer la ligne suivante, une erreur sera déclenchée puisqu'il existe déjà une espèce dont le nom latin est "Canis canis".


```sql
INSERT INTO Espece (nom_courant, nom_latin, description)
VALUES ('Chien en peluche', 'Canis canis', 'Tout doux, propre et  silencieux');
```


```console
ERROR 1062 (23000): Duplicate entry 'Canis canis' for key 'nom_latin'
```

Par contre, si l'on utilise le mot-clé `IGNORE` :


```sql
INSERT IGNORE INTO Espece (nom_courant, nom_latin, description)
VALUES ('Chien en peluche', 'Canis canis', 'Tout doux, propre et  silencieux');
```


```console
Query OK, 0 rows affected (0.01 sec)
```


Plus d'erreur, la ligne n'a simplement pas été insérée.


# Modification

Si l'on essaye de modifier l'espèce des chats, pour lui donner comme nom latin _Canis canis_, une erreur sera déclenchée, sauf si l'on ajoute l'option `IGNORE`.


```sql
UPDATE Espece SET nom_latin = 'Canis canis' WHERE nom_courant = 'Chat';
```


```console
ERROR 1062 (23000): Duplicate entry 'Canis canis' for key 'nom_latin'
```


```sql
UPDATE IGNORE Espece SET nom_latin = 'Canis canis' WHERE nom_courant = 'Chat';
```


```console
Query OK, 0 rows affected (0.01 sec)
```

Les chats sont toujours des "Felix silvestris" !


# LOAD DATA INFILE

La même option est disponible avec la commande `LOAD DATA INFILE`, ce qui est plutôt pratique si vous voulez éviter de devoir traficoter votre fichier suite à une insertion partielle due à une ligne qui ne respecte pas une contrainte d'unicité.


## Syntaxe


```sql
LOAD DATA [LOCAL] INFILE 'nom_fichier' IGNORE    -- IGNORE se place juste avant INTO, comme dans INSERT
INTO TABLE nom_table
[FIELDS
    [TERMINATED BY '\t']
    [ENCLOSED BY '']
    [ESCAPED BY '\\' ]
]
[LINES 
    [STARTING BY '']    
    [TERMINATED BY '\n']
]
[IGNORE nombre LINES]
[(nom_colonne,...)];
```