# Rappels

Nous avons vu qu'il était possible d'afficher des nombres ou des chaînes de caractères avec un simple `SELECT` :

**Exemple** : affichage simple de nombres et de chaînes de caractères.


```sql
SELECT 3, 'Bonjour !';
```




+-+---------+
|3|Bonjour !|
+=+=========+
|3|Bonjour !|
+-+---------+






Vous savez également qu'il est possible de faire diverses opérations mathématiques de la même manière, et que la priorité des opérations est respectée :

**Exemple** : quelques opérations mathématiques.


```sql
SELECT 3+5, 8/3, 10+2/2, (10+2)/2;
```




+---+------+-------+--------+
|3+5|8/3   |10+2/2 |(10+2)/2|
+===+======+=======+========+
|8  |2.6667|11.0000|6.0000  |
+---+------+-------+--------+






## Opérateurs mathématiques


Six opérateurs mathématiques sont utilisables avec MySQL.




+------------+----------------+
|Symbole     |Opération       |
+============+================+
|+           |addition        |
+------------+----------------+
|-           |soustraction    |
+------------+----------------+
|*           |multiplication  |
+------------+----------------+
|/           |division        |
+------------+----------------+
|`DIV`       |division entière|
+------------+----------------+
|`% (ou MOD)`|modulo          |
+------------+----------------+





Pour ceux qui ne le sauraient pas, le **modulo** de a par b est le **reste de la division entière** de _a_ par _b_ (par exemple, le modulo de 13 par 10 vaut 3).

**Exemple** : les six opérateurs mathématiques.


```sql
SELECT 1+1, 4-2, 3*6, 5/2, 5 DIV 2, 5 % 2, 5 MOD 2;
```




+---+---+---+------+-------+-----+-------+
|1+1|4-2|3*6|5/2   |5 DIV 2|5 % 2|5 MOD 2|
+===+===+===+======+=======+=====+=======+
|2  |2  |18 |2.5000|2      |1    |1      |
+---+---+---+------+-------+-----+-------+






# Combiner les données avec des opérations mathématiques

Jusqu'ici, rien de plus simple. Cependant, si l'on veut juste faire 3+6, pas besoin de MySQL, une calculette (ou un cerveau) suffit.
Après tout, dans une base de données, l'important, ce sont les données. Et, il est évidemment possible de faire des opérations mathématiques (et bien d'autres choses) sur les données. 


## Modification de notre base de données

Mais avant de rentrer dans le vif du sujet, je vais vous demander d'ajouter quelque chose à notre base de données. En effet, pour l'instant nous n'avons pas de données numériques, si ce n'est les différents _id_. Or, il n'est pas très intéressant (et ça peut même être dangereux) de manipuler les _id_. 
Par conséquent, nous allons ajouter une colonne _prix_ à nos tables _Espece_ et _Race_, qui contiendra donc le prix à payer pour adopter un animal de telle espèce ou telle race. Cette colonne sera du type `DECIMAL`, avec deux chiffres après la virgule. Elle ne pourra pas contenir de nombres négatifs, et sera donc également `UNSIGNED`.
 
Voici donc les commandes à exécuter :


```sql
ALTER TABLE Race 
ADD COLUMN prix DECIMAL(7,2) UNSIGNED;

ALTER TABLE Espece 
ADD COLUMN prix DECIMAL(7,2) UNSIGNED;

-- Remplissage des colonnes "prix"
UPDATE Espece SET prix = 200 WHERE id = 1;
UPDATE Espece SET prix = 150 WHERE id = 2;
UPDATE Espece SET prix = 140 WHERE id = 3;
UPDATE Espece SET prix = 700 WHERE id = 4;
UPDATE Espece SET prix = 10 WHERE id = 5;
UPDATE Espece SET prix = 75 WHERE id = 6;

UPDATE Race SET prix = 450 WHERE id = 1;
UPDATE Race SET prix = 900 WHERE id = 2;
UPDATE Race SET prix = 950 WHERE id = 3;
UPDATE Race SET prix = 800 WHERE id = 4;
UPDATE Race SET prix = 700 WHERE id = 5;
UPDATE Race SET prix = 1200 WHERE id = 7;
UPDATE Race SET prix = 950 WHERE id = 8;
UPDATE Race SET prix = 600 WHERE id = 9;
```


Le prix de la race sera prioritaire sur le prix de l'espèce. Donc pour un Berger allemand, on ira chercher le prix dans la table _Race_, tandis que pour un bâtard ou une tortue d'Hermann, on prendra le prix de la table _Espece_.


## Opérations sur données sélectionnées

**Exemple 1** : Imaginons que nous voulions savoir le prix à payer pour acheter 3 individus de la même espèce (sans considérer la race). Facile, il suffit de multiplier le prix de chaque espèce par 3.


```sql
SELECT nom_courant, prix*3 AS prix_trio
FROM Espece;
```




+-----------------+---------+
|nom_courant      |prix_trio|
+=================+=========+
|Chien            |600.00   |
+-----------------+---------+
|Chat             |450.00   |
+-----------------+---------+
|Tortue d'Hermann |420.00   |
+-----------------+---------+
|Perroquet amazone|2100.00  |
+-----------------+---------+





**Exemple 2** : toutes les opérations courantes peuvent bien entendu être utilisées.


```sql
SELECT nom_courant, prix, 
       prix+100 AS addition, prix/2 AS division,
       prix-50.5 AS soustraction, prix%3 AS modulo
FROM Espece;
```




+-----------------+------+--------+----------+------------+------+
|nom_courant      |prix  |addition|division  |soustraction|modulo|
+=================+======+========+==========+============+======+
|Chien            |200.00|300.00  |100.000000|149.50      |2.00  |
+-----------------+------+--------+----------+------------+------+
|Chat             |150.00|250.00  |75.000000 |99.50       |0.00  |
+-----------------+------+--------+----------+------------+------+
|Tortue d'Hermann |140.00|240.00  |70.000000 |89.50       |2.00  |
+-----------------+------+--------+----------+------------+------+
|Perroquet amazone|700.00|800.00  |350.000000|649.50      |1.00  |
+-----------------+------+--------+----------+------------+------+






## Modification de données grâce à des opérations mathématiques

Il est tout à fait possible, et souvent fort utile, de modifier des données grâce à des opérations. 

**Exemple** : la commande suivante va augmenter le prix de toutes les races de 35 :


```sql
UPDATE Race 
SET prix = prix + 35;
```

C'est quand même plus élégant que de devoir faire une requête `SELECT` pour savoir le prix actuel des races, calculer ces prix +35, puis faire un `UPDATE` pour chaque race avec le résultat.

Bien entendu, on peut faire ce genre de manipulation également dans une requête de type `INSERT INTO ... SELECT` par exemple. En fait, on peut faire ce genre de manipulation partout.