Nous avons déjà utilisé quelques fonctions dans ce cours. Par exemple, dans le chapitre sur les sous-requêtes, je vous proposais la requête suivante :


```sql
SELECT MIN(date_naissance)     -- On utilise ici une fonction !
FROM (
    SELECT Animal.id, Animal.sexe, Animal.date_naissance, Animal.nom, Animal.espece_id
    FROM Animal
    INNER JOIN Espece
        ON Espece.id = Animal.espece_id
    WHERE Espece.nom_courant IN ('Tortue d''Hermann', 'Perroquet amazone')
) AS tortues_perroquets;
```


Lorsque l'on fait `MIN(date_naissance)`, on **appelle** la **fonction** `MIN()`, en lui donnant en **paramètre** la colonne _date_naissance_ (ou plus précisément, les lignes de la colonne _date_naissance_ sélectionnées par la requête).

Détaillons un peu tout ça !


## Une fonction

Une fonction est un code qui effectue une **série d'instructions bien précises** (dans le cas de `MIN()`, ces instructions visent donc à chercher la valeur minimale), et **renvoie le résultat de ces instructions** (la valeur minimale en question).
Une fonction est définie par **son nom** (exemple : `MIN`) et **ses paramètres**.


## Un paramètre

Un paramètre de fonction est une donnée (ou un ensemble de données) que l'on **fournit à la fonction** afin qu'elle puisse effectuer son action. Par exemple, pour `MIN()`, il faut passer un paramètre : les données parmi lesquelles on souhaite récupérer la valeur minimale. Une fonction peut avoir **un ou plusieurs paramètres, ou n'en avoir aucun**. Dans le cas d'une fonction ayant plusieurs paramètres, l'ordre dans lequel on donne ces paramètres est très important.

On parle aussi des **arguments** d'une fonction.


## Appeler une fonction

Lorsque l'on utilise une fonction, on dit qu'on fait appel à celle-ci. Pour appeler une fonction, il suffit donc de donner son nom, suivi des paramètres éventuels entre parenthèses (lesquelles sont obligatoires, même s'il n'y a aucun paramètre).


## Exemples



```sql
-- Fonction sans paramètre
SELECT PI();    -- renvoie le nombre Pi, avec 5 décimales

-- Fonction avec un paramètre
SELECT MIN(prix) AS minimum  -- il est bien sûr possible d'utiliser les alias !
FROM Espece;

-- Fonction avec plusieurs paramètres
SELECT REPEAT('fort ! Trop ', 4);  -- répète une chaîne (ici : 'fort ! Trop ', répété 4 fois)

-- Même chose qu'au-dessus, mais avec les paramètres dans le mauvais ordre
SELECT REPEAT(4, 'fort ! Trop '); -- la chaîne de caractères 'fort ! Trop ' va être convertie en entier par MySQL, ce qui donne 0. "4" va donc être répété zéro fois...
```




+--------+
|PI()    |
+========+
|3.141593|
+--------+







+-------+
|minimum|
+=======+
|140.00 |
+-------+







+-----------------------------------------------+
|REPEAT('fort ! Trop ', 4)                      |
+===============================================+
|fort ! Trop fort ! Trop fort ! Trop fort ! Trop|
+-----------------------------------------------+







+-------------------------+
|REPEAT(4, 'fort ! Trop ')|
+=========================+
|                         |
+-------------------------+






[[attention]]
| Il n'y a pas d'espace entre le nom de la fonction et la parenthèse ouvrante !



# Fonctions scalaires _vs_ fonctions d'agrégation

On peut distinguer deux types de fonctions : les fonctions scalaires et les fonctions d'agrégation (ou fonctions de groupement). Les **fonctions scalaires** s'appliquent à **chaque ligne indépendamment**, tandis que les **fonctions d'agrégation regroupent les lignes** (par défaut, elles regroupent toutes les lignes en une seule). Un petit exemple rendra cette explication lumineuse.


## Fonction scalaire

La fonction `ROUND(X)` arrondit _X_ à l'entier le plus proche. Il s'agit d'une fonction scalaire.


```sql
SELECT nom, prix, ROUND(prix)
FROM Race;
```




+-------------------+-------+-----------+
|nom                |prix   |ROUND(prix)|
+===================+=======+===========+
|Berger allemand    |485.00 |485        |
+-------------------+-------+-----------+
|Berger blanc suisse|935.00 |935        |
+-------------------+-------+-----------+
|Singapura          |985.00 |985        |
+-------------------+-------+-----------+
|Bleu russe         |835.00 |835        |
+-------------------+-------+-----------+
|Maine coon         |735.00 |735        |
+-------------------+-------+-----------+
|Sphynx             |1235.00|1235       |
+-------------------+-------+-----------+
|Nebelung           |985.00 |985        |
+-------------------+-------+-----------+





Il y a sept races dans ma table, et lorsqu'on applique la fonction `ROUND(X)` à la colonne _prix_, on récupère bien sept lignes.


## Fonction d'agrégation

La fonction `MIN(X)` par contre, est une fonction d'agrégation.


```sql
SELECT MIN(prix)
FROM Race;
```




+---------+
|MIN(prix)|
+=========+
|485.00   |
+---------+





On ne récupère qu'une seule ligne de résultat. Les sept races ont été regroupées (ça n'aurait d'ailleurs pas de sens d'avoir une ligne par race).
Cette particularité des fonctions d'agrégation les rend un peu plus délicates à utiliser, mais offre des possibilités vraiment intéressantes. Nous commencerons donc en douceur avec les fonctions scalaires pour consacrer ensuite plusieurs chapitres à l'utilisation des fonctions de groupement.