Voici donc quelques fonctions scalaires qui vont vous permettre de manipuler les nombres : faire des calculs, des arrondis, prendre un nombre au hasard, etc.


[[information]]
| Toutes les fonctions de cette partie retournent `NULL` en cas d'erreur !



# Arrondis

Arrondir un nombre, c'est trouver une valeur proche de ce nombre avec une précision donnée et selon certains critères.
La précision est en général représentée par le nombre de décimales désirées. Par exemple, pour un prix, on travaille rarement avec plus de deux décimales. Pour un âge, on préférera généralement un nombre entier (c'est-à-dire aucune décimale).
Quant aux critères, il s'agit de décider si l'on veut arrondir au plus proche (ex. : 4,3 arrondi à l'entier le plus proche vaut 4), arrondir au supérieur (ex. : 4,3 arrondi à l'entier supérieur vaut 5) ou arrondir à l'inférieur (ex. : 4,3 arrondi à l'entier inférieur vaut 4).

Voici quatre fonctions permettant d'arrondir les nombres, selon ces différents critères.
Pour les paramètres, _n_ représente le nombre à arrondir, _d_ le nombre de décimales désirées.


## CEIL()

`CEIL(n)` ou `CEILING(n)` arrondit au nombre entier supérieur.


```sql
SELECT CEIL(3.2), CEIL(3.7);
```




+---------+---------+
|CEIL(3.2)|CEIL(3.7)|
+=========+=========+
|4        |4        |
+---------+---------+






## FLOOR()

`FLOOR(n)` arrondit au nombre entier inférieur.


```sql
SELECT FLOOR(3.2), FLOOR(3.7);
```




+----------+----------+
|FLOOR(3.2)|FLOOR(3.7)|
+==========+==========+
|3         |3         |
+----------+----------+






## ROUND()

`ROUND(n, d)` arrondit au nombre à _d_ décimales le plus proche. `ROUND(n)` équivaut à écrire `ROUND(n, 0)`, donc arrondit à l'entier le plus proche.


```sql
SELECT ROUND(3.22, 1), ROUND(3.55, 1), ROUND(3.77, 1);

SELECT ROUND(3.2), ROUND(3.5), ROUND(3.7);
```




+--------------+--------------+--------------+
|ROUND(3.22, 1)|ROUND(3.55, 1)|ROUND(3.77, 1)|
+==============+==============+==============+
|3.2           |3.6           |3.8           |
+--------------+--------------+--------------+







+----------+----------+----------+
|ROUND(3.2)|ROUND(3.5)|ROUND(3.7)|
+==========+==========+==========+
|3         |4         |4         |
+----------+----------+----------+





Si le nombre se trouve juste entre l'arrondi supérieur et l'arrondi inférieur (par exemple si on arrondit 3,5 à un nombre entier), certaines implémentations vont arrondir vers le haut, d'autres vers le bas. Testez donc pour savoir dans quel cas est votre serveur (comme vous voyez, ici, c'est vers le haut).


## TRUNCATE()

`TRUNCATE(n, d)` arrondit en enlevant purement et simplement les décimales en trop (donc arrondi à l'inférieur pour les nombres positifs, au supérieur pour les nombres négatifs).


```sql
SELECT TRUNCATE(3.2, 0), TRUNCATE(3.5, 0), TRUNCATE(3.7, 0);

SELECT TRUNCATE(3.22, 1), TRUNCATE(3.55, 1), TRUNCATE(3.77, 1);
```




+----------------+----------------+----------------+
|TRUNCATE(3.2, 0)|TRUNCATE(3.5, 0)|TRUNCATE(3.7, 0)|
+================+================+================+
|3               |3               |3               |
+----------------+----------------+----------------+







+-----------------+-----------------+-----------------+
|TRUNCATE(3.22, 1)|TRUNCATE(3.55, 1)|TRUNCATE(3.77, 1)|
+=================+=================+=================+
|3.2              |3.5              |3.7              |
+-----------------+-----------------+-----------------+






# Exposants et racines


## Exposants

`POWER(n, e)` (ou `POW(n, e)`) retourne le résultat de _n_ exposant _e_ ($n^e$).

Pour rappel,  _n_ exposant _e_ ($n^e$) veut dire que l'on multiplie _n_ par lui-même, _e_ fois. Donc par exemple, $2^3 = 2\times2\times2 = 8$


```sql
SELECT POW(2, 5), POWER(5, 2);
```




+---------+-----------+
|POW(2, 5)|POWER(5, 2)|
+=========+===========+
|32       |25         |
+---------+-----------+






## Racines

Prendre la racine n^ième^ d'un nombre _x_ ($\sqrt[n]{x}$), c'est trouver le (ou les) nombre(s) _y_ qui répond(ent) à la condition suivante : $y^n = x$. Donc la racine cinquième de 32 ($\sqrt[5]{32}$) vaut 2, puisque $2^5 = 2\times2\times2\times2\times2 = 32$

`SQRT(n)` donne la racine carrée positive de _n_ ($\sqrt{n} = \sqrt[2]{n}$).


```sql
SELECT SQRT(4);
```




+-------+
|SQRT(4)|
+=======+
|2      |
+-------+





Il n'existe pas de fonction particulière pour obtenir la racine n^ième^ d'un nombre pour _n_ > 2. Cependant, pour ceux qui ne le sauraient pas : $\sqrt[n]{x} = x^{\frac{1}{n}}$. Donc, pour obtenir par exemple la racine cinquième de 32, il suffit de faire :


```sql
SELECT POW(32, 1/5);
```




+------------+
|POW(32, 1/5)|
+============+
|2           |
+------------+






# Hasard

Le "vrai" hasard n'existe pas en informatique. Il est cependant possible de simuler le hasard, avec par exemple un générateur de nombres aléatoires. MySQL implémente un générateur de nombres aléatoires auquel vous pouvez faire appel en utilisant la fonction `RAND()`, qui retourne un nombre "aléatoire" entre 0 et 1.


```sql
SELECT RAND();
```




+-------------------+
|RAND()             |
+===================+
|0.08678611469155748|
+-------------------+





Cette fonction peut par exemple être utile pour trier des résultats de manière aléatoire. Un nombre différent va en effet être généré pour chaque ligne de résultat. Le tri se fera alors sur ce nombre généré "au hasard".
Notez que ce n'est pas très bon en termes de performances, étant donné qu'un nombre doit être généré pour chaque ligne.


```sql
SELECT * 
FROM Race 
ORDER BY RAND();
```



# Divers


## SIGN()

`SIGN(n)` renvoie le signe du nombre _n_. Ou plus exactement, `SIGN(n)` renvoie -1 si _n_ est négatif, 0 si _n_ vaut 0, et 1 si _n_ est positif.


```sql
SELECT SIGN(-43), SIGN(0), SIGN(37);
```




+---------+-------+--------+
|SIGN(-43)|SIGN(0)|SIGN(37)|
+=========+=======+========+
|-1       |0      |1       |
+---------+-------+--------+






## ABS()

`ABS(n)` retourne la valeur absolue de _n_, donc sa valeur sans le signe.


```sql
SELECT ABS(-43), ABS(0), ABS(37);
```




+--------+------+-------+
|ABS(-43)|ABS(0)|ABS(37)|
+========+======+=======+
|43      |0     |37     |
+--------+------+-------+






## MOD()

La fonction `MOD(n, div)` retourne le modulo, donc le reste de la division entière de _n_ par _div_ (comme l'opérateur `%` ou `MOD`)


```sql
SELECT MOD(56, 10);
```




+-----------+
|MOD(56, 10)|
+===========+
|6          |
+-----------+