Nous allons maintenant manipuler, triturer, examiner des chaînes de caractères. À toutes fins utiles, je rappelle en passant qu'en SQL, les chaînes de caractères sont entourées de guillemets (simples, mais les guillemets doubles fonctionnent avec MySQL également).


# Longueur et comparaison


## Connaître la longueur d'une chaîne

Trois fonctions permettent d'avoir des informations sur la longueur d'une chaîne de caractères. Chaque fonction calcule la longueur d'une manière particulière.



- `BIT_LENGTH(chaine)` : retourne le nombre de bits de la chaîne. Chaque caractère est encodé sur un ou plusieurs octets, chaque octet représente huit bits.
- `CHAR_LENGTH(chaine)` : (ou `CHARACTER_LENGTH()`) retourne le nombre de caractères de la chaîne.
- `LENGTH(chaine)` : retourne le nombre d'octets de la chaîne. Chaque caractère étant représenté par un ou plusieurs octets (rappel : les caractères que vous pouvez utiliser dépendent de l'encodage choisi).



```sql
SELECT BIT_LENGTH('élevage'), 
       CHAR_LENGTH('élevage'), 
       LENGTH('élevage'); -- Les caractères accentués sont codés sur 2 octets en UTF-8
```




+---------------------+----------------------+-----------------+
|BIT_LENGTH('élevage')|CHAR_LENGTH('élevage')|LENGTH('élevage')|
+=====================+======================+=================+
|64                   |7                     |8                |
+---------------------+----------------------+-----------------+





A priori, dans neuf cas sur dix au moins, la fonction qui vous sera utile est donc `CHAR_LENGTH()`.


## Comparer deux chaînes

La fonction `STRCMP(chaine1, chaine2)` compare les deux chaînes passées en paramètres et retourne 0 si les chaînes sont les mêmes, -1 si la première chaîne est classée avant dans l'ordre alphabétique et 1 dans le cas contraire.


```sql
SELECT STRCMP('texte', 'texte') AS 'texte=texte', 
       STRCMP('texte','texte2') AS 'texte<texte2', 
       STRCMP('chaine','texte') AS 'chaine<texte', 
       STRCMP('texte', 'chaine') AS 'texte>chaine',
       STRCMP('texte3','texte24') AS 'texte3>texte24'; -- 3 est après 24 dans l'ordre alphabétique
```




+-----------+-------------------------------+-------------------------------+------------+--------------+
|texte=texte|texte<texte2                   |chaine<texte                   |texte>chaine|texte3>texte24|
+===========+===============================+===============================+============+==============+
|0          |-1                             |-1                             |1           |1             |
+-----------+-------------------------------+-------------------------------+------------+--------------+






# Retrait et ajout de caractères


## Répéter une chaîne

`REPEAT(c, n)` retourne le texte _c_, _n_ fois.


```sql
SELECT REPEAT('Ok ', 3);
```




+----------------+
|REPEAT('Ok ', 3)|
+================+
|Ok Ok Ok        |
+----------------+






## Compléter/réduire une chaîne

Les fonctions `LPAD()` et `RPAD()` appliquées à une chaîne de caractères retournent cette chaîne en lui donnant une longueur particulière, donnée en paramètre. Si la chaîne de départ est trop longue, elle sera raccourcie, si elle est trop courte, des caractères seront ajoutés, à gauche de la chaîne pour `LPAD()`, à droite pour `RPAD()`.
Ces fonctions nécessitent trois paramètres : la chaîne à transformer (_texte_), la longueur désirée (_long_), et le caractère à ajouter si la chaîne est trop courte (_caract_).


```sql
LPAD(texte, long, caract)
RPAD(texte, long, caract)
```


```sql
SELECT LPAD('texte', 3, '@') AS '3_gauche_@', 
       LPAD('texte', 7, '$') AS '7_gauche_$', 
       RPAD('texte', 5, 'u') AS '5_droite_u', 
       RPAD('texte', 7, '*') AS '7_droite_*', 
       RPAD('texte', 3, '-') AS '3_droite_-';
```




+----------+-----------+----------+----------+----------+
|3_gauche_@|7_gauche_\$|5_droite_u|7_droite_*|3_droite_-|
+==========+===========+==========+==========+==========+
|tex       |\$\$texte  |texte     |texte**   |tex       |
+----------+-----------+----------+----------+----------+






## Ôter les caractères inutiles


Il peut arriver que certaines de vos données aient des caractères inutiles ajoutés avant et/ou après le texte intéressant. Dans ce cas, il vous est possible d'utiliser la fonction `TRIM()`, qui va supprimer tous ces caractères. Cette fonction a une syntaxe un peu particulière que voici :


```sql
TRIM([[BOTH | LEADING | TRAILING] [caract] FROM] texte);
```

Je rappelle que ce qui est entre crochets est facultatif.
On peut donc choisir entre trois options : `BOTH`, `LEADING` et `TRAILING`.


- `BOTH` : les caractères seront éliminés à l'avant, et à l'arrière du texte
- `LEADING` : seuls les caractères à l'avant de la chaîne seront supprimés
- `TRAILING` : seuls les caractères à l'arrière de la chaîne seront supprimés


Si aucune option n'est précisée, c'est `BOTH` qui est utilisé par défaut.

_caract_ est la chaîne de caractères (ou le caractère unique) à éliminer en début et/ou fin de chaîne. Ce paramètre est facultatif : par défaut les espaces blancs seront supprimées. Et _texte_ est bien sûr la chaîne de caractères à traiter.


```sql
SELECT TRIM('   Tralala  ') AS both_espace, 
       TRIM(LEADING FROM '   Tralala  ') AS lead_espace, 
       TRIM(TRAILING FROM '   Tralala  ') AS trail_espace,

       TRIM('e' FROM 'eeeBouHeee') AS both_e,
       TRIM(LEADING 'e' FROM 'eeeBouHeee') AS lead_e,
       TRIM(BOTH 'e' FROM 'eeeBouHeee') AS both_e,

       TRIM('123' FROM '1234ABCD4321') AS both_123;
```




+-----------+-----------+------------+------+-------+------+---------+
|both_espace|lead_espace|trail_espace|both_e|lead_e |both_e|both_123 |
+===========+===========+============+======+=======+======+=========+
|Tralala    |Tralala    |Tralala     |BouH  |BouHeee|BouH  |4ABCD4321|
+-----------+-----------+------------+------+-------+------+---------+






## Récupérer une sous-chaîne

La fonction `SUBSTRING()` retourne une partie d'une chaîne de caractères. Cette partie est définie par un ou deux paramètres : _pos_ (obligatoire), qui donne la position de début de la sous-chaîne, et _long_ (facultatif) qui donne la longueur de la sous-chaîne désirée (si ce paramètre n'est pas précisé, toute la fin de la chaîne est prise).
Quatre syntaxes sont possibles :


- `SUBSTRING(chaine, pos)`
- `SUBSTRING(chaine FROM pos)`
- `SUBSTRING(chaine, pos, long)`
- `SUBSTRING(chaine FROM pos FOR long)`



```sql
SELECT SUBSTRING('texte', 2) AS from2,
        SUBSTRING('texte' FROM 3) AS from3,
        SUBSTRING('texte', 2, 3) AS from2long3, 
        SUBSTRING('texte' FROM 3 FOR 1) AS from3long1;
```




+-----+-----+----------+----------+
|from2|from3|from2long3|from3long1|
+=====+=====+==========+==========+
|exte |xte  |ext       |x         |
+-----+-----+----------+----------+






# Recherche et remplacement


## Rechercher une chaîne de caractères

`INSTR()`, `LOCATE()` et `POSITION()` retournent la position de la première occurrence d'une chaîne de caractères _rech_ dans une chaîne de caractères _chaine_. Ces trois fonctions ont chacune une syntaxe particulière :


- `INSTR(chaine, rech)`
- `LOCATE(rech, chaine)` - les paramètres sont inversés par rapport à `INSTR()`
- `POSITION(rech IN chaine)`


Si la chaîne de caractères _rech_ n'est pas trouvée dans _chaine_, ces fonctions retournent 0. Par conséquent, la première lettre d'une chaîne de caractères est à la position 1 (alors que dans beaucoup de langages de programmation, on commence toujours à la position 0).

`LOCATE()` peut aussi accepter un paramètre supplémentaire : _pos_, qui définit la position dans la chaîne à partir de laquelle il faut rechercher _rech_ : `LOCATE(rech, chaine, pos)`.


```sql
SELECT INSTR('tralala', 'la') AS fct_INSTR,
       POSITION('la' IN 'tralala') AS fct_POSITION,
       LOCATE('la', 'tralala') AS fct_LOCATE,
       LOCATE('la', 'tralala', 5) AS fct_LOCATE2;
```




+---------+------------+----------+-----------+
|fct_INSTR|fct_POSITION|fct_LOCATE|fct_LOCATE2|
+=========+============+==========+===========+
|4        |4           |4         |6          |
+---------+------------+----------+-----------+






## Changer la casse des chaînes

Les fonctions `LOWER(chaine)` et `LCASE(chaine)` mettent toutes les lettres de _chaine_ en minuscules, tandis que `UPPER(chaine)` et `UCASE(chaine)` mettent toutes les lettres en majuscules.


```sql
SELECT LOWER('AhAh') AS minuscule, 
        LCASE('AhAh') AS minuscule2, 
        UPPER('AhAh') AS majuscule,
        UCASE('AhAh') AS majuscule2;
```




+---------+----------+---------+----------+
|minuscule|minuscule2|majuscule|majuscule2|
+=========+==========+=========+==========+
|ahah     |ahah      |AHAH     |AHAH      |
+---------+----------+---------+----------+






## Récupérer la partie gauche ou droite

`LEFT(chaine, long)` retourne les _long_ premiers caractères de _chaine_ en partant de la gauche, et `RIGHT(chaine, long)` fait la même chose en partant de la droite.


```sql
SELECT LEFT('123456789', 5), RIGHT('123456789', 5);
```




+--------------------+---------------------+
|LEFT('123456789', 5)|RIGHT('123456789', 5)|
+====================+=====================+
|12345               |56789                |
+--------------------+---------------------+






## Inverser une chaîne

`REVERSE(chaine)` renvoie _chaine_ en inversant les caractères.


```sql
SELECT REVERSE('abcde');
```




+----------------+
|REVERSE('abcde')|
+================+
|edcba           |
+----------------+






## Remplacer une partie par autre chose

Deux fonctions permettent de remplacer une partie d'une chaîne de caractères : `INSERT()` et `REPLACE()`.


- `INSERT(chaine, pos, long, nouvCaract)`: le paramètre _chaine_ est la chaîne de caractères dont on veut remplacer une partie, _pos_ est la position du premier caractère à remplacer, _long_ le nombre de caractères à remplacer, et _nouvCaract_ est la chaîne de caractères qui viendra remplacer la portion de _chaine_ choisie.
- `REPLACE(chaine, ancCaract, nouvCaract)` : tous les caractères (ou sous-chaînes) _ancCaract_ seront remplacés par _nouvCaract._



```sql
SELECT INSERT('texte', 3, 2, 'blabla') AS fct_INSERT, 
        REPLACE('texte', 'e', 'a') AS fct_REPLACE, 
        REPLACE('texte', 'ex', 'ou') AS fct_REPLACE2;
```




+----------+-----------+------------+
|fct_INSERT|fct_REPLACE|fct_REPLACE2|
+==========+===========+============+
|teblablae |taxta      |toute       |
+----------+-----------+------------+






# Concaténation

Concaténer deux chaînes de caractères signifie les mettre bout à bout pour n'en faire qu'une seule. Deux fonctions scalaires permettent la concaténation : `CONCAT()` et `CONCAT_WS()`. Ces deux fonctions permettent de concaténer autant de chaînes que vous voulez, il suffit de toutes les passer en paramètres. Par conséquent, ces deux fonctions n'ont pas un nombre de paramètres défini.


- `CONCAT(chaine1, chaine2,…)` : renvoie simplement une chaîne de caractères, résultat de la concaténation de toutes les chaînes passées en paramètres.
- `CONCAT_WS(separateur, chaine1, chaine2)` : même chose que `CONCAT()`, sauf que la première chaîne passée sera utilisée comme séparateur, donc placée entre chacune des autres chaînes passées en paramètres.



```sql
SELECT CONCAT('My', 'SQL', '!'), CONCAT_WS('-', 'My', 'SQL', '!');
```




+------------------------+--------------------------------+
|CONCAT('My', 'SQL', '!')|CONCAT_WS('-', 'My', 'SQL', '!')|
+========================+================================+
|MySQL!                  |My-SQL-!                        |
+------------------------+--------------------------------+






# FIELD(), une fonction bien utile pour le tri

La fonction `FIELD(rech, chaine1, chaine2, chaine3,…)` recherche le premier argument (_rech_) parmi les arguments suivants (_chaine1, chaine2, chaine3,…_) et retourne l'index auquel _rech_ est trouvée (1 si _rech_ = _chaine1_, 2 si _rech_ = _chaine2_,…). Si _rech_ n'est pas trouvée parmi les arguments, 0 est renvoyé.


```sql
SELECT FIELD('Bonjour', 'Bonjour !', 'Au revoir', 'Bonjour', 'Au revoir !') AS field_bonjour;
```




+-------------+
|field_bonjour|
+=============+
|3            |
+-------------+





Par conséquent, `FIELD` peut être utilisée pour définir un ordre arbitraire dans une clause `ORDER BY`.

**Exemple** : ordonnons les espèces selon un ordre arbitraire. La fonction `FIELD()` dans la clause `SELECT` n'est là que pour illustrer la façon dont ce tri fonctionne.


```sql
SELECT nom_courant, nom_latin, FIELD(nom_courant, 'Rat brun', 'Chat', 'Tortue d''Hermann', 'Chien', 'Perroquet amazone') AS resultat_field
FROM Espece
ORDER BY FIELD(nom_courant, 'Rat brun', 'Chat', 'Tortue d''Hermann', 'Chien', 'Perroquet amazone');
```




+-----------------+---------------------+--------------+
|nom_courant      |nom_latin            |resultat_field|
+=================+=====================+==============+
|Rat brun         |Rattus norvegicus    |1             |
+-----------------+---------------------+--------------+
|Chat             |Felis silvestris     |2             |
+-----------------+---------------------+--------------+
|Tortue d'Hermann |Testudo hermanni     |3             |
+-----------------+---------------------+--------------+
|Chien            |Canis canis          |4             |
+-----------------+---------------------+--------------+
|Perroquet amazone|Alipiopsitta xanthops|5             |
+-----------------+---------------------+--------------+






[[information]]
| Si vous ne mettez pas toutes les valeurs existantes de la colonne en argument de `FIELD()`, les lignes ayant les valeurs non mentionnées seront classées en premier (puisque `FIELD()` renverra 0).


# Code ASCII


Les deux dernières fonctions que nous allons voir sont `ASCII()` et `CHAR()`, qui sont complémentaires. `ASCII(chaine)` renvoie le code ASCII du premier caractère de la chaîne passée en paramètre, tandis que `CHAR(ascii1, ascii2,…)` retourne les caractères correspondant aux codes ASCII passés en paramètres (autant de paramètres qu'on veut). Les arguments passés à `CHAR()` seront convertis en entiers par MySQL.


```sql
SELECT ASCII('T'), CHAR(84), CHAR('84', 84+32, 84.2);
```




+----------+--------+-----------------------+
|ASCII('T')|CHAR(84)|CHAR('84', 84+32, 84.2)|
+==========+========+=======================+
|84        |T       |TtT                    |
+----------+--------+-----------------------+