La plupart des fonctions d'agrégation vont vous permettre de faire des statistiques sur vos données.


# Nombre de lignes

La fonction `COUNT()` permet de savoir combien de lignes sont sélectionnées par la requête.


```sql
-- Combien de races avons-nous ? --
-- ---------------------------------
SELECT COUNT(*) AS nb_races
FROM Race;

-- Combien de chiens avons-nous ? --
-- ---------------------------------
SELECT COUNT(*) AS nb_chiens
FROM Animal 
INNER JOIN Espece ON Espece.id = Animal.espece_id
WHERE Espece.nom_courant = 'Chien';
```




+--------+
|nb_races|
+========+
|8       |
+--------+







+---------+
|nb_chiens|
+=========+
|21       |
+---------+






## COUNT(*) ou COUNT(colonne)

Vous l'avez vu, j'ai utilisé `COUNT(*)` dans les exemples ci-dessus. Cela signifie que l'on compte tout simplement les lignes, sans se soucier de ce qu'elles contiennent.

Par contre, si on utilise `COUNT(colonne)`, seules les lignes dont la valeur de _colonne_ n'est pas `NULL` seront prises en compte. 

**Exemple** : comptons les lignes de la table _Animal_, avec `COUNT(*)` et `COUNT(race_id)`.


```sql
SELECT COUNT(race_id), COUNT(*)
FROM Animal;
```




+--------------+--------+
|COUNT(race_id)|COUNT(*)|
+==============+========+
|31            |60      |
+--------------+--------+





Il n'y a donc que 31 animaux sur nos 60 pour lesquels la race est définie.


## Doublons

Comme dans une requête `SELECT` tout à fait banale, il est possible d'utiliser le mot-clé `DISTINCT` pour ne pas prendre en compte les doublons.

**Exemple** : comptons le nombre de races distinctes définies dans la table _Animal_.


```sql
SELECT COUNT(DISTINCT race_id)
FROM Animal;
```




+-----------------------+
|COUNT(DISTINCT race_id)|
+=======================+
|7                      |
+-----------------------+





Parmi nos 31 animaux dont la race est définie, on trouve donc 7 races différentes.


# Minimum et maximum

Nous avons déjà eu l'occasion de rencontrer la fonction `MIN(x)`, qui retourne la plus petite valeur de _x_. Il existe également une fonction `MAX(x)`, qui renvoie la plus grande valeur de _x_.


```sql
SELECT MIN(prix), MAX(prix)
FROM Race;
```




+---------+---------+
|MIN(prix)|MAX(prix)|
+=========+=========+
|485.00   |1235.00  |
+---------+---------+





Notez que `MIN()` et `MAX()` ne s'utilisent pas uniquement sur des données numériques. Si vous lui passez des chaînes de caractères, `MIN()` récupérera la première chaîne dans l'ordre alphabétique, `MAX()` la dernière ; avec des dates, `MIN()` renverra la plus vieille et `MAX()` la plus récente.

**Exemple** :


```sql
SELECT MIN(nom), MAX(nom), MIN(date_naissance), MAX(date_naissance)
FROM Animal;
```




+--------+--------+-------------------+-------------------+
|MIN(nom)|MAX(nom)|MIN(date_naissance)|MAX(date_naissance)|
+========+========+===================+===================+
|Anya    |Zonko   |2006-03-15 14:26:00|2010-11-09 00:00:00|
+--------+--------+-------------------+-------------------+






# Somme et moyenne


## Somme

La fonction `SUM(x)` renvoie la somme de _x_.


```sql
SELECT SUM(prix)
FROM Espece;
```




+---------+
|SUM(prix)|
+=========+
|1200.00  |
+---------+






## Moyenne

La fonction `AVG(x)` (du mot anglais _average_) renvoie la valeur moyenne de _x_.


```sql
SELECT AVG(prix)
FROM Espece;
```




+----------+
|AVG(prix) |
+==========+
|240.000000|
+----------+