# Principe

Avec les fonctions d'agrégation, on regroupe plusieurs lignes. Les fonctions statistiques nous permettent d'avoir des informations fort utiles sur le résultat d'une requête, mais parfois, il est intéressant d'avoir également les valeurs concernées.
Ceci est faisable avec `GROUP_CONCAT(nom_colonne)`. Cette fonction concatène les valeurs de _nom_colonne_ pour chaque groupement réalisé.

**Exemple** : on récupère la somme des prix de chaque espèce, et on affiche les espèces concernées par la même occasion.


```sql
SELECT SUM(prix), GROUP_CONCAT(nom_courant)
FROM Espece;
```




+---------+------------------------------------------------------+
|SUM(prix)|GROUP_CONCAT(nom_courant)                             |
+=========+======================================================+
|1200.00  |Chien,Chat,Tortue d'Hermann,Perroquet amazone,Rat brun|
+---------+------------------------------------------------------+






# Syntaxe

Voici la syntaxe de cette fonction :


```sql
GROUP_CONCAT(
              [DISTINCT] col1 [, col2, ...]
              [ORDER BY col [ASC | DESC]]
              [SEPARATOR sep]
            )
```




- `DISTINCT` : sert comme d'habitude à éliminer les doublons.
- `col1` : est le nom de la colonne dont les valeurs doivent être concaténées. C'est le **seul argument obligatoire**.
- `col2,…` : sont les éventuelles autres colonnes (ou chaînes de caractères) à concaténer.
- `ORDER BY` : permet de déterminer dans quel ordre les valeurs seront concaténées.
- `SEPARATOR` : permet de spécifier une chaîne de caractères à utiliser pour séparer les différentes valeurs. Par défaut, c'est une virgule.




# Exemples



```sql
-- --------------------------------------
-- CONCATENATION DE PLUSIEURS COLONNES --
-- --------------------------------------
SELECT SUM(Race.prix), GROUP_CONCAT(Race.nom, Espece.nom_courant)
FROM Race
INNER JOIN Espece ON Espece.id = Race.espece_id;

-- ---------------------------------------------------
-- CONCATENATION DE PLUSIEURS COLONNES EN PLUS JOLI --
-- ---------------------------------------------------
SELECT SUM(Race.prix), GROUP_CONCAT(Race.nom, ' (', Espece.nom_courant, ')')
FROM Race
INNER JOIN Espece ON Espece.id = Race.espece_id;

-- ---------------------------
-- ELIMINATION DES DOUBLONS --
-- ---------------------------
SELECT SUM(Espece.prix), GROUP_CONCAT(DISTINCT Espece.nom_courant) -- Essayez sans le DISTINCT pour voir
FROM Espece
INNER JOIN Race ON Race.espece_id = Espece.id;

-- --------------------------
-- UTILISATION DE ORDER BY --
-- --------------------------
SELECT SUM(Race.prix), GROUP_CONCAT(Race.nom, ' (', Espece.nom_courant, ')' ORDER BY Race.nom DESC)
FROM Race
INNER JOIN Espece ON Espece.id = Race.espece_id;

-- ----------------------------
-- CHANGEMENT DE SEPARATEUR  --
-- ----------------------------
SELECT SUM(Race.prix), GROUP_CONCAT(Race.nom, ' (', Espece.nom_courant, ')' SEPARATOR ' - ')
FROM Race
INNER JOIN Espece ON Espece.id = Race.espece_id;
```