J'ai mentionné le fait qu'il était possible de grouper sur plusieurs colonnes, mais jusqu'à maintenant, cela n'a servi qu'à pouvoir afficher correctement les colonnes voulues, sans que ça n'influe sur les groupes. On n'avait donc en fait qu'un seul critère, représenté par plusieurs colonnes. Voyons maintenant un exemple avec deux critères différents (qui ne créent pas les mêmes groupes).

Les deux requêtes suivantes permettent de savoir combien d'animaux de chaque espèce vous avez dans la table _Animal_, ainsi que combien de mâles et de femelles, toutes espèces confondues.

```sql
SELECT nom_courant, COUNT(*) as nb_animaux
FROM Animal
INNER JOIN Espece ON Espece.id = Animal.espece_id
GROUP BY nom_courant;

SELECT sexe, COUNT(*) as nb_animaux
FROM Animal
GROUP BY sexe;
```




+-----------------+----------+
|nom_courant      |nb_animaux|
+=================+==========+
|Chat             |20        |
+-----------------+----------+
|Chien            |21        |
+-----------------+----------+
|Perroquet amazone|4         |
+-----------------+----------+
|Tortue d'Hermann |15        |
+-----------------+----------+







+----+----------+
|sexe|nb_animaux|
+====+==========+
|NULL|3         |
+----+----------+
|F   |31        |
+----+----------+
|M   |26        |
+----+----------+





En faisant un regroupement multicritère, il est possible de savoir facilement combien de mâles et de femelles **par espèce** il y a dans la table _Animal_. Notez que **l'ordre des critères a son importance**.

**Exemple 1** : on regroupe d'abord sur l'espèce, puis sur le sexe.


```sql
SELECT nom_courant, sexe, COUNT(*) as nb_animaux
FROM Animal
INNER JOIN Espece ON Espece.id = Animal.espece_id
GROUP BY nom_courant, sexe;
```




+-----------------+----+----------+
|nom_courant      |sexe|nb_animaux|
+=================+====+==========+
|Chat             |NULL|2         |
+-----------------+----+----------+
|Chat             |F   |9         |
+-----------------+----+----------+
|Chat             |M   |9         |
+-----------------+----+----------+
|Chien            |F   |11        |
+-----------------+----+----------+
|Chien            |M   |10        |
+-----------------+----+----------+
|Perroquet amazone|F   |1         |
+-----------------+----+----------+
|Perroquet amazone|M   |3         |
+-----------------+----+----------+
|Tortue d'Hermann |NULL|1         |
+-----------------+----+----------+
|Tortue d'Hermann |F   |10        |
+-----------------+----+----------+
|Tortue d'Hermann |M   |4         |
+-----------------+----+----------+





**Exemple 2** : on regroupe d'abord sur le sexe, puis sur l'espèce.


```sql
SELECT nom_courant, sexe, COUNT(*) as nb_animaux
FROM Animal
INNER JOIN Espece ON Espece.id = Animal.espece_id
GROUP BY sexe,nom_courant;
```




+-----------------+----+----------+
|nom_courant      |sexe|nb_animaux|
+=================+====+==========+
|Chat             |NULL|2         |
+-----------------+----+----------+
|Tortue d'Hermann |NULL|1         |
+-----------------+----+----------+
|Chat             |F   |9         |
+-----------------+----+----------+
|Chien            |F   |11        |
+-----------------+----+----------+
|Perroquet amazone|F   |1         |
+-----------------+----+----------+
|Tortue d'Hermann |F   |10        |
+-----------------+----+----------+
|Chat             |M   |9         |
+-----------------+----+----------+
|Chien            |M   |10        |
+-----------------+----+----------+
|Perroquet amazone|M   |3         |
+-----------------+----+----------+
|Tortue d'Hermann |M   |4         |
+-----------------+----+----------+





Étant donné que le regroupement par sexe donnait trois groupes différents, et le regroupement par espèce donnait quatre groupes différents, il peut y avoir jusqu'à douze (3 x 4) groupes lorsque l'on regroupe en se basant sur les deux critères. Ici, il y en aura moins puisque le sexe de tous les chiens et de tous les perroquets est défini (pas de `NULL`).