Il n'est pas possible d'utiliser la clause `WHERE` pour faire des conditions sur une fonction d'agrégation. Donc, si l'on veut afficher les espèces dont on possède plus de 15 individus, la requête suivante ne fonctionnera pas.


```sql
SELECT nom_courant, COUNT(*)
FROM Animal
INNER JOIN Espece ON Espece.id = Animal.espece_id
WHERE COUNT(*) > 15
GROUP BY nom_courant;
```



```console
ERROR 1111 (HY000): Invalid use of group function
```


Il faut utiliser une clause spéciale : `HAVING`. Cette clause se place juste après le `GROUP BY`.


```sql
SELECT nom_courant, COUNT(*)
FROM Animal
INNER JOIN Espece ON Espece.id = Animal.espece_id
GROUP BY nom_courant
HAVING COUNT(*) > 15;
```




+-----------+--------+
|nom_courant|COUNT(*)|
+===========+========+
|Chat       |20      |
+-----------+--------+
|Chien      |21      |
+-----------+--------+





Il est également possible d'utiliser un alias dans une condition `HAVING` :


```sql
SELECT nom_courant, COUNT(*) as nombre
FROM Animal
INNER JOIN Espece ON Espece.id = Animal.espece_id
GROUP BY nom_courant
HAVING nombre > 15;
```




+-----------+------+
|nom_courant|nombre|
+===========+======+
|Chat       |20    |
+-----------+------+
|Chien      |21    |
+-----------+------+






# Optimisation


Les conditions données dans la clause `HAVING` ne doivent pas nécessairement comporter une fonction d'agrégation. Les deux requêtes suivantes donneront par exemple des résultats équivalents :


```sql
SELECT nom_courant, COUNT(*) as nombre
FROM Animal
INNER JOIN Espece ON Espece.id = Animal.espece_id
GROUP BY nom_courant
HAVING nombre > 6 AND SUBSTRING(nom_courant, 1, 1) = 'C'; -- Deux conditions dans HAVING

SELECT nom_courant, COUNT(*) as nombre
FROM Animal
INNER JOIN Espece ON Espece.id = Animal.espece_id
WHERE SUBSTRING(nom_courant, 1, 1) = 'C'                  -- Une condition dans WHERE
GROUP BY nom_courant
HAVING nombre > 6;                                        -- Et une dans HAVING
```


Il est cependant préférable, et de loin, d'utiliser la clause `WHERE` autant que possible, c'est-à-dire pour toutes les conditions, sauf celles utilisant une fonction d'agrégation. En effet, les conditions `HAVING` ne sont absolument pas optimisées, au contraire des conditions `WHERE`.