# Les fonctions

Pour obtenir la date et l'heure actuelles (format `DATETIME`), c'est Byzance : vous avez le choix entre cinq fonctions différentes !


## NOW() et SYSDATE()

`NOW()` est sans doute la fonction MySQL la plus utilisée pour obtenir la date du jour. C'est aussi la plus facile à retenir (bien que les noms des fonctions soient souvent explicites en SQL), puisque _now_ veut dire "maintenant" en anglais. `SYSDATE()` ("system date") est aussi pas mal utilisée.


```sql
SELECT NOW(), SYSDATE();
```




+-------------------+-------------------+
|NOW()              |SYSDATE()          |
+===================+===================+
|2011-10-26 09:40:18|2011-10-26 09:40:18|
+-------------------+-------------------+






## Et les autres

Les trois autres fonctions peuvent s'utiliser avec ou sans parenthèses.


- `CURRENT_TIMESTAMP` ou `CURRENT_TIMESTAMP()`
- `LOCALTIME` ou `LOCALTIME()`
- `LOCALTIMESTAMP` ou `LOCALTIMESTAMP()`



```sql
SELECT LOCALTIME, CURRENT_TIMESTAMP(), LOCALTIMESTAMP;
```




+-------------------+-------------------+-------------------+
|LOCALTIME          |CURRENT_TIMESTAMP()|LOCALTIMESTAMP     |
+===================+===================+===================+
|2011-10-26 10:02:31|2011-10-26 10:02:31|2011-10-26 10:02:31|
+-------------------+-------------------+-------------------+






# Qui peut le plus, peut le moins

Il est tout à fait possible d'utiliser une des fonctions donnant l'heure et la date pour remplir une colonne de type `DATE`, ou de type `TIME`. MySQL convertira simplement le `DATETIME` en `DATE`, ou en `TIME`, en supprimant la partie inutile.


## Exemple

Créons une table de test simple, avec juste trois colonnes. Une de type `DATE`, une de type `TIME`, une de type `DATETIME`. On peut voir que l'insertion d'une ligne en utilisant `NOW()` pour les trois colonnes donne le résultat attendu.


```sql
-- Création d'une table de test toute simple
CREATE TABLE testDate (
    dateActu DATE, 
    timeActu TIME, 
    datetimeActu DATETIME
);

INSERT INTO testDate VALUES (NOW(), NOW(), NOW());

SELECT *
FROM testDate;
```




+----------+--------+-------------------+
|dateActu  |timeActu|datetimeActu       |
+==========+========+===================+
|2011-10-26|11:22:10|2011-10-26 11:22:10|
+----------+--------+-------------------+






# Timestamp Unix

Il existe encore une fonction qui peut donner des informations sur la date et l'heure actuelle, sous forme de timestamp Unix ; ce qui est donc le nombre de secondes écoulées depuis le premier janvier 1970, à 00:00:00. Il s'agit de `UNIX_TIMESTAMP()`.

Je vous la donne au cas où, mais j'espère que vous ne vous en servirez pas pour stocker vos dates sous forme d'`INT` avec un timestamp Unix !


```sql
SELECT UNIX_TIMESTAMP();
```




+----------------+
|UNIX_TIMESTAMP()|
+================+
|1319621754      |
+----------------+