Commençons en douceur avec des fonctions permettant d'extraire une information d'une donnée temporelle. Par exemple, le jour de la semaine, le nom du mois, l'année, etc.


# Informations sur la date


## Extraire la partie DATE

La fonction `DATE(datetime)` permet d'extraire la partie `DATE` d'une donnée de type `DATETIME` (ou `DATE` mais c'est moins utile…).


```sql
SELECT nom, date_naissance, 
        DATE(date_naissance) AS uniquementDate
FROM Animal
WHERE espece_id = 4;
```




+--------+-------------------+--------------+
|nom     |date_naissance     |uniquementDate|
+========+===================+==============+
|Safran  |2007-03-04 19:36:00|2007-03-04    |
+--------+-------------------+--------------+
|Gingko  |2008-02-20 02:50:00|2008-02-20    |
+--------+-------------------+--------------+
|Bavard  |2009-03-26 08:28:00|2009-03-26    |
+--------+-------------------+--------------+
|Parlotte|2009-03-26 07:55:00|2009-03-26    |
+--------+-------------------+--------------+






## Le jour

Les fonctions suivantes donnent des informations sur le jour :


- `DAY(date)` ou `DAYOFMONTH(date)` : donne le jour du mois (sous forme de nombre entier de 1 à 31) ;
- `DAYOFWEEK(date)` : donne l'index du jour de la semaine (nombre de 1 à 7 avec 1 = dimanche, 2 = lundi,…7 = samedi) ;
- `WEEKDAY(date)` : donne aussi l'index du jour de la semaine, de manière un peu différente (nombre de 0 à 6 avec 0 = lundi, 1 = mardi,…6 = dimanche) ;
- `DAYNAME(date)` : donne le nom du jour de la semaine ;
- `DAYOFYEAR(date)` : retourne le numéro du jour par rapport à l'année (de 1 à 366 donc).


**Exemples** :

```sql
SELECT nom, DATE(date_naissance) AS date_naiss, 
        DAY(date_naissance) AS jour, 
        DAYOFMONTH(date_naissance) AS jour, 
        DAYOFWEEK(date_naissance) AS jour_sem,
        WEEKDAY(date_naissance) AS jour_sem2,
        DAYNAME(date_naissance) AS nom_jour, 
        DAYOFYEAR(date_naissance) AS jour_annee
FROM Animal
WHERE espece_id = 4;
```




+--------+----------+----+----+--------+---------+---------+----------+
|nom     |date_naiss|jour|jour|jour_sem|jour_sem2|nom_jour |jour_annee|
+========+==========+====+====+========+=========+=========+==========+
|Safran  |2007-03-04|4   |4   |1       |6        |Sunday   |63        |
+--------+----------+----+----+--------+---------+---------+----------+
|Gingko  |2008-02-20|20  |20  |4       |2        |Wednesday|51        |
+--------+----------+----+----+--------+---------+---------+----------+
|Bavard  |2009-03-26|26  |26  |5       |3        |Thursday |85        |
+--------+----------+----+----+--------+---------+---------+----------+
|Parlotte|2009-03-26|26  |26  |5       |3        |Thursday |85        |
+--------+----------+----+----+--------+---------+---------+----------+





Tout ça fonctionne très bien, mais ce serait encore mieux si l'on pouvait avoir le nom des jours en français plutôt qu'en anglais. Aucun problème, il suffit de le demander, en exécutant la requête suivante :


```sql
SET lc_time_names = 'fr_FR';
```

Et voilà le travail :


```sql
SELECT nom, date_naissance, 
        DAYNAME(date_naissance) AS jour_semaine 
FROM Animal
WHERE espece_id = 4;
```




+--------+-------------------+------------+
|nom     |date_naissance     |jour_semaine|
+========+===================+============+
|Safran  |2007-03-04 19:36:00|dimanche    |
+--------+-------------------+------------+
|Gingko  |2008-02-20 02:50:00|mercredi    |
+--------+-------------------+------------+
|Bavard  |2009-03-26 08:28:00|jeudi       |
+--------+-------------------+------------+
|Parlotte|2009-03-26 07:55:00|jeudi       |
+--------+-------------------+------------+






## La semaine

À partir d'une date, il est possible de calculer la semaine à laquelle correspond celle-ci. S'agit-il de la première semaine de l'année ? De la quinzième ? Ceci peut être obtenu grâce à trois fonctions : `WEEK(date)`, `WEEKOFYEAR(date)` et `YEARWEEK(date)`.


- `WEEK(date)` : donne uniquement le numéro de la semaine (un nombre entre 0 et 53, puisque 7 x 52 = 364, donc en un an, il y a 52 semaines et 1 ou 2 jours d'une 53^e^ semaine).
- `WEEKOFYEAR(date)` : donne uniquement le numéro de la semaine (un nombre entre 1 et 53).
- `YEARWEEK(date)` : donne également l'année.



```sql
SELECT nom, date_naissance, WEEK(date_naissance) AS semaine, WEEKOFYEAR(date_naissance) AS semaine2, YEARWEEK(date_naissance) AS semaine_annee
FROM Animal
WHERE espece_id = 4;
```




+--------+-------------------+-------+--------+-------------+
|nom     |date_naissance     |semaine|semaine2|semaine_annee|
+========+===================+=======+========+=============+
|Safran  |2007-03-04 19:36:00|9      |9       |200709       |
+--------+-------------------+-------+--------+-------------+
|Gingko  |2008-02-20 02:50:00|7      |8       |200807       |
+--------+-------------------+-------+--------+-------------+
|Bavard  |2009-03-26 08:28:00|12     |13      |200912       |
+--------+-------------------+-------+--------+-------------+
|Parlotte|2009-03-26 07:55:00|12     |13      |200912       |
+--------+-------------------+-------+--------+-------------+





`WEEK()` et `YEARWEEK()` peuvent également accepter un deuxième argument, qui sert à spécifier si la semaine doit commencer le lundi ou le dimanche, et ce qu'on considère comme la première semaine de l'année. Selon l'option utilisée par `WEEK()`, le résultat de cette fonction peut donc différer de celui de `WEEKOFYEAR()`. Si ces options vous intéressent, je vous invite à aller vous renseigner dans la documentation officielle.


## Le mois

Pour le mois, il existe deux fonctions  : `MONTH(date)` qui donne le numéro du mois (nombre de 1 à 12) et `MONTHNAME(date)` qui donne le nom du mois.


```sql
SELECT nom, date_naissance, MONTH(date_naissance) AS numero_mois, MONTHNAME(date_naissance) AS nom_mois
FROM Animal
WHERE espece_id = 4;
```




+--------+-------------------+-----------+--------+
|nom     |date_naissance     |numero_mois|nom_mois|
+========+===================+===========+========+
|Safran  |2007-03-04 19:36:00|3          |mars    |
+--------+-------------------+-----------+--------+
|Gingko  |2008-02-20 02:50:00|2          |février |
+--------+-------------------+-----------+--------+
|Bavard  |2009-03-26 08:28:00|3          |mars    |
+--------+-------------------+-----------+--------+
|Parlotte|2009-03-26 07:55:00|3          |mars    |
+--------+-------------------+-----------+--------+






## L'année

Enfin, la fonction `YEAR(date)` extrait l'année.


```sql
SELECT nom, date_naissance, YEAR(date_naissance)
FROM Animal
WHERE espece_id = 4;
```




+--------+-------------------+--------------------+
|nom     |date_naissance     |YEAR(date_naissance)|
+========+===================+====================+
|Safran  |2007-03-04 19:36:00|2007                |
+--------+-------------------+--------------------+
|Gingko  |2008-02-20 02:50:00|2008                |
+--------+-------------------+--------------------+
|Bavard  |2009-03-26 08:28:00|2009                |
+--------+-------------------+--------------------+
|Parlotte|2009-03-26 07:55:00|2009                |
+--------+-------------------+--------------------+






# Informations sur l'heure

En ce qui concerne l'heure, voici quatre fonctions intéressantes (et faciles à retenir) qui s'appliquent à une donnée de type `DATETIME` ou `TIME`:


- `TIME(datetime)` : qui extrait l'heure complète (le `TIME`) ;
- `HOUR(heure)` : qui extrait l'heure ;
- `MINUTE(heure)` : qui extrait les minutes ;
- `SECOND(heure)` : qui extrait les secondes.



```sql
SELECT nom, date_naissance, 
       TIME(date_naissance) AS time_complet, 
       HOUR(date_naissance) AS heure, 
       MINUTE(date_naissance) AS minutes, 
       SECOND(date_naissance) AS secondes
FROM Animal
WHERE espece_id = 4;
```




+--------+-------------------+------------+-----+-------+--------+
|nom     |date_naissance     |time_complet|heure|minutes|secondes|
+========+===================+============+=====+=======+========+
|Safran  |2007-03-04 19:36:00|19:36:00    |19   |36     |0       |
+--------+-------------------+------------+-----+-------+--------+
|Gingko  |2008-02-20 02:50:00|02:50:00    |2    |50     |0       |
+--------+-------------------+------------+-----+-------+--------+
|Bavard  |2009-03-26 08:28:00|08:28:00    |8    |28     |0       |
+--------+-------------------+------------+-----+-------+--------+
|Parlotte|2009-03-26 07:55:00|07:55:00    |7    |55     |0       |
+--------+-------------------+------------+-----+-------+--------+