Avec les fonctions que nous venons de voir, vous êtes maintenant capables d'afficher une date dans un joli format, par exemple "le lundi 8 novembre 1987".


```sql
SELECT nom, date_naissance, CONCAT_WS(' ', 'le', DAYNAME(date_naissance), DAY(date_naissance), MONTHNAME(date_naissance), YEAR(date_naissance)) AS jolie_date
FROM Animal
WHERE espece_id = 4;
```




+--------+-------------------+---------------------------+
|nom     |date_naissance     |jolie_date                 |
+========+===================+===========================+
|Safran  |2007-03-04 19:36:00|le dimanche 4 mars 2007    |
+--------+-------------------+---------------------------+
|Gingko  |2008-02-20 02:50:00|le mercredi 20 février 2008|
+--------+-------------------+---------------------------+
|Bavard  |2009-03-26 08:28:00|le jeudi 26 mars 2009      |
+--------+-------------------+---------------------------+
|Parlotte|2009-03-26 07:55:00|le jeudi 26 mars 2009      |
+--------+-------------------+---------------------------+





Cependant, il faut bien avouer que c'est un peu long à écrire. Heureusement, il existe une fonction qui va nous permettre de faire la même chose, en bien plus court : `DATE_FORMAT(date, format)`.

Cette fonction `DATE_FORMAT()` a donc deux paramètres :


- date : la date à formater (`DATE`, `TIME` ou `DATETIME`) ;
- format : le format voulu.



# Format

Le format à utiliser doit être donné sous forme de chaîne de caractères. Cette chaîne peut contenir un ou plusieurs spécificateurs dont les plus courants sont listés dans le tableau suivant.




+-------------+-------------------------------------------------------------+
|Spécificateur|Description                                                  |
+=============+=============================================================+
|%d           |Jour du mois (nombre à deux chiffres, de 00 à 31)            |
+-------------+-------------------------------------------------------------+
|%e           |Jour du mois (nombre à un ou deux chiffres, de 0 à 31)       |
+-------------+-------------------------------------------------------------+
|%D           |Jour du mois, avec suffixe (1rst, 2nd,…, 31th) **en anglais**|
+-------------+-------------------------------------------------------------+
|%w           |Numéro du jour de la semaine (dimanche = 0,…, samedi = 6)    |
+-------------+-------------------------------------------------------------+
|%W           |Nom du jour de la semaine                                    |
+-------------+-------------------------------------------------------------+
|%a           |Nom du jour de la semaine en abrégé                          |
+-------------+-------------------------------------------------------------+
|%m           |Mois (nombre de deux chiffres, de 00 à 12)                   |
+-------------+-------------------------------------------------------------+
|%c           |Mois (nombre de un ou deux chiffres, de 0 à 12)              |
+-------------+-------------------------------------------------------------+
|%M           |Nom du mois                                                  |
+-------------+-------------------------------------------------------------+
|%b           |Nom du mois en abrégé                                        |
+-------------+-------------------------------------------------------------+
|%y           |Année, sur deux chiffres                                     |
+-------------+-------------------------------------------------------------+
|%Y           |Année, sur quatre chiffres                                   |
+-------------+-------------------------------------------------------------+
|%r           |Heure complète, format 12h (hh:mm:ss AM/PM)                  |
+-------------+-------------------------------------------------------------+
|%T           |Heure complète, format 24h (hh:mm:ss)                        |
+-------------+-------------------------------------------------------------+
|%h           |Heure sur deux chiffres et sur 12 heures (de 00 à 12)        |
+-------------+-------------------------------------------------------------+
|%H           |Heure sur deux chiffres et sur 24 heures (de 00 à 23)        |
+-------------+-------------------------------------------------------------+
|%l           |Heure sur un ou deux chiffres et sur 12 heures (de 0 à 12)   |
+-------------+-------------------------------------------------------------+
|%k           |Heure sur un ou deux chiffres et sur 24 heures (de 0 à 23)   |
+-------------+-------------------------------------------------------------+
|%i           |Minutes (de 00 à 59)                                         |
+-------------+-------------------------------------------------------------+
|%s ou %S     |Secondes (de 00 à 59)                                        |
+-------------+-------------------------------------------------------------+
|%p           |AM/PM                                                        |
+-------------+-------------------------------------------------------------+





Tous les caractères ne faisant pas partie d'un spécificateur sont simplement recopiés tels quels.


# Exemples


## Même résultat que précédemment…

…Avec une requête bien plus courte :


```sql
SELECT nom, date_naissance, DATE_FORMAT(date_naissance, 'le %W %e %M %Y') AS jolie_date
FROM Animal
WHERE espece_id = 4;
```




+--------+-------------------+---------------------------+
|nom     |date_naissance     |jolie_date                 |
+========+===================+===========================+
|Safran  |2007-03-04 19:36:00|le dimanche 4 mars 2007    |
+--------+-------------------+---------------------------+
|Gingko  |2008-02-20 02:50:00|le mercredi 20 février 2008|
+--------+-------------------+---------------------------+
|Bavard  |2009-03-26 08:28:00|le jeudi 26 mars 2009      |
+--------+-------------------+---------------------------+
|Parlotte|2009-03-26 07:55:00|le jeudi 26 mars 2009      |
+--------+-------------------+---------------------------+






## Autres exemples
 

[[attention]]
| Attention à bien échapper les guillemets éventuels dans la chaîne de caractères du format.


```sql
SELECT DATE_FORMAT(NOW(), 'Nous sommes aujourd''hui le %d %M de l''année %Y. Il est actuellement %l heures et %i minutes.') AS Top_date_longue;

SELECT DATE_FORMAT(NOW(), '%d %b. %y - %r') AS Top_date_courte;
```




+--------------------------------------------------------------------------------------------------+
|Top_date_longue                                                                                   |
+==================================================================================================+
|Nous sommes aujourd'hui le 27 octobre de l'année 2011. Il est actuellement 3 heures et 17 minutes.|
+--------------------------------------------------------------------------------------------------+







+------------------------+
|Top_date_courte         |
+========================+
|27 oct. 11 - 03:34:40 PM|
+------------------------+






# Fonction supplémentaire pour l'heure

`DATE_FORMAT()` peut s'utiliser sur des données de type `DATE`, `TIME` ou `DATETIME`. Mais il existe également une fonction `TIME_FORMAT(heure, format)`, qui ne sert qu'à formater les heures (et ne doit donc pas s'utiliser sur une `DATE`). Elle s'utilise exactement de la même manière, simplement il faut y utiliser des spécificateurs ayant du sens pour une donnée `TIME`, sinon `NULL` ou `0` est renvoyé.
Si un mauvais format de `TIME` ou `DATETIME` est donné à `TIME_FORMAT()` (par exemple, si on lui donne une DATE), MySQL va tenter d'interpréter la donnée et renverra bien un résultat, mais celui-ci n'aura peut-être pas beaucoup de sens (pour vous du moins).

**Exemples** : 


```sql
-- Sur une DATETIME
SELECT TIME_FORMAT(NOW(), '%r') AS sur_datetime, 
       TIME_FORMAT(CURTIME(), '%r') AS sur_time, 
       TIME_FORMAT(NOW(), '%M %r') AS mauvais_specificateur, 
       TIME_FORMAT(CURDATE(), '%r') AS sur_date;
```




+------------+-----------+---------------------+-----------+
|sur_datetime|sur_time   |mauvais_specificateur|sur_date   |
+============+===========+=====================+===========+
|10:58:47 AM |10:58:47 AM|NULL                 |12:20:12 AM|
+------------+-----------+---------------------+-----------+




L'application de `TIME_FORMAT()` sur `CURDATE()` a renvoyé un avertissement :




+-------+----+--------------------------------------------+
|Level  |Code|Message                                     |
+=======+====+============================================+
|Warning|1292|Truncated incorrect time value: '2012-04-30'|
+-------+----+--------------------------------------------+






# Formats standards

Il existe un certain nombre de formats de date et d'heure standards, prédéfinis, que l'on peut utiliser dans la fonction `DATE_FORMAT()`. Pour obtenir ces formats, il faut appeler la fonction `GET_FORMAT(type, standard)`.

Le paramètre type doit être choisi entre les trois types de données : `DATE`, `TIME` et `DATETIME`.

Il existe cinq formats standards :


- `'EUR'`
- `'USA'`
- `'ISO'`
- `'JIS'`
- `'INTERNAL'`


Voici un tableau reprenant les différentes possibilités :




+---------------------------------+-------------------+-------------------+
|Fonction                         |Format             |Exemple            |
+=================================+===================+===================+
|`GET_FORMAT(DATE,'USA')`         |'%m.%d.%Y'         |10.30.1988         |
+---------------------------------+-------------------+-------------------+
|`GET_FORMAT(DATE,'JIS')`         |'%Y-%m-%d'         |1988-10-30         |
+---------------------------------+-------------------+-------------------+
|`GET_FORMAT(DATE,'ISO')`         |'%Y-%m-%d'         |1988-10-30         |
+---------------------------------+-------------------+-------------------+
|`GET_FORMAT(DATE,'EUR')`         |'%d.%m.%Y'         |30.10.1988         |
+---------------------------------+-------------------+-------------------+
|`GET_FORMAT(DATE,'INTERNAL')`    |'%Y%m%d'           |19881031           |
+---------------------------------+-------------------+-------------------+
|`GET_FORMAT(DATETIME,'USA')`     |'%Y-%m-%d %H.%i.%s'|1988-10-30 13.44.33|
+---------------------------------+-------------------+-------------------+
|`GET_FORMAT(DATETIME,'JIS')`     |'%Y-%m-%d %H:%i:%s'|1988-10-30 13:44:33|
+---------------------------------+-------------------+-------------------+
|`GET_FORMAT(DATETIME,'ISO')`     |'%Y-%m-%d %H:%i:%s'|1988-10-30 13:44:33|
+---------------------------------+-------------------+-------------------+
|`GET_FORMAT(DATETIME,'EUR')`     |'%Y-%m-%d %H.%i.%s'|1988-10-30 13.44.33|
+---------------------------------+-------------------+-------------------+
|`GET_FORMAT(DATETIME,'INTERNAL')`|'%Y%m%d%H%i%s'     |19881030134433     |
+---------------------------------+-------------------+-------------------+
|`GET_FORMAT(TIME,'USA')`         |'%h:%i:%s %p'      |1:44:33 PM         |
+---------------------------------+-------------------+-------------------+
|`GET_FORMAT(TIME,'JIS')`         |'%H:%i:%s'         |13:44:33           |
+---------------------------------+-------------------+-------------------+
|`GET_FORMAT(TIME,'ISO')`         |'%H:%i:%s'         |13:44:33           |
+---------------------------------+-------------------+-------------------+
|`GET_FORMAT(TIME,'EUR')`         |'%H.%i.%S'         |13.44.33           |
+---------------------------------+-------------------+-------------------+
|`GET_FORMAT(TIME,'INTERNAL')`    |'%H%i%s'           |134433             |
+---------------------------------+-------------------+-------------------+





**Exemples** :


```sql
SELECT DATE_FORMAT(NOW(), GET_FORMAT(DATE, 'EUR')) AS date_eur,
       DATE_FORMAT(NOW(), GET_FORMAT(TIME, 'JIS')) AS heure_jis,
       DATE_FORMAT(NOW(), GET_FORMAT(DATETIME, 'USA')) AS date_heure_usa;
```




+----------+---------+-------------------+
|date_eur  |heure_jis|date_heure_usa     |
+==========+=========+===================+
|29.04.2012|11:20:55 |2012-04-29 11.20.55|
+----------+---------+-------------------+