## Intervalle

Certaines des fonctions et opérations suivantes utilisent le mot-clé `INTERVAL`, permettant de définir un intervalle de temps à ajouter ou à soustraire d'une date.

Un intervalle de temps est défini par une quantité et une unité ("3 jours" par exemple, "3" étant la quantité, "jour" l'unité). En MySQL, il existe une vingtaine d'unités possibles pour un `INTERVAL`, dont une partie est listée dans le tableau ci-dessous.




+---------------+--------------+
|Unité          |Format        |
+===============+==============+
|`SECOND`       |-             |
+---------------+--------------+
|`MINUTE`       |-             |
+---------------+--------------+
|`HOUR`         |-             |
+---------------+--------------+
|`DAY`          |-             |
+---------------+--------------+
|`WEEK`         |-             |
+---------------+--------------+
|`MONTH`        |-             |
+---------------+--------------+
|`YEAR`         |-             |
+---------------+--------------+
|`MINUTE_SECOND`|`'m:S'`       |
+---------------+--------------+
|`HOUR_SECOND`  |`'HH:mm:SS'`  |
+---------------+--------------+
|`HOUR_MINUTE`  |`'HH:mm'`     |
+---------------+--------------+
|`DAY_SECOND`   |`'J HH:mm:SS'`|
+---------------+--------------+
|`DAY_MINUTE`   |`'J HH:mm'`   |
+---------------+--------------+
|`DAY_HOUR`     |`'J HH'`      |
+---------------+--------------+
|`YEAR_MONTH`   |`'A-M'`       |
+---------------+--------------+





Notez que, comme pour `DATE`, `DATETIME` et `TIME`, les signes de ponctuation séparant les différentes parties d'un intervalle ne doivent pas nécessairement être `'-'` pour la partie date et `':'` pour la partie heure. Il ne s'agit que de suggestions. N'importe quel signe de ponctuation (ou aucun) sera accepté.


# Ajout d'un intervalle de temps

Trois fonctions permettent d'ajouter un intervalle de temps à une date (de type `DATE` ou `DATETIME`) :


- `ADDDATE()` : qui s'utilise avec un `INTERVAL` ou un nombre de jours.
- `DATE_ADD()` : qui s'utilise avec un `INTERVAL`.
- `TIMESTAMPADD()` : qui n'utilise pas d'`INTERVAL` mais un nombre plus limité d'unités de temps.



## ADDDATE()

Cette fonction peut s'utiliser de deux manières : soit en précisant un intervalle de temps avec le mot-clé `INTERVAL`, soit en donnant un nombre de jours à ajouter à la date.


- `ADDDATE(date, INTERVAL quantite unite)`
- `ADDDATE(date, nombreJours)`


**Exemples** :


```sql
SELECT ADDDATE('2011-05-21', INTERVAL 3 MONTH) AS date_interval,                              -- Avec DATE et INTERVAL
       ADDDATE('2011-05-21 12:15:56', INTERVAL '3 02:10:32' DAY_SECOND) AS datetime_interval, -- Avec DATETIME et INTERVAL
       ADDDATE('2011-05-21', 12) AS date_nombre_jours,                                        -- Avec DATE et nombre de jours
       ADDDATE('2011-05-21 12:15:56', 42) AS datetime_nombre_jours;                           -- Avec DATETIME et nombre de jours
```




+-------------+-------------------+-----------------+---------------------+
|date_interval|datetime_interval  |date_nombre_jours|datetime_nombre_jours|
+=============+===================+=================+=====================+
|2011-08-21   |2011-05-24 14:26:28|2011-06-02       |2011-07-02 12:15:56  |
+-------------+-------------------+-----------------+---------------------+






## DATE_ADD()


`DATE_ADD(date, INTERVAL quantite unite)` s'utilise exactement de la même manière que `ADDDATE(date, INTERVAL quantite unite)`.

**Exemples** :


```sql
SELECT DATE_ADD('2011-05-21', INTERVAL 3 MONTH) AS avec_date,                               -- Avec DATE
       DATE_ADD('2011-05-21 12:15:56', INTERVAL '3 02:10:32' DAY_SECOND) AS avec_datetime;  -- Avec DATETIME
```




+----------+-------------------+
|avec_date |avec_datetime      |
+==========+===================+
|2011-08-21|2011-05-24 14:26:28|
+----------+-------------------+






## Opérateur +

Il est également possible d'ajouter un intervalle de temps à une date en utilisant simplement l'opérateur `+` et un `INTERVAL`. L'intervalle peut se trouver à droite ou à gauche du signe `+`.

**Exemples** :


```sql
SELECT '2011-05-21' + INTERVAL 5 DAY AS droite,                    -- Avec DATE et intervalle à droite
       INTERVAL '3 12' DAY_HOUR + '2011-05-21 12:15:56' AS gauche; -- Avec DATETIME et intervalle à gauche
```




+----------+-------------------+
|droite    |gauche             |
+==========+===================+
|2011-05-26|2011-05-25 00:15:56|
+----------+-------------------+






## TIMESTAMPADD()

`TIMESTAMPADD(unite, quantite, date)` est un peu plus restreint que `DATE_ADD()` et `ADDDATE()`. En effet, cette fonction n'utilise pas d'`INTERVAL`. Il faut cependant définir une unité parmi les suivantes : `FRAC_SECOND`, `SECOND`, `MINUTE`, `HOUR`, `DAY`, `WEEK`, `MONTH`, `QUARTER`, et `YEAR`. 

**Exemple** :


```sql
SELECT TIMESTAMPADD(DAY, 5, '2011-05-21') AS avec_date,                   -- Avec DATE
       TIMESTAMPADD(MINUTE, 34, '2011-05-21 12:15:56') AS avec_datetime;  -- Avec DATETIME
```




+----------+-------------------+
|avec_date |avec_datetime      |
+==========+===================+
|2011-05-26|2011-05-21 12:49:56|
+----------+-------------------+






## ADDTIME()

La fonction `ADDTIME(expr1, expr2)` permet d'ajouter _expr2_ (de type `TIME`) à _expr1_ (de type `DATETIME` ou `TIME`). Le résultat sera du même type que _expr1_.

**Exemples** :


```sql
SELECT NOW() AS Maintenant, ADDTIME(NOW(), '01:00:00') AS DansUneHeure,                  -- Avec un DATETIME
       CURRENT_TIME() AS HeureCourante, ADDTIME(CURRENT_TIME(), '03:20:02') AS PlusTard; -- Avec un TIME
```




+-------------------+-------------------+-------------+--------+
|Maintenant         |DansUneHeure       |HeureCourante|PlusTard|
+===================+===================+=============+========+
|2012-04-29 12:08:33|2012-04-29 13:08:33|12:08:33     |15:28:35|
+-------------------+-------------------+-------------+--------+






# Soustraction d'un intervalle de temps


## SUBDATE(), DATE_SUB() et SUBTIME()

`SUBDATE()`, `DATE_SUB()` et `SUBTIME()` sont les équivalents de `ADDDATE()`, `DATE_ADD()` et `ADDTIME()` pour la soustraction. Elles s'utilisent exactement de la même manière.

**Exemples** :


```sql
SELECT SUBDATE('2011-05-21 12:15:56', INTERVAL '3 02:10:32' DAY_SECOND) AS SUBDATE1, 
       SUBDATE('2011-05-21', 12) AS SUBDATE2,
       DATE_SUB('2011-05-21', INTERVAL 3 MONTH) AS DATE_SUB;

SELECT SUBTIME('2011-05-21 12:15:56', '18:35:15') AS SUBTIME1,
       SUBTIME('12:15:56', '8:35:15') AS SUBTIME2;
```




+-------------------+----------+----------+
|SUBDATE1           |SUBDATE2  |DATE_SUB  |
+===================+==========+==========+
|2011-05-18 10:05:24|2011-05-09|2011-02-21|
+-------------------+----------+----------+







+-------------------+--------+
|SUBTIME1           |SUBTIME2|
+===================+========+
|2011-05-20 17:40:41|03:40:41|
+-------------------+--------+






## Opérateur -

Tout comme l'opérateur `+` peut s'utiliser pour ajouter un intervalle de temps, il est possible d'utiliser l'opérateur `-` pour en soustraire un. Cependant, pour la soustraction, la date doit impérativement se trouver à gauche du signe `-`, et l'intervalle à droite. Il n'est en effet pas possible de soustraire une date d'un intervalle.

**Exemple** :


```sql
SELECT '2011-05-21' - INTERVAL 5 DAY;
```




+-----------------------------+
|'2011-05-21' - INTERVAL 5 DAY|
+=============================+
|2011-05-16                   |
+-----------------------------+






## Soustraire, c'est ajouter un négatif

Un `INTERVAL` peut être défini avec une quantité négative, et ajouter un intervalle négatif, c'est soustraire un intervalle positif. De même soustraire un intervalle négatif revient à ajouter un intervalle positif.

Par conséquent, dans les requêtes suivantes, les deux parties du `SELECT` sont équivalentes.


```sql
SELECT ADDDATE(NOW(), INTERVAL -3 MONTH) AS ajout_negatif, SUBDATE(NOW(), INTERVAL 3 MONTH) AS retrait_positif;
SELECT DATE_ADD(NOW(), INTERVAL 4 HOUR) AS ajout_positif, DATE_SUB(NOW(), INTERVAL -4 HOUR) AS retrait_negatif;
SELECT NOW() + INTERVAL -15 MINUTE AS ajout_negatif, NOW() - INTERVAL 15 MINUTE AS retrait_positif;
```




+-------------------+-------------------+
|ajout_negatif      |retrait_positif    |
+===================+===================+
|2011-09-01 16:04:26|2011-09-01 16:04:26|
+-------------------+-------------------+







+-------------------+-------------------+
|ajout_positif      |retrait_negatif    |
+===================+===================+
|2011-12-01 20:04:26|2011-12-01 20:04:26|
+-------------------+-------------------+







+-------------------+-------------------+
|ajout_negatif      |retrait_positif    |
+===================+===================+
|2011-12-01 15:49:28|2011-12-01 15:49:28|
+-------------------+-------------------+