# Créer une date/heure à partir d'autres informations


## À partir d'un timestamp Unix

La fonction `FROM_UNIXTIME(ts)` renvoie un `DATETIME` à partir du timestamp Unix _ts_.


```sql
SELECT FROM_UNIXTIME(1325595287);
```




+-------------------------+
|FROM_UNIXTIME(1325595287)|
+=========================+
|2012-01-03 13:54:47      |
+-------------------------+





Notez que la fonction `UNIX_TIMESTAMP()`, que nous avons vue lors d'un chapitre précédent et qui donne le timestamp actuel, peut également s'utiliser avec un `DATETIME` en paramètre ; dans ce cas, elle fait l'inverse de la fonction `FROM_UNIXTIME(ts)` : elle renvoie le timestamp Unix du `DATETIME` passé en paramètre.


```sql
SELECT UNIX_TIMESTAMP('2012-01-03 13:54:47');
```




+-------------------------------------+
|UNIX_TIMESTAMP('2012-01-03 13:54:47')|
+=====================================+
|1325595287                           |
+-------------------------------------+






## À partir de différents éléments d'une date/heure

La fonction `MAKEDATE()` crée une `DATE` à partir d'une année et d'un numéro de jour (1 étant le premier janvier, 32 le premier février, etc.). Quant à la fonction `MAKETIME()`, elle crée un `TIME` à partir d'une heure et d'un nombre de minutes et de secondes.


```sql
SELECT MAKEDATE(2012, 60) AS 60eJour2012, MAKETIME(3, 45, 34) AS heureCree;
```




+-----------+---------+
|60eJour2012|heureCree|
+===========+=========+
|2012-02-29 |03:45:34 |
+-----------+---------+






# Convertir un TIME en secondes, et _vice versa_

Il est toujours utile de connaître la fonction `SEC_TO_TIME()`qui convertit un nombre de secondes en une donnée de type `TIME`, et son opposé `TIME_TO_SEC()`qui convertit un `TIME` en un nombre de secondes.

**Exemples** :


```sql
SELECT SEC_TO_TIME(102569), TIME_TO_SEC('01:00:30');
```




+-------------------+-----------------------+
|SEC_TO_TIME(102569)|TIME_TO_SEC('01:00:30')|
+===================+=======================+
|28:29:29           |3630                   |
+-------------------+-----------------------+






[[information]]
| Je vous rappelle qu'il est normal d'avoir un nombre d'heures supérieur à 24 dans un `TIME`, puisque `TIME` sert à stocker une heure ou une durée, et peut par conséquent aller de `'-838:59:59'` à `'838:59:59'`.
 

# Dernier jour du mois

Enfin, voici la dernière fonction que nous verrons dans ce chapitre : `LAST_DAY(date)`. Cette fonction donne le dernier jour du mois de la date passée en paramètre. Cela permet par exemple de voir que 2012 est une année bissextile, contrairement à l'an 2100.


```sql
SELECT LAST_DAY('2012-02-03') AS fevrier2012, LAST_DAY('2100-02-03') AS fevrier2100;
```




+-----------+-----------+
|fevrier2012|fevrier2100|
+===========+===========+
|2012-02-29 |2100-02-28 |
+-----------+-----------+