Derrière ce titre mystérieux se cache un concept très important !


[[question]]
| Quels sont les **critères** qu'un système utilisant les transactions doit respecter pour être **fiable** ?

Il a été défini que ces critères sont au nombre de quatre : **A**tomicité, **C**ohérence, **I**solation et **D**urabilité. Soit, si on prend la première lettre de chaque critère : ACID.
Voyons donc en détail ces quatre critères.


# A pour Atomicité

Atome signifie étymologiquement "**qui ne peut être divisé**".
Une transaction doit être atomique, c'est-à-dire qu'elle doit former une **entité complète et indivisible**. Chaque élément de la transaction, chaque requête effectuée, ne peut exister que dans la transaction.

Si l'on reprend l'exemple du virement bancaire, en utilisant les transactions, les deux étapes (débit du compte donneur d'ordre, crédit du compte bénéficiaire) ne peuvent exister indépendamment l'une de l'autre. Si l'une est exécutée, l'autre doit l'être également. Il s'agit d'un tout.


[[question]]
| Peut-on dire que nos transactions sont atomiques ?

Oui. Si une transaction en cours est interrompue, aucune des requêtes exécutées ne sera validée. De même, en cas d'erreur, il suffit de faire un `ROLLBACK` pour annuler toute la transaction. Et si tout se passe bien, un `COMMIT` validera l'intégralité de la transaction en une fois.


# C pour cohérence

Les données doivent rester **cohérentes dans tous les cas** : que la transaction se termine sans encombre, qu'une erreur survienne, ou que la transaction soit interrompue. Un virement dont seule l'étape de débit du donneur d'ordre est exécutée  produit des données incohérentes (la disparition de 300 euros jamais arrivés chez le bénéficiaire). Avec une transaction, cette incohérence n'apparaît jamais. Tant que la totalité des étapes n'a pas été réalisée avec succès, les données restent dans leur état initial.


[[question]]
| Nos transactions permettent-elles d'assurer la cohérence des données ?

Oui, les changements de données ne sont validés qu'une fois que toutes les étapes ont été exécutées. De l'extérieur de la transaction, le moment entre les deux étapes d'un virement n'est jamais visible.


# I pour Isolation

Chaque transaction doit être isolée, donc **ne pas interagir avec une autre transaction**.


[[question]]
| Nos transactions sont-elles isolées ?


## Test

Dans votre client MySQL, exécutez les requêtes suivantes (ne commitez pas) pour modifier le _pere_id_ du rat Bibo :


[[attention]]
| Copiez-collez tout le bloc dans votre client MySQL



```sql
START TRANSACTION; -- On ouvre une transaction

UPDATE Animal     -- On modifie Bibo
SET pere_id = 73
WHERE espece_id = 5 AND nom = 'Bibo';

SELECT id, nom, commentaires, pere_id, mere_id
FROM Animal
WHERE espece_id = 5;
```


À nouveau, ouvrez une deuxième session, tout en laissant la première ouverte (démarrez un deuxième client SQL et connectez-vous à votre base de données). 
Exécutez les requêtes suivantes, pour modifier les _commentaires_ de Bibo.


[[attention]]
| À nouveau, prenez bien tout le bloc d'un coup, vous suivrez plus facilement les explications qui suivent.



```sql
START TRANSACTION;   -- On ouvre une transaction

SELECT id, nom, commentaires, pere_id, mere_id
FROM Animal
WHERE espece_id = 5;

UPDATE Animal       -- On modifie la perruche Bibo
SET commentaires = 'Agressif'
WHERE espece_id = 5 AND nom = 'Bibo';

SELECT id, nom, commentaires, pere_id, mere_id
FROM Animal
WHERE espece_id = 5;
```


Le résultat n'est pas du tout le même dans les deux sessions. En effet, dans la première, on a la confirmation que la requête `UPDATE` a été effectuée :


```console
Query OK, 1 row affected (0.00 sec)
Rows matched: 1  Changed: 1  Warnings: 0
```


Et le `SELECT` renvoie bien les données modifiées (_pere_id_ n'est plus `NULL` pour Bibo) :




+--+----+------------+-------+-------+
|id|nom |commentaires|pere_id|mere_id|
+==+====+============+=======+=======+
|69|Baba|NULL        |NULL   |NULL   |
+--+----+------------+-------+-------+
|70|Bibo|NULL        |73     |72     |
+--+----+------------+-------+-------+
|72|Momy|NULL        |NULL   |NULL   |
+--+----+------------+-------+-------+
|73|Popi|NULL        |NULL   |NULL   |
+--+----+------------+-------+-------+
|75|Mimi|NULL        |NULL   |NULL   |
+--+----+------------+-------+-------+





Par contre, dans la deuxième session, on a d'abord fait un `SELECT`, et Bibo n'a toujours pas de père (puisque ça n'a pas été commité dans la première session). Donc on s'attendrait à ce que la requête `UPDATE` laisse _pere_id_ à `NULL` et modifie _commentaires_.




+--+----+------------+-------+-------+
|id|nom |commentaires|pere_id|mere_id|
+==+====+============+=======+=======+
|69|Baba|NULL        |NULL   |NULL   |
+--+----+------------+-------+-------+
|70|Bibo|NULL        |NULL   |72     |
+--+----+------------+-------+-------+
|72|Momy|NULL        |NULL   |NULL   |
+--+----+------------+-------+-------+
|73|Popi|NULL        |NULL   |NULL   |
+--+----+------------+-------+-------+
|75|Mimi|NULL        |NULL   |NULL   |
+--+----+------------+-------+-------+





Seulement voilà, la requête `UPDATE` ne fait rien ! La session semble bloquée : pas de message de confirmation après la requête `UPDATE`, et le second `SELECT` n'a pas été effectué.


```console
mysql>
mysql> UPDATE Animal       -- On modifie Bibo
    -> SET commentaires = 'Agressif'
    -> WHERE espece_id = 5 AND nom = 'Bibo';
_
```


Commitez maintenant les changements dans la première session (celle qui n'est pas bloquée). Retournez voir dans la seconde session : elle s'est débloquée et indique maintenant un message de confirmation aussi :


```console
Query OK, 1 row affected (5.17 sec)
Rows matched: 1  Changed: 1  Warnings: 0
```


Qui plus est, le `SELECT` a été exécuté (vous devrez peut-être appuyer sur ||Entrée|| pour que ce soit envoyé au serveur) et les modifications ayant été faites par la session 1 ont été prises en compte : _commentaires_ vaut `'Agressif'` et _pere_id_ vaut `73` !




+--+----+------------+-------+-------+
|id|nom |commentaires|pere_id|mere_id|
+==+====+============+=======+=======+
|69|Baba|NULL        |NULL   |NULL   |
+--+----+------------+-------+-------+
|70|Bibo|Agressif    |73     |72     |
+--+----+------------+-------+-------+
|72|Momy|NULL        |NULL   |NULL   |
+--+----+------------+-------+-------+
|73|Popi|NULL        |NULL   |NULL   |
+--+----+------------+-------+-------+
|75|Mimi|NULL        |NULL   |NULL   |
+--+----+------------+-------+-------+






[[information]]
| Il est possible que votre seconde session indique ceci : `ERROR 1205 (HY000): Lock wait timeout exceeded; try restarting transaction`. Cela signifie que la session est restée bloquée trop longtemps et que par conséquent la transaction a été automatiquement fermée (avec un rollback des requêtes effectuées). Dans ce cas, recommencez l'opération.


Il n'y a plus qu'à commiter les changements faits par la deuxième session, et c'est terminé ! Si vous ne commitez pas, _commentaires_ restera `NULL`. Par contre, _pere_id_ vaudra toujours `73` puisque ce changement-là a été commité par la première session.


## Conclusion

La deuxième session n'a pas interagi avec les changements faits par la première session, chaque transaction est bien **isolée**.


[[question]]
| Et la première session qui bloque la seconde, ce n'est pas une interaction ça ?

Pas dans le cadre des critères ACID. Oui, la première session provoque un retard dans l'exécution des requêtes de la deuxième session, mais les critères de fiabilité que nous examinons ici concernent **les données impactées par les transactions**, et non le déroulement de celles-ci (qui importe peu finalement). 
Ce blocage a pour effet d'empêcher la deuxième session d'écraser un changement fait par la première. Donc, ce blocage a bien pour effet l'isolation des transactions.


## Verrous

Le blocage de la deuxième session vient en fait de ce que la première session, en faisant sa requête `UPDATE`, a **automatiquement posé un verrou** sur la ligne contenant Bobi le rat, empêchant toute modification tant que la transaction était en cours. Les verrous faisant l'objet du prochain chapitre, je n'en dis pas plus pour l'instant.


## Utilité

Je vous l'accorde, vous n'allez pas vous amuser tous les jours à ouvrir deux sessions MySQL. Par contre, pour une application pouvant être utilisée par plusieurs personnes en même temps (qui toutes travaillent sur la même base de données), il est impératif que ce critère soit respecté.
Prenons l'exemple simple d'un jeu par navigateur : de nombreux joueurs peuvent être connectés en même temps, et effectuer des opérations différentes. Si les transactions ne sont pas isolées, une partie des actions des joueurs risquerait de se voir annulées. On isole donc les transactions grâce aux verrous (qui sont ici automatiquement posés mais ce n'est pas toujours le cas). 


# D pour Durabilité


Une fois la transaction terminée, les données résultant de cette transaction doivent être **stockées de manière durable**, et pouvoir être récupérées, en cas de crash du serveur par exemple.


[[question]]
| Nos transactions modifient-elles les données de manière durable ?

Oui, une fois les changements commités, ils sont stockés définitivement (jusqu'à modification par une nouvelle transaction).

*[ACID]: Atomicité Cohérence Isolation Durabilité