Lorsqu'une session MySQL pose un verrou sur un élément de la base de données, cela veut dire qu'il **restreint, voire interdit, l'accès à cet élément aux autres sessions MySQL** qui voudraient y accéder.


# Verrous de table et verrous de ligne

Il est possible de poser un **verrou sur une table entière**, ou **seulement sur une ou plusieurs lignes d'une table**. Étant donné qu'un verrou empêche l'accès d'autres sessions, il est en général plus intéressant de poser un verrou sur la plus petite partie de la base possible.

Par exemple, si l'on travaille avec les chiens de la table _Animal_. 


- On peut poser un verrou sur toute la table _Animal_. Dans ce cas, les autres sessions n'auront pas accès à cette table, tant que le verrou sera posé. Qu'elles veuillent en utiliser les chiens, les chats, ou autre, tout leur sera refusé.
- On peut aussi poser un verrou uniquement sur les lignes de la table qui contiennent des chiens. De cette manière, les autres sessions pourront accéder aux chats, aux perroquets, etc. Elles pourront toujours travailler, tant qu'elles n'utilisent pas les chiens.


Cette notion d'accès simultané aux données par plusieurs sessions différentes s'appelle la **concurrence**. Plus la concurrence est possible, donc plus le nombre de sessions pouvant accéder aux données simultanément est grand, mieux c'est. 
En effet, prenons l'exemple d'un site web. En général, on préfère permettre à plusieurs utilisateurs de surfer en même temps, sans devoir attendre entre chaque action de pouvoir accéder aux informations chacun à son tour. Or, chaque utilisateur crée une session chaque fois qu'il se connecte à la base de données (pour lire les informations ou les modifier).
Préférez donc (autant que possible) les verrous de ligne aux verrous de table !


# Avertissements

Les informations données dans ce chapitre **concernent exclusivement MySQL**, et en particulier les tables utilisant **les moteurs MyISAM ou InnoDB** (selon le type de verrou utilisé). En effet, les verrous sont implémentés différemment selon les SGDB, et même selon le moteur de table en ce qui concerne MySQL. Si le principe général reste toujours le même, certains comportements et certaines options peuvent différer d'une implémentation à l'autre. N'hésitez pas à vous renseigner plus avant.

Par ailleurs, je vous présente ici les principes généraux et les principales options, mais il faut savoir qu'il y a énormément à dire sur les verrous, et que j'ai donc dû faire un sérieux tri des informations avant de rédiger ce chapitre. À nouveau, en cas de doute, ou si vous avez besoin d'informations précises, je vous conseille vraiment de consulter la documentation officielle (si possible en anglais, car elle est infiniment plus complète qu'en français).

Enfin, dernier avertissement : de nombreux changements dans l'implémentation des verrous sont advenus lors du développement des dernières versions de MySQL. Aussi, la différence entre les verrous dans la version 5.0 et la version 5.5 est assez importante. Tout ce que je présente dans ce chapitre concerne la version 5.5. Vérifiez bien votre version, et si vous consultez la documentation officielle, prenez bien celle qui concerne votre propre version.


# Modification de notre base de données

Nous allons ajouter deux tables à notre base de données, afin d'illustrer au mieux l'intérêt et l'utilisation des verrous : une table _Client_, qui contiendra les coordonnées des clients de notre élevage, et une table _Adoption_, qui contiendra les renseignements concernant les adoptions faites par nos clients. 
Dorénavant, certains animaux présents dans notre table _Animal_ ne seront plus disponibles, car ils auront été adoptés. Nous les garderons cependant dans notre base de données. Avant toute adoption, il nous faudra donc vérifier la disponibilité de l'animal.

Voici les requêtes à effectuer pour faire ces changements.


[[secret]]
| ```sql
| -- Table Client
| CREATE TABLE Client (
| 	id SMALLINT UNSIGNED AUTO_INCREMENT NOT NULL,
| 	nom VARCHAR(100) NOT NULL,
| 	prenom VARCHAR(60) NOT NULL,
| 	adresse VARCHAR(200),
| 	code_postal VARCHAR(6),
| 	ville VARCHAR(60),
| 	pays VARCHAR(60),
| 	email VARBINARY(100),
| 	PRIMARY KEY (id),
| 	UNIQUE INDEX ind_uni_email (email)
| ) ENGINE = InnoDB;
| 
| -- Table Adoption
| CREATE TABLE Adoption (
| 	client_id SMALLINT UNSIGNED NOT NULL,
| 	animal_id SMALLINT UNSIGNED NOT NULL,
| 	date_reservation DATE NOT NULL,
| 	date_adoption DATE,
|         prix DECIMAL(7,2) UNSIGNED NOT NULL,
| 	paye TINYINT(1) NOT NULL DEFAULT 0,
| 	PRIMARY KEY (client_id, animal_id),
| 	CONSTRAINT fk_client_id FOREIGN KEY (client_id) REFERENCES Client(id),
| 	CONSTRAINT fk_adoption_animal_id FOREIGN KEY (animal_id) REFERENCES Animal(id),
| 	UNIQUE INDEX ind_uni_animal_id (animal_id)
| ) ENGINE = InnoDB;
| 
| -- Insertion de quelques clients
| INSERT INTO Client (prenom, nom, adresse, code_postal, ville, pays, email) VALUES ('Jean', 'Dupont', 'Rue du Centre, 5', '45810', 'Houtsiplou', 'France', 'jean.dupont@email.com');
| INSERT INTO Client (prenom, nom, adresse, code_postal, ville, pays, email) VALUES ('Marie', 'Boudur', 'Place de la Gare, 2', '35840', 'Troudumonde', 'France', 'marie.boudur@email.com');
| INSERT INTO Client (prenom, nom, adresse, code_postal, ville, pays, email) VALUES ('Fleur', 'Trachon', 'Rue haute, 54b', '3250', 'Belville', 'Belgique', 'fleurtrachon@email.com');
| INSERT INTO Client (prenom, nom, adresse, code_postal, ville, pays, email) VALUES ('Julien', 'Van Piperseel',  NULL, NULL, NULL, NULL, 'jeanvp@email.com');
| INSERT INTO Client (prenom, nom, adresse, code_postal, ville, pays, email) VALUES ('Johan', 'Nouvel',  NULL, NULL, NULL, NULL, 'johanetpirlouit@email.com');
| INSERT INTO Client (prenom, nom, adresse, code_postal, ville, pays, email) VALUES ('Frank', 'Germain',  NULL, NULL, NULL, NULL, 'francoisgermain@email.com');
| INSERT INTO Client (prenom, nom, adresse, code_postal, ville, pays, email) VALUES ('Maximilien', 'Antoine', 'Rue Moineau, 123', '4580', 'Trocoul', 'Belgique', 'max.antoine@email.com');
| INSERT INTO Client (prenom, nom, adresse, code_postal, ville, pays, email) VALUES ('Hector', 'Di Paolo', NULL, NULL, NULL, NULL, 'hectordipao@email.com');
| INSERT INTO Client (prenom, nom, adresse, code_postal, ville, pays, email) VALUES ('Anaelle', 'Corduro',  NULL, NULL, NULL, NULL, 'ana.corduro@email.com');
| INSERT INTO Client (prenom, nom, adresse, code_postal, ville, pays, email) VALUES ('Eline', 'Faluche', 'Avenue circulaire, 7', '45870', 'Garduche', 'France', 'elinefaluche@email.com');
| INSERT INTO Client (prenom, nom, adresse, code_postal, ville, pays, email) VALUES ('Carine', 'Penni', 'Boulevard Haussman, 85', '1514', 'Plasse', 'Suisse', 'cpenni@email.com');
| INSERT INTO Client (prenom, nom, adresse, code_postal, ville, pays, email) VALUES ('Virginie', 'Broussaille', 'Rue du Fleuve, 18', '45810', 'Houtsiplou', 'France', 'vibrousaille@email.com');
| INSERT INTO Client (prenom, nom, adresse, code_postal, ville, pays, email) VALUES ('Hannah', 'Durant', 'Rue des Pendus, 66', '1514', 'Plasse', 'Suisse', 'hhdurant@email.com');
| INSERT INTO Client (prenom, nom, adresse, code_postal, ville, pays, email) VALUES ('Elodie', 'Delfour', 'Rue de Flore, 1', '3250', 'Belville', 'Belgique', 'e.delfour@email.com');
| INSERT INTO Client (prenom, nom, adresse, code_postal, ville, pays, email) VALUES ('Joel', 'Kestau', NULL, NULL, NULL, NULL, 'joel.kestau@email.com');
| 
| -- Insertion de quelques adoptions
| INSERT INTO Adoption (client_id, animal_id, date_reservation, date_adoption, prix, paye) VALUES (1, 39, '2008-08-17', '2008-08-17', 735.00, 1);
| INSERT INTO Adoption (client_id, animal_id, date_reservation, date_adoption, prix, paye) VALUES (1, 40, '2008-08-17', '2008-08-17', 735.00, 1);
| INSERT INTO Adoption (client_id, animal_id, date_reservation, date_adoption, prix, paye) VALUES (2, 18, '2008-06-04', '2008-06-04', 485.00, 1);
| INSERT INTO Adoption (client_id, animal_id, date_reservation, date_adoption, prix, paye) VALUES (3, 27, '2009-11-17', '2009-11-17', 200.00, 1);
| INSERT INTO Adoption (client_id, animal_id, date_reservation, date_adoption, prix, paye) VALUES (4, 26, '2007-02-21', '2007-02-21', 485.00, 1);
| INSERT INTO Adoption (client_id, animal_id, date_reservation, date_adoption, prix, paye) VALUES (4, 41, '2007-02-21', '2007-02-21', 835.00, 1);
| INSERT INTO Adoption (client_id, animal_id, date_reservation, date_adoption, prix, paye) VALUES (5, 21, '2009-03-08', '2009-03-08', 200.00, 1);
| INSERT INTO Adoption (client_id, animal_id, date_reservation, date_adoption, prix, paye) VALUES (6, 16, '2010-01-27', '2010-01-27', 200.00, 1);
| INSERT INTO Adoption (client_id, animal_id, date_reservation, date_adoption, prix, paye) VALUES (7, 5, '2011-04-05', '2011-04-05', 150.00, 1);
| INSERT INTO Adoption (client_id, animal_id, date_reservation, date_adoption, prix, paye) VALUES (8, 42, '2008-08-16', '2008-08-16', 735.00, 1);
| INSERT INTO Adoption (client_id, animal_id, date_reservation, date_adoption, prix, paye) VALUES (9, 55, '2011-02-13', '2011-02-13', 140.00, 1);
| INSERT INTO Adoption (client_id, animal_id, date_reservation, date_adoption, prix, paye) VALUES (9, 54, '2011-02-13', '2011-02-13', 140.00, 1);
| INSERT INTO Adoption (client_id, animal_id, date_reservation, date_adoption, prix, paye) VALUES (10, 49, '2010-08-17', '2010-08-17', 140.00, 1);
| INSERT INTO Adoption (client_id, animal_id, date_reservation, date_adoption, prix, paye) VALUES (11, 62, '2011-03-01', '2011-03-01', 630.00, 1);
| INSERT INTO Adoption (client_id, animal_id, date_reservation, date_adoption, prix, paye) VALUES (12, 69, '2007-09-20', '2007-09-20', 10.00, 1);
| INSERT INTO Adoption (client_id, animal_id, date_reservation, date_adoption, prix, paye) VALUES (13, 57, '2012-01-10', '2012-01-10', 700.00, 1);
| INSERT INTO Adoption (client_id, animal_id, date_reservation, date_adoption, prix, paye) VALUES (14, 58, '2012-02-25', '2012-02-25', 700.00,  1);
| INSERT INTO Adoption (client_id, animal_id, date_reservation, date_adoption, prix, paye) VALUES (15, 30, '2008-08-17', '2008-08-17', 735.00, 1);
| INSERT INTO Adoption (client_id, animal_id, date_reservation, date_adoption, prix, paye) VALUES (11, 32, '2008-08-17', '2010-03-09', 140.00, 1);
| INSERT INTO Adoption (client_id, animal_id, date_reservation, date_adoption, prix, paye) VALUES (9, 33, '2007-02-11', '2007-02-11', 835.00, 1);
| INSERT INTO Adoption (client_id, animal_id, date_reservation, date_adoption, prix, paye) VALUES (2, 3, '2011-03-12', '2011-03-12', 835.00, 1);
| ```


La table _Adoption_ ne contient pas de colonne _id_ auto-incrémentée. Par contre, on a bien défini une clé primaire, mais une clé primaire **composite** (sur plusieurs colonnes). En effet, une adoption est définie par un client adoptant un animal. Il n'est pas nécessaire d'ajouter une colonne supplémentaire pour définir individuellement chaque ligne ; le couple (_client_id, animal_id_) fait très bien l'affaire (il est composé de deux `SMALLINT`, donc les recherches sur cette clé seront rapides).
Notez que nous définissons également un index `UNIQUE` sur la colonne _animal_id_. Par conséquent, on aurait même pu définir directement _animal_id_ comme étant la clé primaire. Je trouvais cependant plus logique d'inclure le client dans la définition d'une adoption. C'est un choix plutôt arbitraire, qui a surtout comme avantage de vous montrer un exemple de clé composite.