[[attention]]
| Ces  verrous ne peuvent pas être posés sur une table utilisant le moteur MyISAM ! Tout ce qui est dit ici concerne les tables **InnoDB** uniquement.

Comme les verrous de table, les verrous de ligne peuvent être de deux types :


- **Les verrous partagés** : permettent aux autres sessions de lire les données, mais pas de les modifier (équivalents aux verrous de table de lecture) ;
- **Les verrous exclusifs** : ne permettent ni la lecture ni la modification des données (équivalents aux verrous d'écriture).



# Requêtes de modification, insertion et suppression



- Les requêtes de **modification et suppression** des données **posent automatiquement un verrou exclusif sur les lignes concernées**, à savoir les lignes sélectionnées par la clause `WHERE`, ou toutes les lignes s'il n'y a pas de clause `WHERE` (ou s'il n'y a pas d'index, sur les colonnes utilisées comme nous verrons plus loin).
- Les requêtes d'insertion quant à elles posent un **verrou exclusif sur la ligne insérée**.




# Requêtes de sélection

Les requêtes de sélection, par défaut, ne posent pas de verrous. Il faut donc en poser explicitement au besoin.


## Verrou partagé

Pour poser un verrou partagé, on utilise `LOCK IN SHARE MODE` à la fin de la requête `SELECT`.

```sql
SELECT * FROM Animal WHERE espece_id = 5 LOCK IN SHARE MODE;
```

Cette requête pose donc un verrou partagé sur les lignes de la table _Animal_ pour lesquelles _espece_id_ vaut 5.

Ce verrou signifie en fait, pour les autres sessions : "Je suis en train de lire ces données. Vous pouvez venir les lire aussi, mais pas les modifier tant que je n'ai pas terminé.".


## Verrou exclusif

Pour poser un verrou exclusif, on utilise `FOR UPDATE` à la fin de la requête `SELECT`.

```sql
SELECT * FROM Animal WHERE espece_id = 5 FOR UPDATE;
```

Ce verrou signifie aux autres sessions : "Je suis en train de lire ces données dans le but probable de faire une modification. Ne les lisez pas avant que j'aie fini (et bien sûr, ne les modifiez pas).".


# Transactions et fin d'un verrou de ligne

Les verrous de ligne ne sont donc pas posés par des commandes spécifiques, mais par des requêtes de sélection, insertion ou modification.
Ces verrous existent donc uniquement tant que la requête qui les a posés interagit avec les données.

Par conséquent, ce type de verrou s'utilise en conjonction avec les transactions. En effet, hors transaction, dès qu'une requête est lancée, elle est effectuée et les éventuelles modifications des données sont immédiatement validées. 
Par contre, dans le cas d'une requête faite dans une transaction, les changements ne sont pas validés tant que la transaction n'a pas été commitée. Donc, à partir du moment où une requête a été exécutée dans une transaction, et jusqu'à la fin de la transaction (`COMMIT` ou `ROLLBACK`), la requête a **potentiellement** un effet sur les données.
C'est à ce moment-là (quand une requête a été exécutée mais pas validée ou annulée) qu'il est intéressant de verrouiller les données qui vont **potentiellement être modifiées** (ou supprimées) par la transaction.

Un verrou de ligne est donc lié à la transaction dans laquelle il est posé. Dès que l'on fait un `COMMIT` ou un `ROLLBACK` de la transaction, le verrou est levé.


# Exemples


## Verrou posé par une requête de modification

Session 1 :

```sql
START TRANSACTION;

UPDATE Client SET pays = 'Suisse'
WHERE id = 8;           -- un verrou exclusif sera posé sur la ligne avec id = 8
```


Session 2:

```sql
START TRANSACTION;

SELECT * FROM Client
WHERE id = 8;      -- pas de verrou

SELECT * FROM Client 
WHERE id = 8 
LOCK IN SHARE MODE; -- on essaye de poser un verrou partagé
```


La première session a donc posé un verrou exclusif automatiquement en faisant un `UPDATE`.

La seconde session fait d'abord une simple sélection, sans poser de verrou. Pas de problème, la requête passe.


[[question]]
| Ah ? La requête passe ? Et c'est normal ? Et le verrou exclusif alors ?

Oui, c'est normal et c'est important de comprendre pourquoi.
En fait, lorsqu'une session **démarre une transaction**, **elle prend en quelque sorte une photo des tables dans leur état actuel** (les modifications non commitées n'étant pas visibles). La transaction va alors travailler sur la base de cette photo, tant qu'on ne lui demande pas d'aller vérifier que les données n'ont pas changé. Donc le `SELECT` ne voit pas les changements, et ne se heurte pas au verrou, puisque celui-ci est posé sur les lignes de la table, et non pas sur la photo de cette table que détient la session. 


[[question]]
| Et comment fait-on pour demander à la session d'actualiser sa photo ?

On lui demande de poser un verrou ! Lorsqu'une session **pose un verrou** sur une table, elle est obligée de travailler vraiment avec la table, et pas sur sa photo. Elle va donc aller chercher les dernières infos disponibles, et actualiser sa photo par la même occasion.
On le voit bien avec la seconde requête, qui tente de poser un verrou partagé (qui vise donc uniquement la lecture). Elle va d'abord chercher les lignes les plus à jour et tombe sur le verrou posé par la première session ; elle se retrouve alors bloquée jusqu'à ce que la première session ôte le verrou exclusif.

Session 1 :

```sql
COMMIT;
```


En committant les changements de la session 1, le verrou exclusif posé par la requête de modification est relâché. La session 2 est donc libre de poser à son tour un verrou partagé. 

On peut également essayer la même manœuvre, avec cette fois-ci un `UPDATE` plutôt qu'un `SELECT ... LOCK IN SHARE MODE` (donc une requête qui va tenter de poser un verrou exclusif plutôt qu'un verrou partagé).

Session 1 :

```sql
START TRANSACTION;

UPDATE Adoption SET paye = 0
WHERE client_id = 11;
```


Session 2 :

```sql
START TRANSACTION;

UPDATE Adoption SET paye = 1
WHERE animal_id = 32; -- l'animal 32 a été adopté par le client 11
```


Comme prévu, la seconde session est bloquée, jusqu'à ce que la première session termine sa transaction. Validez la transaction de la première session, puis de la seconde. Le comportement sera le même si la deuxième session fait un `DELETE` sur les lignes verrouillées, ou un `SELECT ... FOR UPDATE`.


## Verrou posé par une requête d'insertion

Session 1 :

```sql
START TRANSACTION;

INSERT INTO Adoption (client_id, animal_id, date_reservation, prix)
VALUES (12, 75, NOW(), 10.00);
```


Session 2 :

```sql
SELECT * FROM Adoption
WHERE client_id > 13
LOCK IN SHARE MODE;

SELECT * FROM Adoption
WHERE client_id < 13
LOCK IN SHARE MODE;
```


La première session insère une adoption pour le client 12 et pose un verrou exclusif sur cette ligne.
La seconde session fait deux requêtes `SELECT` en posant un verrou partagé : l'une qui sélectionne les adoptions des clients avec un id supérieur à 13 ; l'autre qui sélectionne les adoptions des clients avec un id inférieur à 13.
Seule la seconde requête `SELECT` se heurte au verrou posé par la première session, puisqu'elle tente de récupérer notamment les adoptions du client 12, dont une est verrouillée.

Dès que la session 1 commite l'insertion, la sélection se fait dans la session 2.

Session 1 : 

```sql
COMMIT;
```



## Verrou posé par une requête de sélection

Voyons d'abord le comportement d'un verrou partagé, posé par `SELECT ... LOCK IN SHARE MODE`.

Session 1 :

```sql
START TRANSACTION;

SELECT * FROM Client
WHERE id < 5
LOCK IN SHARE MODE;
```

Session 2 :

```sql
START TRANSACTION;

SELECT * FROM Client 
WHERE id BETWEEN 3 AND 8;

SELECT * FROM Client
WHERE id BETWEEN 3 AND 8
LOCK IN SHARE MODE;

SELECT * FROM Client
WHERE id BETWEEN 3 AND 8
FOR UPDATE;
```


La première session pose un verrou partagé sur les clients 1, 2, 3 et 4.
La seconde session fait trois requêtes de sélection. Toutes les trois concernent les clients 3 à 8 (dont les deux premiers sont verrouillés).


- Requête 1 : ne pose aucun verrou (travaille sur une "photo" de la table et pas sur les vraies données) donc s'effectue sans souci.
- Requête 2 : pose un verrou partagé, ce qui est faisable sur une ligne verrouillée par un verrou partagé. Elle s'effectue également.
- Requête 3 : tente de poser un verrou exclusif, ce qui lui est refusé.


Bien entendu, des requêtes `UPDATE` ou `DELETE` (posant des verrous exclusifs) faites par la deuxième session se verraient, elles aussi, bloquées.

Terminez les transactions des deux sessions (par un rollback ou un commit).

Quant aux requêtes `SELECT ... FOR UPDATE` posant un verrou exclusif, elles provoqueront exactement les mêmes effets qu'une requête `UPDATE` ou `DELETE` (après tout, un verrou exclusif, c'est un verrou exclusif).

Session 1 :

```sql
START TRANSACTION;

SELECT * FROM Client
WHERE id < 5
FOR UPDATE;
```

Session 2 :

```sql
START TRANSACTION;

SELECT * FROM Client 
WHERE id BETWEEN 3 AND 8;

SELECT * FROM Client
WHERE id BETWEEN 3 AND 8
LOCK IN SHARE MODE;
```


Cette fois-ci, même la requête `SELECT ... LOCK IN SHARE MODE` de la seconde session est bloquée (comme le serait une requête `SELECT ... FOR UPDATE`, ou une requête `UPDATE`, ou une requête `DELETE`).


# En résumé



- On pose un verrou partagé lorsqu'on fait une requête dans le but de lire des données.
- On pose un verrou exclusif lorsqu'on fait une requête dans le but (immédiat ou non) de modifier des données.
- Un verrou partagé sur les lignes _x_ va permettre aux autres sessions d'obtenir également un verrou partagé sur les lignes _x_, mais pas d'obtenir un verrou exclusif.
- Un verrou exclusif sur les lignes _x_ va empêcher les autres sessions d'obtenir un verrou sur les lignes _x_, qu'il soit partagé ou exclusif.



[[information]]
| En fait, ils portent plutôt bien leurs noms ces verrous !



# Rôle des index

Tentons une nouvelle expérience.

Session 1 :

```sql
START TRANSACTION;
UPDATE Animal 
SET commentaires = CONCAT_WS(' ', 'Animal fondateur.', commentaires) -- On ajoute une phrase de commentaire
WHERE date_naissance < '2007-01-01';                                 -- à tous les animaux nés avant 2007
```


Session 2 :

```sql
START TRANSACTION;
UPDATE Animal 
SET commentaires = 'Aveugle'                  -- On modifie les commentaires
WHERE date_naissance = '2008-03-10 13:40:00'; -- De l'animal né le 10 mars 2008 à 13h40
```


Dans la session 1, on fait un `UPDATE` sur les animaux nés avant 2007. On s'attend donc à pouvoir utiliser les animaux nés après dans une autre session, puisque InnoDB pose des verrous sur les lignes et pas sur toute la table.
Pourtant, la session 2 semble bloquée lorsque l'on fait un `UPDATE` sur un animal né en 2008.
Faites un rollback sur la session 1 ; ceci débloque la session 2. Annulez également la requête de cette session.


[[question]]
| Ce comportement est donc en contradiction avec ce qu'on obtenait précédemment. Quelle est la différence ?

Le sous-titre vous a évidemment soufflé la réponse : la différence se trouve au niveau des index. Voyons donc ça !
Voici une commande qui va vous afficher les index présents sur la table _Animal_ :

```sql
SHOW INDEX FROM Animal;
```




+------+----------+---------------------+-----------+----+
|Table |Non_unique|Key_name             |Column_name|Null|
+======+==========+=====================+===========+====+
|Animal|0         |PRIMARY              |id         |    |
+------+----------+---------------------+-----------+----+
|Animal|0         |ind_uni_nom_espece_id|nom        |YES |
+------+----------+---------------------+-----------+----+
|Animal|0         |ind_uni_nom_espece_id|espece_id  |    |
+------+----------+---------------------+-----------+----+
|Animal|1         |fk_race_id           |race_id    |YES |
+------+----------+---------------------+-----------+----+
|Animal|1         |fk_espece_id         |espece_id  |    |
+------+----------+---------------------+-----------+----+
|Animal|1         |fk_mere_id           |mere_id    |YES |
+------+----------+---------------------+-----------+----+
|Animal|1         |fk_pere_id           |pere_id    |YES |
+------+----------+---------------------+-----------+----+






[[information]]
| Une partie des colonnes du résultat montré ici a été retirée pour des raisons de clarté.


Nous avons donc des index sur les colonnes suivantes : _id_, _nom_, _mere_id_, _pere_id_, _espece_id_ et _race_id_.
Mais aucun index sur la colonne _date_naissance_.

Il semblerait donc que lorsque l'on pose un verrou, avec dans la clause `WHERE` de la requête une colonne indexée (_espece_id_), le verrou est bien posé uniquement sur les lignes pour lesquelles _espece_id_ vaut la valeur recherchée.
Par contre, si dans la clause `WHERE` on utilise une colonne non-indexée (_date_naissance_), MySQL n'est pas capable de déterminer quelles lignes doivent être bloquées, donc on se retrouve avec toutes les lignes bloquées.


[[question]]
| Pourquoi faut-il un index pour pouvoir poser un verrou efficacement ?

C'est très simple ! Vous savez que lorsqu'une colonne est indexée (que ce soit un index simple, unique, ou une clé primaire ou étrangère), MySQL stocke les valeurs de cette colonne **en les triant**. Du coup, lors d'une recherche sur l'index, pas besoin de parcourir toutes les lignes, il peut utiliser des algorithmes de recherche performants et trouver facilement les lignes concernées.
S'il n'y a pas d'index par contre, toutes les lignes doivent être parcourues chaque fois que la recherche est faite, et il n'y a donc pas moyen de verrouiller simplement une partie de l'index (donc une partie des lignes). Dans ce cas, MySQL verrouille toutes les lignes.

Cela fait une bonne raison de plus de mettre des index sur les colonnes qui servent fréquemment dans vos clauses `WHERE` !

Encore une petite expérience pour illustrer le rôle des index dans le verrouillage de lignes :

Session 1 :

```sql
START TRANSACTION;
UPDATE Animal  -- Modification de tous les rats
SET commentaires = CONCAT_WS(' ', 'Très intelligent.', commentaires)
WHERE espece_id = 5;
```


Session 2 :

```sql
START TRANSACTION;
UPDATE Animal
SET commentaires = 'Aveugle'
WHERE id = 34; -- Modification de l'animal 34 (un chat)
UPDATE Animal
SET commentaires = 'Aveugle'
WHERE id = 72; -- Modification de l'animal 72 (un rat)
```


La session 1 se sert de l'index sur _espece_id_ pour verrouiller les lignes contenant des rats bruns. Pendant ce temps, la session 2 veut modifier deux animaux : un chat et un rat, en se basant sur leur _id_.
La modification du chat se fait sans problème, par contre, la modification du rat est bloquée, tant que la transaction de la session 1 est ouverte. 
Faites un rollback des deux transactions.

On peut conclure de cette expérience que, bien que MySQL utilise les index pour verrouiller les lignes, il n'est pas nécessaire d'utiliser le même index pour avoir des accès concurrents.


# Lignes fantômes et index de clé suivante


[[question]]
| Qu'est-ce qu'une ligne fantôme ?

Dans une session, démarrons une transaction et sélectionnons toutes les adoptions faites par les clients dont l'_id_ dépasse 13, avec un verrou exclusif.

Session 1 :

```sql
START TRANSACTION;

SELECT * FROM Adoption WHERE client_id > 13 FOR UPDATE; -- ne pas oublier le FOR UPDATE pour poser le verrou
```

La requête va poser un verrou exclusif sur toutes les lignes dont _client_id_ vaut 14 ou plus.




+---------+---------+----------------+-------------+------+----+
|client_id|animal_id|date_reservation|date_adoption|prix  |paye|
+=========+=========+================+=============+======+====+
|14       |58       |2012-02-25      |2012-02-25   |700.00|1   |
+---------+---------+----------------+-------------+------+----+
|15       |30       |2008-08-17      |2008-08-17   |735.00|1   |
+---------+---------+----------------+-------------+------+----+





Imaginons maintenant qu'une seconde session démarre une transaction à ce moment-là, insère et commite une ligne dans _Adoption_ pour le client 15.
Si, par la suite, la première session refait la même requête de sélection avec verrou exclusif, elle va faire apparaître une troisième ligne de résultat : l'adoption nouvellement insérée (étant donné que pour poser le verrou, la session va aller chercher les données les plus à jour, prenant en compte le commit de la seconde session).

Cette ligne nouvellement apparue malgré les verrous est une "ligne fantôme".

Pour pallier ce problème, qui est contraire au principe d'isolation, **les verrous posés par des requêtes de lecture, de modification et de suppression sont des verrous dits "de clé suivante"** ; ils empêchent l'insertion d'une ligne dans les espaces entre les lignes verrouillées, ainsi que dans l'espace juste après les lignes verrouillées.


[[question]]
| L'espace entre ? L'espace juste après ?

Nous avons vu que les verrous se basent sur les index pour verrouiller uniquement les lignes nécessaires. Voici un petit schéma qui vous expliquera ce qu'est cet "index de clé suivante".

On peut représenter l'index sur _client_id_ de la table _Adoption_ de la manière suivante (je ne mets que les _client_id_ < 10) :
![Représentation de l'index sur client_id](/media/galleries/1255/db3bbdbb-4529-4464-abf0-4ce22d0ced80.png) 

Si l'on insère une adoption avec 4 pour _client_id_, l'index va être réorganisé de la manière suivante :
![Insertion d'une nouvelle valeur dans l'index](/media/galleries/1255/82588263-3e80-4d8d-ab26-87851203b213.png)

Mais, si l'on pose un verrou de clé suivante sur l'index, sur les lignes dont _client_id_ vaut 4, on va alors verrouiller **les lignes, les espaces entre les lignes et les espaces juste après**. Ceci va bloquer l'insertion de la nouvelle ligne :
![Verrou de clé suivante](/media/galleries/1255/b24c25ac-ecf7-4d37-b83f-7141387e21ee.png)

**Démonstration**

On a toujours un verrou exclusif (grâce à notre `SELECT ... FOR UPDATE`) sur les _client_id_ supérieurs à 14 dans la session 1 (sinon, reposez-le).

Session 2 :

```sql
START TRANSACTION;

INSERT INTO Adoption (client_id, animal_id, date_reservation, prix) 
VALUES (15, 61, NOW(), 735.00);
```


L'insertion est bloquée ! Pas de risque de voir apparaître une ligne fantôme.
Annulez les deux transactions.


## Exception

Si la clause `WHERE` concerne un index `UNIQUE` (cela inclut bien sûr les clés primaires) et recherche une seule valeur (exemple : `WHERE id = 4`), alors seule la ligne concernée (si elle existe) est verrouillée, et pas l'espace juste après dans l'index. Forcément, s'il s'agit d'un index `UNIQUE`, l'insertion d'une nouvelle valeur ne changera rien : `WHERE id = 4` ne renverra jamais qu'une seule ligne.


# Pourquoi poser un verrou exclusif avec une requête SELECT ?

Après tout, une requête `SELECT` ne fait jamais que lire des données. Que personne ne puisse les modifier pendant qu'on est en train de les lire, c'est tout à fait compréhensible. Mais pourquoi carrément interdire aux autres de les lire aussi ?

Tout simplement parce que certaines données sont lues dans le but prévisible et avoué de les modifier immédiatement après.

L'exemple typique est la vérification de stock dans un magasin (ou dans un élevage d'animaux).
Un client arrive et veut adopter un chat, on vérifie donc les chats disponibles pour l'adoption, en posant un verrou partagé :

Session 1 :

```sql
START TRANSACTION;

SELECT Animal.id, Animal.nom, Animal.date_naissance, Race.nom as race, COALESCE(Race.prix, Espece.prix) as prix
FROM Animal
INNER JOIN Espece ON Animal.espece_id = Espece.id
LEFT JOIN Race ON Animal.race_id = Race.id             -- Jointure externe, on ne veut pas que les chats de race
WHERE Espece.nom_courant = 'Chat'                      -- Uniquement les chats...
AND Animal.id NOT IN (SELECT animal_id FROM Adoption)  -- ... qui n'ont pas encore été adoptés
LOCK IN SHARE MODE;
```




+--+--------+-------------------+----------+------+
|id|nom     |date_naissance     |race      |prix  |
+==+========+===================+==========+======+
|2 |Roucky  |2010-03-24 02:23:00|NULL      |150.00|
+--+--------+-------------------+----------+------+
|8 |Bagherra|2008-09-11 15:38:00|Maine coon|735.00|
+--+--------+-------------------+----------+------+
|29|Fiero   |2009-05-14 06:30:00|Singapura |985.00|
+--+--------+-------------------+----------+------+
|31|Filou   |2008-02-20 15:45:00|Bleu russe|835.00|
+--+--------+-------------------+----------+------+
|34|Capou   |2008-04-20 03:22:00|Maine coon|735.00|
+--+--------+-------------------+----------+------+
|35|Raccou  |2006-05-19 16:56:00|Bleu russe|835.00|
+--+--------+-------------------+----------+------+
|36|Boucan  |2009-05-14 06:42:00|Singapura |985.00|
+--+--------+-------------------+----------+------+
|37|Callune |2006-05-19 16:06:00|Nebelung  |985.00|
+--+--------+-------------------+----------+------+
|38|Boule   |2009-05-14 06:45:00|Singapura |985.00|
+--+--------+-------------------+----------+------+
|43|Cracotte|2007-03-12 11:54:00|Maine coon|735.00|
+--+--------+-------------------+----------+------+
|44|Cawette |2006-05-19 16:16:00|Nebelung  |985.00|
+--+--------+-------------------+----------+------+
|61|Yoda    |2010-11-09 00:00:00|Maine coon|735.00|
+--+--------+-------------------+----------+------+






[[information]]
| Je rappelle que la fonction `COALESCE()` prend un nombre illimité de paramètres, et renvoie le premier paramètre non `NULL` qu'elle rencontre. Donc ici, s'il s'agit d'un chat de race, _Race.prix_ ne sera pas `NULL` et sera donc renvoyé. Par contre, s'il n'y a pas de race, _Race.prix_ sera `NULL`, mais pas _Espece.prix_, qui sera alors sélectionné.


Si, pendant que le premier client fait son choix, un second client arrive, qui veut adopter un chat Maine Coon, il va également chercher la liste des chats disponibles. Et vu qu'on travaille pour l'instant en verrous partagés, il va pouvoir l'obtenir.

Session 2 :

```sql
START TRANSACTION;

SELECT Animal.id, Animal.nom, Animal.date_naissance, Race.nom as race, COALESCE(Race.prix, Espece.prix) as prix
FROM Animal
INNER JOIN Espece ON Animal.espece_id = Espece.id
INNER JOIN Race ON Animal.race_id = Race.id            -- Jointure interne cette fois
WHERE Race.nom = 'Maine Coon'                          -- Uniquement les Maine Coon...
AND Animal.id NOT IN (SELECT animal_id FROM Adoption)  -- ... qui n'ont pas encore été adoptés
LOCK IN SHARE MODE;
```




+--+--------+-------------------+----------+------+
|id|nom     |date_naissance     |race      |prix  |
+==+========+===================+==========+======+
|8 |Bagherra|2008-09-11 15:38:00|Maine coon|735.00|
+--+--------+-------------------+----------+------+
|34|Capou   |2008-04-20 03:22:00|Maine coon|735.00|
+--+--------+-------------------+----------+------+
|43|Cracotte|2007-03-12 11:54:00|Maine coon|735.00|
+--+--------+-------------------+----------+------+
|61|Yoda    |2010-11-09 00:00:00|Maine coon|735.00|
+--+--------+-------------------+----------+------+





C'est alors que le premier client, M. Dupont, décide de craquer pour Bagherra.


```sql
INSERT INTO Adoption (client_id, animal_id, date_reservation, prix, paye)
SELECT id, 8, NOW(), 735.00, 1
FROM Client
WHERE email = 'jean.dupont@email.com';

COMMIT;
```


Et M. Durant jette également son dévolu sur Bagherra (qui est décidément très très mignon) !


```sql
INSERT INTO Client (nom, prenom, email)
VALUES ('Durant', 'Philippe', 'phidu@email.com');

INSERT INTO Adoption (client_id, animal_id, date_reservation, prix, paye)
VALUES (LAST_INSERT_ID(), 8, NOW(), 735.00, 0);
```


L'insertion dans _Client_ fonctionne mais l'insertion dans _Adoption_ pose problème :

```console
ERROR 1062 (23000): Duplicate entry '8' for key 'ind_uni_animal_id'
```

Et pour cause : Bagherra vient d'être adopté, à l'instant. 
Furieux, M. Durant s'en va, et l'élevage a perdu un client. Il ne reste plus qu'à annuler sa transaction.

C'est pour éviter ce genre de situation qu'il vaut parfois mieux mettre un verrou exclusif sur une sélection. Si l'on sait que cette sélection sert à déterminer quels changements vont être faits, ce n'est pas la peine de laisser quelqu'un d'autre lire des informations qui cesseront d'être justes incessamment sous peu.