Nous avons vu que par défaut : 


- lorsque l'on démarre une transaction, la session prend une photo des tables, et travaille uniquement sur cette photo (donc sur des données potentiellement périmées) tant qu'elle ne pose pas un verrou ;
- les requêtes `SELECT` ne posent pas de verrous si l'on ne le demande pas explicitement ;
- les requêtes `SELECT ... LOCK IN SHARE MODE`, `SELECT ... FOR UPDATE`, `DELETE` et `UPDATE` posent un verrou de clé suivante (sauf dans le cas d'une recherche sur index unique, avec une valeur unique).


Ce comportement est **défini par le niveau d'isolation** des transactions.


# Syntaxe

Pour définir le niveau d'isolation des transactions, on utilise la requête suivante :

```sql
SET [GLOBAL | SESSION] TRANSACTION ISOLATION LEVEL { READ UNCOMMITTED | READ COMMITTED  | REPEATABLE READ  | SERIALIZABLE }
```



- Le mot-clé `GLOBAL` définit le niveau d'isolation pour toutes les sessions MySQL qui seront créées dans le futur. Les sessions existantes ne sont pas affectées.
- `SESSION` définit le niveau d'isolation pour la session courante.
- Si l'on ne précise ni `GLOBAL`, ni `SESSION`, le niveau d'isolation défini ne concernera que la prochaine transaction que l'on ouvrira dans la session courante.



# Les différents niveaux


## REPEATABLE READ

Il s'agit du **niveau par défaut**, celui avec lequel vous travaillez depuis le début. _Repeatable read_ signifie "lecture répétable", c'est-à-dire que si l'on fait plusieurs requêtes de sélection (**non-verrouillantes**) de suite, elles donneront toujours le même résultat, quels que soient les changements effectués par d'autres sessions.
Si l'on pense à bien utiliser les verrous là où c'est nécessaire, c'est un niveau d'isolation tout à fait suffisant.


## READ COMMITTED

Avec ce niveau d'isolation, chaque requête `SELECT` (non-verrouillante) va reprendre une "photo" à jour de la base de données, même si plusieurs `SELECT` se font dans la même transaction. Ainsi, un `SELECT` verra toujours les derniers changements commités, même s'ils ont été faits dans une autre session, après le début de la transaction.


## READ UNCOMMITTED

Le niveau `READ UNCOMMITTED` fonctionne comme `READ COMMITTED`, si ce n'est qu'il autorise la "lecture sale". C'est-à-dire qu'une session sera capable de lire des changements encore non commités par d'autres sessions.

**Exemple**

Session 1 :

```sql
START TRANSACTION;

UPDATE Race 
SET prix = 0 
WHERE id = 7;
```


Session 2 :

```sql
SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED;
START TRANSACTION;

SELECT id, nom, espece_id, prix FROM Race;
```




+--+-------------------+---------+------+
|id|nom                |espece_id|prix  |
+==+===================+=========+======+
|1 |Berger allemand    |1        |485.00|
+--+-------------------+---------+------+
|2 |Berger blanc suisse|1        |935.00|
+--+-------------------+---------+------+
|3 |Singapura          |2        |985.00|
+--+-------------------+---------+------+
|4 |Bleu russe         |2        |835.00|
+--+-------------------+---------+------+
|5 |Maine coon         |2        |735.00|
+--+-------------------+---------+------+
|7 |Sphynx             |2        |0.00  |
+--+-------------------+---------+------+
|8 |Nebelung           |2        |985.00|
+--+-------------------+---------+------+
|9 |Rottweiller        |1        |600.00|
+--+-------------------+---------+------+





La modification faite par la session 1 n'a pas été commitée. Elle ne sera donc potentiellement jamais validée, auquel cas, elle n'affectera jamais les données.
Pourtant, la session 2 voit ce changement de données non-commitées. "Lecture sale" n'a pas une connotation négative par hasard, bien entendu ! Aussi, évitez de travailler avec ce niveau d'isolation.
Annulez la modification de données réalisée par la session 1 et terminez la transaction de la seconde session.


## SERIALIZABLE

Ce niveau d'isolation se comporte comme `REPEATABLE READ`, sauf que lorsque le mode autocommit est désactivé, tous les `SELECT` simples sont implicitement convertis en `SELECT ... LOCK IN SHARE MODE`.