# Définitions


## Variable

Une variable est une **information stockée**, constituée d'un **nom** et d'une **valeur**.
On peut par exemple avoir la variable _age_ (son nom), ayant pour valeur _24_.


## Variable utilisateur

Une variable utilisateur est une variable, **définie par l'utilisateur**. Les variables utilisateur MySQL doivent toujours être **précédées du signe @**.

Une variable utilisateur peut contenir quatre types de valeur :


- un entier (ex. : `24`) ;
- un réel (ex. : `7,8`) ;
- une chaîne de caractères (ex. : `'Hello World !'`) ;
- une chaîne binaire, auquel cas il faut faire précéder la chaîne du caractère b (ex. : `b'011001'`). Nous n'en parlerons pas dans ce cours.



[[attention]]
| Pour les noms de vos variables utilisateur utilisez uniquement des lettres, des chiffres, des ||_||, des ||\$|| et des ||.||. Attention au fait que les noms des variables ne sont pas sensibles à la casse : `@A` est la même variable utilisateur que `@a.`

Il existe également ce qu'on appelle des **variables système**, qui sont des variables prédéfinies par MySQL, et des variables locales, que nous verrons avec les procédures stockées.


# Créer et modifier une variable utilisateur


## SET

La manière la plus classique de créer ou modifier une variable utilisateur est d'utiliser la commande `SET`.


```sql
SET @age = 24;                                  -- Ne pas oublier le @
SET @salut = 'Hello World !', @poids = 7.8;     -- On peut créer plusieurs variables en même temps
```


Si la variable utilisateur existe déjà, sa valeur sera modifiée, sinon, elle sera créée.


```sql
SELECT @age, @poids, @salut;
```




+----+------+-------------+
|@age|@poids|@salut       |
+====+======+=============+
|24  |7.8   |Hello World !|
+----+------+-------------+






## Opérateur d'assignation

Il est également possible d'assigner une valeur à une variable utilisateur directement dans une requête, en utilisant l'opérateur d'assignation **:=** (n'oubliez pas les ||:||, sinon il s'agit de l'opérateur de comparaison de valeurs).


```sql
SELECT @age := 32, @poids := 48.15, @perroquet := 4;
```




+----------+---------------+---------------+
|@age := 32|@poids := 48.15|@perroquet := 4|
+==========+===============+===============+
|32        |48.15          |4              |
+----------+---------------+---------------+






[[information]]
| On peut utiliser l'opérateur d'assignation **:=** dans une commande `SET` également : `SET @chat := 2;`



# Utilisation d'une variable utilisateur


## Ce qu'on peut faire

Une fois votre variable utilisateur créée, vous pouvez bien sûr l'afficher avec `SELECT`. Mais vous pouvez également l'utiliser dans des requêtes ou faire des calculs avec.


```sql
SELECT id, sexe, nom, commentaires, espece_id 
FROM Animal 
WHERE espece_id = @perroquet; -- On sélectionne les perroquets

SET @conversionDollar = 1.31564;       -- On crée une variable contenant le taux de conversion des euros en dollars
SELECT prix AS prix_en_euros,         -- On sélectionne le prix des races, en euros et en dollars.
        ROUND(prix * @conversionDollar, 2) AS prix_en_dollars,   -- En arrondissant à deux décimales
        nom FROM Race;
```



[[information]]
| Si vous utilisez une variable utilisateur qui n'a pas été définie, vous n'obtiendrez aucune erreur. Simplement, la variable aura comme valeur `NULL`.


## Ce qu'on ne peut pas faire

Avec les variables utilisateur, on peut donc dynamiser un peu nos requêtes. Cependant, il n'est pas possible d'utiliser les variables utilisateur pour stocker un nom de table ou de colonne qu'on introduirait ensuite directement dans la requête. Ni pour stocker une partie de commande SQL. **Les variables utilisateur stockent des données**.

**Exemples**


```sql
SET @table_clients = 'Client';

SELECT * FROM @table_clients;
```


```console
ERROR 1064 (42000): You have an error in your SQL syntax; check the manual that corresponds to your MySQL server version for the right syntax to use near '@table_clients' at line 1
```



```sql
SET @colonnes = 'id, nom, description';

SELECT @colonnes FROM Race WHERE espece_id = 1;
```




+--------------------+
|@colonnes           |
+====================+
|id, nom, description|
+--------------------+
|id, nom, description|
+--------------------+
|id, nom, description|
+--------------------+





C'est logique, dans une requête, les noms de tables/colonnes et les commandes SQL ne peuvent pas être représentées comme des chaînes de caractères : on ne les entoure pas de guillemets.


# Portée des variables utilisateurs

Une variable utilisateur n'existe que pour la session dans laquelle elle a été définie. Lorsque vous vous déconnectez, toutes vos variables utilisateurs sont automatiquement réinitialisées. De plus, deux sessions différentes ne partagent pas leurs variables utilisateur.

**Exemple** 

Session 1:

```sql
SET @essai = 3;
```


Session 2:

```sql
SELECT @essai;
```




+------+
|@essai|
+======+
|NULL  |
+------+





De même, si vous assignez une valeur à _@essai_ dans la session 2, _@essai_ vaudra toujours 3 pour la session 1.