Pendant ce cours, nous avons créé une base de données : _elevage_. Vous avez peut-être créé également d'autres bases de données pour vos tests et projets personnels.

Mais ce ne sont pas les seules bases de données existant sur votre serveur MySQL. Connectez-vous avec l'utilisateur "root" (sinon certaines bases de données vous seront cachées) et exécutez la requête suivante :


```sql
SHOW DATABASES;
```




+------------------+
|Database          |
+==================+
|information_schema|
+------------------+
|elevage           |
+------------------+
|mysql             |
+------------------+
|performance_schema|
+------------------+
|test              |
+------------------+





La base _elevage_ est bien là, en compagnie de quelques autres :


- _information_schema_ : cette base de données stocke les informations sur toutes les bases de données. Les tables, les colonnes, le type des colonnes, les procédures des bases de données y sont recensées, avec leurs caractéristiques. Nous verrons cette base de données plus en détail dans le prochain chapitre.
- _performance_schema_ : permet de stocker des informations sur les actions effectuées sur le serveur (temps d'exécution, temps d'attente dus aux verrous, etc.)
- _test_ : il s'agit d'une base de test automatiquement créée. Si vous ne l'avez pas utilisée, elle ne contient rien.
- _mysql_ : qui contient de nombreuses informations sur le serveur. Entre autres, c'est dans cette base que sont stockés **les utilisateurs et leurs privilèges**.



# Les utilisateurs et leurs privilèges


## Privilèges des utilisateurs

Lorsque l'on se connecte à MySQL, on le fait avec un utilisateur. Chaque utilisateur possède une série de privilèges, relatifs aux données stockées sur le serveur : le privilège de sélectionner les données, de les modifier, de créer des objets, etc. 
Ces privilèges peuvent exister à plusieurs niveaux : base de données, tables, colonnes, procédures, etc.

Par exemple, au tout début de ce cours, vous avez créé un utilisateur avec la commande suivante :


```sql
GRANT ALL PRIVILEGES ON elevage.* TO 'sdz'@'localhost' IDENTIFIED BY 'mot_de_passe';
```


Cette requête a créé un utilisateur _'sdz'@'localhost'_, et lui a donné tous les droits sur la base de données _elevage_.


## Stockage des utilisateurs et privilèges

Toutes ces informations sont stockées dans la base de données _mysql_.
Les utilisateurs sont stockés dans la table _user_, avec leurs privilèges globaux (c'est-à-dire valables au niveau du serveur, sur toutes les bases de données). La base _mysql_ possède par ailleurs quatre tables permettant de stocker les privilèges des utilisateurs à différents niveaux.


- _db_ : privilèges au niveau des bases de données.
- _tables_priv_ : privilèges au niveau des tables.
- _columns_priv_ : privilèges au niveau des colonnes.
- _proc_priv_ : privilèges au niveau des routines (procédures et fonctions stockées).



## Modifications

Il est tout à fait possible d'ajouter, modifier et supprimer des utilisateurs en utilisant des requêtes `INSERT`, `UPDATE` ou `DELETE` directement sur la table _mysql.user_. De même pour leurs privilèges, avec les tables _mysql.db_, _mysql.tables_priv_, _mysql.columns_priv_ et _mysql.procs_priv_.

Cependant, en général, on utilise plutôt des commandes dédiées à cet usage. Ainsi, pas besoin de se soucier de la structure de ces tables, et le risque d'erreur est moins grand. Ce sont ces commandes que nous allons décortiquer dans la suite de ce chapitre.


[[information]]
| Tout comme on peut préciser la table à laquelle appartient une colonne en préfixant le nom de la colonne par le nom de la table : _nom_table.nom_colonne_, il est possible de préciser à quelle base de données appartient une table : _nom_bdd.nom_table_, voire même préciser à la fois la table et la base de données dans laquelle se trouve une colonne : _nom_bdd.nom_table.nom_colonne_.