Comme son nom l'indique, la base de données _information_schema_ contient des informations sur les schémas.
En MySQL, un schéma est une base de données. Ce sont des synonymes. La base _information_schema_ contient donc des **informations sur les bases de données**.


[[attention]]
| Cette définition de "schéma" n'est pas universelle, loin s'en faut. Dans certains SGBD la notion de schéma est plus proche de celle d'utilisateur que de base de données. Pour Oracle par exemple, un schéma représente l'ensemble des objets appartenant à un utilisateur.


Voyons ce qu'on trouve comme tables dans cette base de données.


```sql
SHOW TABLES FROM information_schema;
```




+----------------------------+
|Tables_in_information_schema|
+============================+
|CHARACTER_SETS              |
+----------------------------+
|COLUMNS                     |
+----------------------------+
|COLUMN_PRIVILEGES           |
+----------------------------+
|REFERENTIAL_CONSTRAINTS     |
+----------------------------+
|ROUTINES                    |
+----------------------------+
|SESSION_VARIABLES           |
+----------------------------+
|STATISTICS                  |
+----------------------------+
|TABLES                      |
+----------------------------+
|TABLE_CONSTRAINTS           |
+----------------------------+
|TABLE_PRIVILEGES            |
+----------------------------+
|TRIGGERS                    |
+----------------------------+
|USER_PRIVILEGES             |
+----------------------------+
|VIEWS                       |
+----------------------------+





[[information]]
| Le tableau ci-dessus ne reprend qu'une partie des tables d'_information_schema_.


Cette base contient donc des informations sur les tables, les colonnes, les contraintes, les vues, etc., des bases de données stockées sur le serveur MySQL.
En fait, c'est de cette base de données que sont extraites les informations affichées grâce à la commande `SHOW`.

Par conséquent, si les informations données par `SHOW` ne suffisent pas, il est possible d'interroger directement cette base de données.

Prenons par exemple la table _VIEWS_ de cette base de données. Quelles informations contient-elle ?


```sql
SHOW COLUMNS FROM VIEWS FROM information_schema;
```




+--------------------+------------+----+---+-------+-----+
|Field               |Type        |Null|Key|Default|Extra|
+====================+============+====+===+=======+=====+
|TABLE_CATALOG       |varchar(512)|YES |   |NULL   |     |
+--------------------+------------+----+---+-------+-----+
|TABLE_SCHEMA        |varchar(64) |NO  |   |       |     |
+--------------------+------------+----+---+-------+-----+
|TABLE_NAME          |varchar(64) |NO  |   |       |     |
+--------------------+------------+----+---+-------+-----+
|VIEW_DEFINITION     |longtext    |NO  |   |NULL   |     |
+--------------------+------------+----+---+-------+-----+
|CHECK_OPTION        |varchar(8)  |NO  |   |       |     |
+--------------------+------------+----+---+-------+-----+
|IS_UPDATABLE        |varchar(3)  |NO  |   |       |     |
+--------------------+------------+----+---+-------+-----+
|DEFINER             |varchar(77) |NO  |   |       |     |
+--------------------+------------+----+---+-------+-----+
|SECURITY_TYPE       |varchar(7)  |NO  |   |       |     |
+--------------------+------------+----+---+-------+-----+
|CHARACTER_SET_CLIENT|varchar(32) |NO  |   |       |     |
+--------------------+------------+----+---+-------+-----+
|COLLATION_CONNECTION|varchar(32) |NO  |   |       |     |
+--------------------+------------+----+---+-------+-----+





La colonne _TABLE_NAME_ contient le nom de la vue. Interrogeons donc cette table, afin d'avoir des informations sur la vue _V_Animal_details_.


```sql
USE information_schema; -- On sélectionne la base de données

SELECT TABLE_SCHEMA, TABLE_NAME, VIEW_DEFINITION, IS_UPDATABLE, DEFINER, SECURITY_TYPE
FROM VIEWS
WHERE TABLE_NAME = 'V_Animal_details' \G
```



```sql
*************************** 1. row ***************************
   TABLE_SCHEMA: elevage
     TABLE_NAME: V_Animal_details
VIEW_DEFINITION: select `elevage`.`animal`.`id` AS `id`,`elevage`.`animal`.`sexe` AS `sexe`,`elevage`.`animal`.`date_naissance` AS `date_naissance`,
                        `elevage`.`animal`.`nom` AS `nom`,`elevage`.`animal`.`commentaires` AS `commentaires`,
                        `elevage`.`animal`.`espece_id` AS `espece_id`,`elevage`.`animal`.`race_id` AS `race_id`,
                        `elevage`.`animal`.`mere_id` AS `mere_id`,`elevage`.`animal`.`pere_id` AS `pere_id`,
                        `elevage`.`animal`.`disponible` AS `disponible`,`elevage`.`espece`.`nom_courant` AS `espece_nom`,
                        `elevage`.`race`.`nom` AS `race_nom` 
                 from ((`elevage`.`animal` 
                 join `elevage`.`espece` on((`elevage`.`animal`.`espece_id` = `elevage`.`espece`.`id`))) 
                 left join `elevage`.`race` on((`elevage`.`animal`.`race_id` = `elevage`.`race`.`id`)))
   IS_UPDATABLE: YES
        DEFINER: sdz@localhost
  SECURITY_TYPE: DEFINER
```



[[information]]
| La définition de la vue s'affiche en réalité sur une seule ligne, et n'est donc pas indentée. J'ai ajouté l'indentation ici pour que ce soit plus clair.


Voyons encore deux exemples d'exploitation des données d'_information_schema_.

**Exemple 1** : données sur les contraintes de la table _Animal_.


```sql
SELECT CONSTRAINT_SCHEMA, CONSTRAINT_NAME, TABLE_NAME, CONSTRAINT_TYPE
FROM TABLE_CONSTRAINTS
WHERE CONSTRAINT_SCHEMA = 'elevage' AND TABLE_NAME = 'Animal';
```




+-----------------+---------------------+----------+---------------+
|CONSTRAINT_SCHEMA|CONSTRAINT_NAME      |TABLE_NAME|CONSTRAINT_TYPE|
+=================+=====================+==========+===============+
|elevage          |PRIMARY              |Animal    |PRIMARY KEY    |
+-----------------+---------------------+----------+---------------+
|elevage          |ind_uni_nom_espece_id|Animal    |UNIQUE         |
+-----------------+---------------------+----------+---------------+
|elevage          |fk_race_id           |Animal    |FOREIGN KEY    |
+-----------------+---------------------+----------+---------------+
|elevage          |fk_espece_id         |Animal    |FOREIGN KEY    |
+-----------------+---------------------+----------+---------------+
|elevage          |fk_mere_id           |Animal    |FOREIGN KEY    |
+-----------------+---------------------+----------+---------------+
|elevage          |fk_pere_id           |Animal    |FOREIGN KEY    |
+-----------------+---------------------+----------+---------------+





Les contraintes `NOT NULL` ne sont pas reprises dans cette table (on peut les trouver dans la table _COLUMNS_).

**Exemple 2** : données sur la procédure _maj_vm_revenus()_.


```sql
SELECT ROUTINE_NAME, ROUTINE_SCHEMA, ROUTINE_TYPE, ROUTINE_DEFINITION, DEFINER, SECURITY_TYPE 
FROM ROUTINES 
WHERE ROUTINE_NAME = 'maj_vm_revenus' \G
```


Les routines comprennent les procédures stockées et les fonctions stockées (qui ne sont pas couvertes par ce cours).


```sql
*************************** 1. row ***************************
      ROUTINE_NAME: maj_vm_revenus
    ROUTINE_SCHEMA: elevage
      ROUTINE_TYPE: PROCEDURE
ROUTINE_DEFINITION: BEGIN
    TRUNCATE VM_Revenus_annee_espece;

    INSERT INTO VM_Revenus_annee_espece
    SELECT YEAR(date_reservation) AS annee, Espece.id AS espece_id, SUM(Adoption.prix) AS somme, COUNT(Adoption.animal_id) AS nb
    FROM Adoption
    INNER JOIN Animal ON Animal.id = Adoption.animal_id
    INNER JOIN Espece ON Animal.espece_id = Espece.id
    GROUP BY annee, Espece.id;
END
           DEFINER: sdz@localhost
     SECURITY_TYPE: DEFINER
```