Le comportement de MySQL est régi par une série de valeurs contenues dans les **variables système**. Elles déterminent par exemple à quelle valeur commencent les colonnes auto-incrémentées (par défaut : 1), de combien ces valeurs s'incrémentent à chaque insertion (par défaut : 1 également), le moteur de stockage par défaut, le temps d'attente acceptable lorsqu'une requête se heurte à un verrou, etc.

Voici quelques variables système. Vous verrez que vous en connaissez déjà quelques-unes.
Pour une liste complète, reportez-vous à la documentation officielle.




+----------------------+--------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
|Nom                   |Définition                                                                                                                                                                |
+======================+==========================================================================================================================================================================+
|_autocommit_          |Définit si le mode auto-commit est activé ou non, donc si les requêtes sont automatiquement commitées ou s'il est nécessaire de les commiter pour qu'elles prennent effet.|
+----------------------+--------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
|_character_set_client_|Jeu de caractère (encodage) utilisé par le client MySQL.                                                                                                                  |
+----------------------+--------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
|_default_week_format_ |Mode par défaut de la fonction `WEEK()`.                                                                                                                                  |
+----------------------+--------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
|_foreign_key_checks_  |Définit si les contraintes de clé étrangère doivent être vérifiées.                                                                                                       |
+----------------------+--------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
|_ft_min_word_len_     |Taille minimale d'un mot pour qu'il soit inclus dans une recherche `FULLTEXT`.                                                                                            |
+----------------------+--------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
|_last_insert_id_      |Valeur retournée par la fonction `LAST_INSERT_ID()`.                                                                                                                      |
+----------------------+--------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
|_max_connections_     |Nombre maximal de connexions autorisées au serveur.                                                                                                                       |
+----------------------+--------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
|_storage_engine_      |Moteur de stockage par défaut.                                                                                                                                            |
+----------------------+--------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
|_tx_isolation_        |Définit le niveau d'isolation des transactions.                                                                                                                           |
+----------------------+--------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
|_unique_checks_       |Définit si les contraintes d'unicité doivent être vérifiées.                                                                                                              |
+----------------------+--------------------------------------------------------------------------------------------------------------------------------------------------------------------------+





Pour connaître les valeurs actuelles des variables système, on peut utiliser la commande suivante :


```sql
SHOW VARIABLES;
```


Si l'on a une idée de la variable que l'on cherche, on peut utiliser la clause `LIKE`, ou la clause `WHERE`.

**Exemple 1** : variables en rapport avec l'auto-incrémentation.


```sql
SHOW VARIABLES LIKE '%auto_increment%';
```




+------------------------+-----+
|Variable_name           |Value|
+========================+=====+
|auto_increment_increment|1    |
+------------------------+-----+
|auto_increment_offset   |1    |
+------------------------+-----+





**Exemple 2** : affichage de la valeur de _unique_checks_.


```sql
SHOW VARIABLES LIKE 'unique_checks';
```




+-------------+-----+
|Variable_name|Value|
+=============+=====+
|unique_checks|ON   |
+-------------+-----+





On peut utiliser également une requête `SELECT`, en faisant précéder le nom de la variable de deux caractères ||@||.

**Exemple**


```sql
SELECT @@autocommit;
```




+------------+
|@@autocommit|
+============+
|1           |
+------------+






# Niveau des variables système

Les variables système existent à deux niveaux différents : 


- **Global** : c'est la variable au niveau du serveur MySQL même ;
- **Session** : c'est la variable au niveau de la session.


Lorsque l'on démarre le serveur MySQL, les variables système sont donc initialisées à leur valeur par défaut (nous verrons comment modifier ces valeurs plus tard), au niveau global.
Lorsque l'on ouvre une session MySQL et que l'on se connecte au serveur, les variables système au niveau session sont initialisées à partir des variables système globales. Il est cependant possible de modifier les variables système, au niveau de la session ou directement au niveau global. On peut donc se retrouver avec des variables système différentes pour les deux niveaux.

Il est possible de préciser le niveau que l'on désire afficher avec `SHOW VARIABLES`, en ajoutant `GLOBAL` ou `SESSION` (ou `LOCAL`, qui est synonyme de `SESSION`).


```sql
SHOW GLOBAL VARIABLES;
SHOW SESSION VARIABLES;

SELECT @@GLOBAL.nom_variable;
SELECT @@SESSION.nom_variable;
```


Par défaut, si rien n'est précisé, ce sont les variables de session qui sont affichées.


## Variables système n'existant qu'à un niveau
 
Il existe un certain nombre de variables pour lequel le niveau global n'existe pas. Ces variables sont initialisées au début de la connexion, et leur modification ne peut affecter que la session en cours.
La variable système _autocommit_ par exemple n'existe que pour la session. De même que _last_insert_id_, ce qui est logique : `LAST_INSERT_ID()` renvoie la valeur de la dernière auto-incrémentation réalisée par la session. Cela n'a donc pas de sens d'avoir cette variable à un niveau global.

De même, certaines variables système n'ont pas de sens au niveau de la session et n'y existent donc pas. C'est le cas par exemple de la variable _max_connections_, qui détermine le nombre maximum de sessions connectées simultanément au serveur.

Pour ces variables, le comportement de la commande `SHOW VARIABLES` diffère du comportement de la commande `SELECT @@nom_variable`.


- Si vous ne précisez pas le niveau voulu avec `SHOW VARIABLES`, vous aurez la valeur de la variable de session si elle existe, sinon, la valeur au niveau global. `SELECT` agira de la même manière.
- Si vous précisez un niveau, et qu'une des variables n'existe pas à ce niveau, `SHOW` donnera sa valeur à l'autre niveau (sans prévenir).
- Par contre, si vous précisez un niveau avec `SELECT`, et que la variable système n'y existe pas, une erreur est déclenchée.


**Exemples** : _last_insert_id_ n'existe qu'au niveau de la session, _max_connections_ n'existe qu'au niveau global.


```sql
SHOW VARIABLES LIKE 'last_insert_id';
SHOW SESSION VARIABLES LIKE 'last_insert_id';
SHOW GLOBAL VARIABLES LIKE 'last_insert_id';
```


Ces trois requêtes donneront exactement le même résultat :




+--------------+-----+
|Variable_name |Value|
+==============+=====+
|last_insert_id|0    |
+--------------+-----+






```sql
SHOW VARIABLES LIKE 'max_connections';
SHOW SESSION VARIABLES LIKE 'max_connections';
SHOW GLOBAL VARIABLES LIKE 'max_connections';
```





+---------------+-----+
|Variable_name  |Value|
+===============+=====+
|max_connections|151  |
+---------------+-----+





Avec `SELECT`, si l'on ne précise pas le niveau, ou si l'on précise le bon niveau, le résultat est le même.


```sql
SELECT @@max_connections AS max_connections, @@last_insert_id AS last_insert_id;
SELECT @@GLOBAL.max_connections AS max_connections, @@SESSION.last_insert_id AS last_insert_id;
```




+---------------+--------------+
|max_connections|last_insert_id|
+===============+==============+
|151            |0             |
+---------------+--------------+





Par contre, si l'on précise le mauvais niveau, on obtient une erreur.


```sql
SELECT @@SESSION.max_connections;
SELECT @@GLOBAL.last_insert_id;
```


```console
ERROR 1238 (HY000): Variable 'max_connections' is a GLOBAL variable
ERROR 1238 (HY000): Variable 'last_insert_id' is a SESSION variable
```