Comme pour les variables utilisateur et les variables locales, il est possible de modifier la valeur des variables système en utilisant la commande `SET`.

Cependant, toutes ne permettent pas ce genre de modification. La liste des variables système qui l'autorisent, appelées variables système dynamiques, se trouve dans la documentation officielle.

Pour modifier une variable au niveau global, il est nécessaire d'avoir le privilège global `SUPER`. À moins que vous n'ayez accordé ce privilège à l'un de vos utilisateurs, seul l'utilisateur "root" en est donc capable.

Deux syntaxes sont possibles avec `SET` :


```sql
SET niveau nom_variable = valeur;

-- OU

SET @@niveau.nom_variable = valeur;
```

**Exemples**


```sql
SET SESSION max_tmp_tables = 5;         -- Nombre maximal de tables temporaires 
SET @@GLOBAL.storage_engine = InnoDB;   -- Moteur de stockage par défaut
```


On peut aussi omettre le niveau, auquel cas c'est la variable système au niveau de la session qui sera modifiée. Si elle n'existe qu'au niveau global, une erreur sera déclenchée.

**Exemples**


```sql
SET max_tmp_tables = 12;
SET @@max_tmp_tables = 8;

SET @@max_connections = 200;
```


Les deux premières commandes fonctionnent, mais la troisième échoue avec l'erreur suivante :


```console
ERROR 1229 (HY000): Variable 'max_connections' is a GLOBAL variable and should be set with SET GLOBAL
```



# Effet de la modification selon le niveau

Il est important de se rendre compte de la différence qu'il y a entre modifier la valeur d'une variable système au niveau de la session et au niveau du serveur (niveau global).

Lorsque l'on modifie une variable système globale, la valeur de la même variable système au niveau de la session ne change pas.
Par conséquent, cela n'affecte pas du tout la session en cours.

Par contre, cela affectera toutes les sessions qui se connectent au serveur après la modification (jusqu'à arrêt/redémarrage du serveur).

**Exemple** : on a modifié _storage_engine_ au niveau global, cela n'a pas affecté _storage_engine_ au niveau session.


```sql
SELECT @@GLOBAL.storage_engine, @@SESSION.storage_engine;
```




+-----------------------+------------------------+
|@@GLOBAL.storage_engine|@@SESSION.storage_engine|
+=======================+========================+
|InnoDB                 |MyISAM                  |
+-----------------------+------------------------+





Si l'on crée une table dans la session courante, elle utilisera toujours le moteur MyISAM par défaut. Par contre, si l'on ouvre une autre session, les variables système de session étant initialisées à la connexion à partir des variables système du serveur, _storage_engine_ aura la valeur InnoDB pour le serveur et pour la session. Par conséquent, toute table créée par cette nouvelle session utilisera InnoDB par défaut.

À l'inverse, modifier la valeur d'une variable système au niveau de la session n'affectera que la session. Toutes les sessions futures reprendront la valeur globale à l'initialisation des variables système de la session.


# Les commandes SET spéciales

Pour certaines variables système, MySQL a créé des commandes `SET` spéciales.

**Exemples**


```sql
SET NAMES encodage;
```


Cette commande modifie trois variables système, au niveau de la session : _character_set_client_, _character_set_connection_ et _character_set_results_.


```sql
SET [GLOBAL | SESSION] TRANSACTION ISOLATION LEVEL { REPEATABLE READ | READ COMMITTED | READ UNCOMMITTED | SERIALIZABLE }
```


Cette commande modifie la variable système _tx_isolation_ au niveau demandé.