Vous connaissez maintenant deux techniques permettant de modifier des variables système de session : 


- pendant la session avec `SET @@SESSION.nomVariable = valeur` ;
- au démarrage de la session avec les options du client mysql.


Pour les variables système globales, il existe l'équivalent :


- vous savez déjà qu'il est possible de faire `SET @@GLOBAL.nomVariable = valeur` ;
- nous allons maintenant voir comment préciser des options au démarrage **du serveur**.




[[question]]
| Démarrer le serveur ? On n'a jamais démarré le serveur !

Mais si ! Au tout début, lors de l'installation de MySQL, le serveur a été démarré. Selon votre configuration et/ou votre système d'exploitation, il a été démarré soit automatiquement, soit en tapant `mysqld_safe` ou `mysqld` en ligne de commande. Depuis, chaque fois que vous rallumez votre ordinateur, le serveur est démarré automatiquement (c'est en tout cas le comportement par défaut, modifiable à l'installation et/ou par la suite).

Pour stopper le serveur, on utilise l'utilitaire mysqladmin (en ligne de commande bien sûr) :


```console
mysqladmin -u root -p shutdown
```


Bien entendu, si vous n'avez pas défini de mot de passe pour l'utilisateur _root_ (mauvaise idée !), enlevez le `-p`.

Et pour (re)démarrer le serveur :


```console
mysqld
```


Ou, si vous êtes sous MacOS ou Linux, vous pouvez utiliser _mysqld_safe_, qui ajoute quelques options de sécurisation :


```console
mysqld_safe
```




[[information]]
| Vous connaissez donc maintenant quatre logiciels installés lors de la mise en place de MySQL sur votre ordinateur :
| 
| 
| - **_myqsl_**: le logiciel client ;
| - **_mysqld_**: le serveur ;
| - **_mysqladmin_**: qui permet des opérations d'administration ;
| - **_mysqldump_**: qui permet de faire des backups de vos bases de données.


## Les options du serveur

La syntaxe est la même que lors du démarrage du client : `--option[=valeur]`.

**Quelques exemples** :




+---------------------------------+-----------------+-----------------------------------------------------------------------------------+
|Option                           |Raccourci        |Explication                                                                        |
+=================================+=================+===================================================================================+
|_--character-set-server=encodage_|_-C charset_name_|Définit l'encodage par défaut utilisé par le serveur                               |
+---------------------------------+-----------------+-----------------------------------------------------------------------------------+
|_--default-storage-engine=type_  |-                |Définit le moteur de stockage par défaut.                                          |
+---------------------------------+-----------------+-----------------------------------------------------------------------------------+
|_--default-time-zone=timezone_   |-                |Définit le fuseau horaire à utiliser.                                              |
+---------------------------------+-----------------+-----------------------------------------------------------------------------------+
|_--init-file=nomFichier_         |-                |Les requêtes définies dans le dossier donné sont exécutées au démarrage du serveur.|
+---------------------------------+-----------------+-----------------------------------------------------------------------------------+
|_--language=langue_              |_-L langue_      |Définit la langue à utiliser pour les messages d'erreur.                           |
+---------------------------------+-----------------+-----------------------------------------------------------------------------------+
|_--transaction-isolation=niveau_ |-                |Définit le niveau d'isolation des transactions.                                    |
+---------------------------------+-----------------+-----------------------------------------------------------------------------------+




La plupart des options du serveur (mais pas toutes) modifient les variables système globales.

**Exemple** : on lance le serveur en définissant l'encodage utilisé par celui-ci.


```console
mysqld -C greek
```


Il suffit alors de démarrer une session pour tester notre nouvelle configuration.


```sql
SHOW GLOBAL VARIABLES LIKE 'character_set_server';
```




+--------------------+-----+
|Variable_name       |Value|
+====================+=====+
|character_set_server|greek|
+--------------------+-----+